package filter;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;

import utils.Activity;
import utils.ActivityCategory;
import utils.TimeSpan;

public class ActivitiesPerTimeSpan extends Filter
{
	private static String newLine = System.getProperties().getProperty("line.separator");
	public int lengthOfTimeSpan;

	private ArrayList<ActivityCategory> activityCategories;
	private LinkedList<TimeSpan> timeSpans = new LinkedList<TimeSpan>();


	public ActivitiesPerTimeSpan(int timeSpan, ArrayList<ActivityCategory> activityCategories)
	{
		this.lengthOfTimeSpan = timeSpan;
		this.activityCategories = activityCategories;
	}


	public int getTimeSpan()
	{
		return lengthOfTimeSpan;
	}


	public void setTimeSpan(int timeSpan)
	{
		this.lengthOfTimeSpan = timeSpan;
	}


	public LinkedList<TimeSpan> getTimeSpans()
	{
		return timeSpans;
	}


	public ArrayList<ActivityCategory> getActivityCategories()
	{
		return activityCategories;
	}


	public void setActivityCategories(ArrayList<ActivityCategory> activityCategories)
	{
		this.activityCategories = activityCategories;
	}


	@Override
	public void computeInfo(String content)
	{
		ActivityCategory currentActivityCategory = null;

		Activity activity = new Activity(content);

//		long timestamp = Long.parseLong(splitContent[0]);
//		String activity = splitContent[1];

		for (ActivityCategory category : activityCategories)
		{
			if (category.getActivities().contains(activity.getDescription()))
			{
				currentActivityCategory = category;
				break;
			}
		}

		if (currentActivityCategory == null)
			return;

		if (timeSpans.size() > 0)
		{
			TimeSpan lastTimeSpan = timeSpans.getLast();

            while (activity.getTimestamp() > lastTimeSpan.getEndTime()) {
                TimeSpan timeSpan = new TimeSpan(activityCategories);
                timeSpan.setStartTime(lastTimeSpan.getEndTime() + 1);
                timeSpan.setEndTime(timeSpan.getStartTime() + lengthOfTimeSpan);
                lastTimeSpan = timeSpan;
                timeSpans.add(timeSpan);
            }
            lastTimeSpan.count(currentActivityCategory);
            lastTimeSpan.addActivity(activity);


//			if (activity.getTimestamp() < lastTimeSpan.getEndTime())
//			{
//				lastTimeSpan.count(currentActivityCategory);
//				lastTimeSpan.addActivity(activity);
//			}
//			else
//			{
//				TimeSpan timeSpan = new TimeSpan(activityCategories);
//
//				timeSpan.setStartTime(lastTimeSpan.getEndTime() + 1);
//				timeSpan.setEndTime(timeSpan.getStartTime() + lengthOfTimeSpan);
//				timeSpan.count(currentActivityCategory);
//				timeSpan.addActivity(activity);
//
//				timeSpans.add(timeSpan);
//			}
		}
		else
		{
			TimeSpan timeSpan = new TimeSpan(activityCategories);

			timeSpan.setStartTime(activity.getTimestamp());
			timeSpan.setEndTime(activity.getTimestamp() + lengthOfTimeSpan);

			timeSpan.count(currentActivityCategory);
			timeSpan.addActivity(activity);

			timeSpans.add(timeSpan);
		}
	}


	@Override
	public String getInfo()
	{
		info += "Number of activities per time span".toUpperCase() + newLine;

		info += "Total number of activities: " + TimeSpan.totalNumOfActivities + newLine;

		info += "Highest count (global): " + TimeSpan.highestCount + newLine;

		info += "--- Time spans ---" + newLine;

		for (TimeSpan timeSpan : timeSpans)
		{
			info += timeSpan.toString() + newLine;
		}

		return info;
	}


	public boolean exportData(ArrayList<ActivityCategory> activityCategories, String filePath)
	{
		BufferedWriter br;
		
		try
		{
			br = new BufferedWriter(new FileWriter(filePath));
			
			String line = "";
			int counter = 0;
			
			for (ActivityCategory activityCategory : activityCategories)
			{
				line += activityCategory.getId();
				
				if (counter < activityCategories.size()-1)
				{
					line += ",";
				}
				
				counter++;
			}
			
			br.write(line + newLine);
			
			for (TimeSpan timeSpan : timeSpans)
			{
				line = "";
				counter = 0;
				
				for (ActivityCategory activityCategory : activityCategories)
				{
					line += timeSpan.getCounters().get(activityCategory);
					
					if (counter < activityCategories.size()-1)
					{
						line += ",";
					}
					
					counter++;
				}
				
				br.write(line + newLine);
			}
			
			br.close();
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return true;
	}
}
