package filter;

import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import utils.Activity;
import utils.ActivityCategory;

public class ActivityCategoryPatternFinder extends Filter
{
	private static String newLine = System.getProperties().getProperty("line.separator");
	public static int highestCount = 1;

	private List<ActivityCategory> activityCategories;
	private String[] slidingWindow;
	private HashMap<String, Integer> patternCounter;
	private int nextIndex;
	private int filter;

	private boolean includeHierarchyInfo = true;
	private boolean ignoreSameActivities = true;


	public ActivityCategoryPatternFinder(int windowSize, List<ActivityCategory> activityCategories, int filter)
	{
		this.activityCategories = activityCategories;
		slidingWindow = new String[windowSize];
		patternCounter = new HashMap<String, Integer>();
		nextIndex = 0;

		this.filter = filter;
	}


	public int getWindowSize()
	{
		return slidingWindow.length;
	}


	public void countPattern(String fingerprint)
	{
		if (patternCounter.containsKey(fingerprint))
		{
			Integer newValue = patternCounter.get(fingerprint) + 1;
			patternCounter.put(fingerprint, newValue);

			if (highestCount < newValue)
			{
				highestCount = newValue;
			}
		}
		else
		{
			patternCounter.put(fingerprint, 1);
		}
	}


	public String findCategory(String activity)
	{
		for (ActivityCategory category : activityCategories)
		{
			if (category.getActivities().contains(activity))
			{
				return category.getId();
			}
		}

		return null;
	}


	@Override
	public void computeInfo(String content)
	{
		String activityDescription = null;

		Activity currentActivity = new Activity(content);

		activityDescription = findCategory(currentActivity.getDescription());

		if (activityDescription == null)
		{
			return;
		}

		if (ignoreSameActivities && nextIndex > 0 && activityDescription.equals(slidingWindow[nextIndex - 1]))
		{
			return;
		}

		slidingWindow[nextIndex++] = activityDescription;

		if (nextIndex >= slidingWindow.length)
		{
			String fingerprint = "|";

			for (int i = 0; i < slidingWindow.length; i++)
			{
				if (includeHierarchyInfo)
				{
					fingerprint += slidingWindow[i] + " H" + currentActivity.getFields().get(2) + "|";
				}
				else
				{
					fingerprint += slidingWindow[i] + "|";
				}

				if (i + 1 < slidingWindow.length)
				{
					slidingWindow[i] = slidingWindow[i + 1];
				}
			}

			countPattern(fingerprint);

			nextIndex = slidingWindow.length - 1;
		}
	}


	@Override
	public String getInfo()
	{
		info += "Analysis of activity category patterns".toUpperCase() + newLine;

		info += "Sliding window size: " + slidingWindow.length + newLine;

		info += "Filter numbers lower than: " + filter + newLine;

		info += "--- Activity patterns ---" + newLine;

		for (Entry<String, Integer> entry : patternCounter.entrySet())
		{
			if (entry.getValue() >= filter)
			{
				info += entry.getKey() + ": " + entry.getValue() + newLine;
			}
		}

		return info;
	}
}
