package filter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map.Entry;

import main.LFAProperties;
import utils.ActivitiesGroup;
import utils.Activity;
import utils.ActivityCategory;
import utils.Counter;

import static main.LFAProperties.*;

public class GroupedActivities extends Filter
{
	private LinkedList<ActivitiesGroup> groups = new LinkedList<ActivitiesGroup>();
	private ArrayList<ActivityCategory> activityCategories;
	private HashMap<String, Counter> totalAmountOfActivities = new HashMap<String, Counter>();


	public GroupedActivities(ArrayList<ActivityCategory> activityCategories)
	{
		this.activityCategories = activityCategories;

		for (ActivityCategory activityCategory : activityCategories)
		{
			totalAmountOfActivities.put(activityCategory.getId(), new Counter());
		}

		totalAmountOfActivities.put(LFAProperties.MISC_GROUP.getId(), new Counter());
		totalAmountOfActivities.put(LFAProperties.IDLE_GROUP.getId(), new Counter());
	}


	public LinkedList<ActivitiesGroup> getGroups()
	{
		return groups;
	}


	@Override
	public void computeInfo(String content)
	{
		ActivityCategory currentCategory = LFAProperties.MISC_GROUP;

		Activity activity = new Activity(content);

		if (!content.contains("EOF"))
		{
			for (ActivityCategory category : activityCategories)
			{
				if (category.containsActivity(activity.getDescription()))
				{
					currentCategory = category;
					break;
				}
			}

			if (groups.size() > 0)
			{
				ActivitiesGroup lastGroup = groups.getLast();

				if (lastGroup.getRelatedCategory().getId().equals(currentCategory.getId())) // ||
																							// lastGroup.getRelatedCategory().getId().equals(MISC_GROUP))
				{
					lastGroup.addActivity(activity);
					lastGroup.setRelatedCategory(currentCategory);
					// totalAmountOfActivities.get(currentCategory).inc();
					lastGroup.setEndTime(activity.getTimestamp());
				}
				else
				{
					ActivitiesGroup group = new ActivitiesGroup();

					switch (gaMode)
					{
						case FAIR:
							long timePeriod = (activity.getTimestamp() - lastGroup.getEndTime()) / 2;
							lastGroup.setEndTime(lastGroup.getEndTime() + timePeriod);
							lastGroup.setTimePeriod(lastGroup.getEndTime() - lastGroup.getStartTime());

							group.setTimePeriod(timePeriod);
							group.setStartTime(activity.getTimestamp() - timePeriod);
							group.setEndTime(activity.getTimestamp());

							break;
							
						case IDLE:
							ActivitiesGroup idleGroup = new ActivitiesGroup();

							lastGroup.setTimePeriod(lastGroup.getEndTime() - lastGroup.getStartTime());

							idleGroup.setRelatedCategory(LFAProperties.IDLE_GROUP);
							idleGroup.setStartTime(lastGroup.getEndTime());
							idleGroup.setEndTime(activity.getTimestamp());
							idleGroup.setTimePeriod(activity.getTimestamp() - lastGroup.getEndTime());

							groups.add(idleGroup);

							group.setTimePeriod(0);
							group.setStartTime(activity.getTimestamp());
							group.setEndTime(activity.getTimestamp());

							break;
					}

					group.setRelatedCategory(currentCategory);
					group.addActivity(activity);
					// totalAmountOfActivities.get(currentCategory.getId()).inc();

					groups.add(group);
				}
			}
			else
			{
				ActivitiesGroup group = new ActivitiesGroup();

				group.addActivity(activity);
				// totalAmountOfActivities.get(currentCategory).inc();
				group.setRelatedCategory(currentCategory);
				group.setTimePeriod(activity.getTimestamp());
				group.setStartTime(activity.getTimestamp());
				group.setEndTime(activity.getTimestamp());

				groups.add(group);
			}
		}
		else
		{
			ActivitiesGroup lastGroup = groups.getLast();
			lastGroup.setEndTime(activity.getTimestamp());
			lastGroup.setTimePeriod(lastGroup.getEndTime() - lastGroup.getStartTime());
		}
	}


	@Override
	public String getInfo()
	{

		info += "GroupedActivities:" + newLine;

		// print actions performed by every user and by keyboard
		for (ActivitiesGroup group : groups)
		{
			// group.setVerbose(false);
			info += group.toString() + newLine;
		}

		info += "Total amount of activities by groups:" + newLine;

		for (Entry<String, Counter> entry : totalAmountOfActivities.entrySet())
		{
			info += "GID " + entry.getKey() + ": " + entry.getValue() + newLine;
		}

		return info;
	}
}
