package main;

import filter.ActivitiesPerTimeSpan;
import filter.GroupedActivities;
import filter.Parser;
import filter.Timer;
import utils.ActivityCategory;
import visualization.GroupedActivitiesVisualization;
import visualization.TimeSpansVisualization;
import visualization.VisualizationFrame;

import java.awt.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

public class Main {
    /**
     * @param args args[0] : Name of the input logfile args[1] : Optional basedir
     */
    public static void main(String[] args) {

        int categoryPartition = 3;
        int visualizationMode = 1;
        boolean createCategoriesFromFile = true;
        boolean ignoreMiscCategory = true;

        if (args.length >= 2) {
            String inputFilename = args[0];
            String outputFilename = inputFilename.substring(0, inputFilename.indexOf(LFAProperties.FILE_EXTENSION)) + "_analysis" + LFAProperties.FILE_EXTENSION;
            String basedir = args[1];
            ArrayList<String> activities = new ArrayList<String>();

            boolean showVisualization = args.length == 3 ? Boolean.parseBoolean(args[2]) : false;

            Parser parser = new Parser(basedir);

            ArrayList<ActivityCategory> activityCategoriesList;
            HashMap<String, ActivityCategory> activityCategories;
            ActivityCategory tempCategory;
            Color[] colors = new Color[]{new Color(164, 108, 191), new Color(155, 187, 89), new Color(118, 164, 218), new Color(253, 202, 1), new Color(195, 90, 88)};

            if (createCategoriesFromFile) {
                activityCategoriesList = new ArrayList<ActivityCategory>();
                activityCategories = new HashMap<String, ActivityCategory>();

                BufferedReader reader;
                String content;
                int currentColor = 0;

                try {
                    reader = new BufferedReader(new FileReader(basedir + "all_logs.txt"));

                    try {
                        while ((content = reader.readLine()) != null) {

                            String[] columns = content.split(LFAProperties.SEPARATOR);

                            activities.add(columns[0]);

                            String activityCategoryName = columns[1];//.split("_")[0];

                            int count = 0;
                            for (ActivityCategory activityCategory : activityCategoriesList) {
                                if (activityCategory.getId().equals(activityCategoryName)) {
                                    activityCategory.addActivity(columns[0]);
                                    break;
                                } else {
                                    count++;
                                }
                            }

                            if (count >= activityCategoriesList.size()) {

                                if (ignoreMiscCategory && activityCategoryName.equalsIgnoreCase("misc"))   {
                                    continue;
                                }

                                tempCategory = new ActivityCategory(activityCategoryName, colors[currentColor++], activityCategoriesList.size()); // Color.PURPLE
                                tempCategory.addActivity(columns[0]);
                                activityCategoriesList.add(tempCategory);
                                activityCategories.put(activityCategoryName, tempCategory);

                                if (currentColor >= colors.length) {
                                    currentColor = 0;
                                }
                            }
                        }
                    } finally {
                        reader.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }

                ActivityCategory activityCategory;



                activityCategory = activityCategories.get("PREVIEW_TOOL");
                //activityCategory.setId("execution");
                activityCategory.setId("DP");
                activityCategory.setOrder(1);
                activityCategory.setColor(new Color(195, 90, 88));

                activityCategory = activityCategories.get("PREVIEW_POPUP");
                //activityCategory.setId("analyze");
                activityCategory.setId("PP");
                activityCategory.setOrder(2);
                activityCategory.setColor(new Color(118, 164, 218));

//                activityCategory = activityCategories.get("WEBVIEW");
//                //activityCategory.setId("comparison");
//                activityCategory.setId("WEBVIEW");
//                activityCategory.setOrder(2);
//                activityCategory.setColor(new Color(155, 187, 89));

                activityCategory = activityCategories.get("BROWSE");
                //activityCategory.setId("graph");
                activityCategory.setId("B");
                activityCategory.setOrder(3);
                activityCategory.setColor(new Color(253, 202, 1));

                activityCategory = activityCategories.get("ANNOTATION");
                //activityCategory.setId("navigation");
                activityCategory.setId("A");
                activityCategory.setOrder(4);
                activityCategory.setColor(new Color(0xC4C4C4));

//                activityCategory = activityCategories.get("CREATE");
//                //activityCategory.setId("navigation");
//                activityCategory.setId("CREATE");
//                activityCategory.setOrder(5);
//                activityCategory.setColor(new Color(0xC04BC4));
            } else {
                switch (categoryPartition) {
                    case 1:
                        activityCategoriesList = new ArrayList<ActivityCategory>();

                        // tempCategory = new ActivityCategory("misc", new Color(164, 108, 191), 4); //
                        // Color.PURPLE
                        // tempCategory
                        // .addActivities(new String[] { "Undo last Action", "Open File", "Open Dir",
                        // "Sort Nodes" });
                        // activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("open", new Color(164, 108, 191), 4); // Color.PURPLE
                        tempCategory.addActivities(new String[]{"Open File", "Open Dir"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("zoom", new Color(155, 187, 89), 3); // Color.GREEN
                        tempCategory
                                .addActivities(new String[]{"Zoom In", "Zoom Out", "Zoom to FileSize", "Zoom Reset"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("mark", new Color(118, 164, 218), 2); // Color.BLUE
                        tempCategory.addActivities(new String[]{"Mark Node", "Unmark Node", "Unmark All"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("add/remove", new Color(253, 202, 1), 1); // Color.ORANGE
                        tempCategory.addActivities(new String[]{"Add File", "Add Dir", "Rename Node", "Delete Node/s"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("manipulate", new Color(195, 90, 88), 0); // Color.RED
                        tempCategory.addActivities(new String[]{"Merge Nodes", "Move Node/s"});
                        activityCategoriesList.add(tempCategory);

                        break;

                    case 2:
                        activityCategoriesList = new ArrayList<ActivityCategory>();

                        tempCategory = new ActivityCategory("undo", new Color(164, 108, 191), 4); // Color.PURPLE
                        tempCategory.addActivities(new String[]{"Undo last Action"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("open", new Color(253, 202, 1), 3); // Color.ORANGE
                        tempCategory.addActivities(new String[]{"Open File", "Open Dir"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("mark", new Color(118, 164, 218), 2); // Color.BLUE
                        tempCategory.addActivities(new String[]{"Mark Node", "Unmark Node", "Unmark All"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("layout", new Color(155, 187, 89), 1); // Color.GREEN
                        tempCategory.addActivities(new String[]{"Sort Nodes", "Zoom In", "Zoom Out", "Zoom to FileSize",
                                "Zoom Reset"});
                        activityCategoriesList.add(tempCategory);

                        tempCategory = new ActivityCategory("modify", new Color(195, 90, 88), 0); // Color.RED
                        tempCategory.addActivities(new String[]{"Add File", "Add Dir", "Rename Node", "Delete Node/s",
                                "Merge Nodes", "Move Node/s"});
                        activityCategoriesList.add(tempCategory);

                        break;
                    default:
                        Collections.sort(activities);
                        activityCategoriesList = ActivityCategory.createSingleCategories(activities.toArray(new String[0]));
                        break;
                }
            }

            ActivitiesPerTimeSpan activitiesPerTimeSpan = new ActivitiesPerTimeSpan(LFAProperties.TS_TIME_SPAN_LENGTH, activityCategoriesList);
            // ActivityCategoryPatternFinder slidingWindow = new ActivityCategoryPatternFinder(4,
            // activityCategoriesList, 4);
            //ActivityPatternFinder activityPatternFinder = new ActivityPatternFinder(4, activities.toArray(new String[0]), 4);

            GroupedActivities groupedActivities = new GroupedActivities(activityCategoriesList);

            parser.addFilter(new Timer());
            // parser.addFilter(new CREWAActionCounter());
            // parser.addFilter(new CREWAControlCounter());
            // parser.addFilter(new CREWACheckActions());

            parser.addFilter(activitiesPerTimeSpan);
            //parser.addFilter(activityPatternFinder);
            parser.addFilter(groupedActivities);

            // parse input file and compute filters
            try {
                parser.parse(inputFilename);
            } catch (IOException e) {
                e.printStackTrace();
            }

            // print results to outputfile
            try {
                parser.print(outputFilename);
            } catch (IOException e) {
                e.printStackTrace();
            }

            activitiesPerTimeSpan.exportData(activityCategoriesList,
                    basedir + inputFilename.substring(0, inputFilename.indexOf(LFAProperties.FILE_EXTENSION)) + "_activities_"
                            + (LFAProperties.TS_TIME_SPAN_LENGTH / 1000) + "sec" + LFAProperties.FILE_EXTENSION
            );

            if (showVisualization) {
                VisualizationFrame visualizationFrame = new VisualizationFrame();

                if (visualizationMode == 1) {
                    TimeSpansVisualization timeSpansVis = new TimeSpansVisualization(activitiesPerTimeSpan.getTimeSpans(),
                            basedir + inputFilename.substring(0, inputFilename.indexOf(LFAProperties.FILE_EXTENSION)) + ".png");
                    timeSpansVis.setHighestCount(58);
                    visualizationFrame.add(timeSpansVis, BorderLayout.CENTER);

                    switch (categoryPartition) {
                        case 1:
                        case 2:
                            break;
                        default:
                            timeSpansVis.setLayoutProperties(1920, 306, 10, 40);
                    }
                } else if (visualizationMode == 0) {
                    GroupedActivitiesVisualization groupedActivitiesVis = new GroupedActivitiesVisualization(
                            groupedActivities.getGroups(), activityCategoriesList, basedir
                            + inputFilename.substring(0, inputFilename.indexOf(LFAProperties.FILE_EXTENSION)) + ".png"
                    );
                    visualizationFrame.add(groupedActivitiesVis, BorderLayout.CENTER);
                }

                visualizationFrame.setTitle(inputFilename);
                visualizationFrame.setVisible(true);
            }
        } else {
            System.err.print("Please set correct inputfilename!");
        }
    }
}
