package utils;

import java.util.LinkedList;

public class ActivitiesGroup
{
	private static String newLine = System.getProperties().getProperty("line.separator");
	public static float maxActivitiesPerMinute = 0;
	public static int maxNumOfActivities = 0;

	private LinkedList<Activity> activities;
	private long startTime;
	private long endTime;
	private long timePeriod;
	private boolean verbose;
	
	private ActivityCategory relatedCategory;


	public ActivitiesGroup()
	{
		activities = new LinkedList<Activity>();
		startTime = 0;
		verbose = true;
	}


	public ActivitiesGroup(ActivitiesGroup groupToCopy)
	{
		this.activities = new LinkedList<Activity>();
		this.startTime = groupToCopy.startTime;
		this.endTime = groupToCopy.endTime;
		this.timePeriod = groupToCopy.timePeriod;
		this.verbose = groupToCopy.verbose;

		for (Activity activity : groupToCopy.activities)
		{
			this.activities.add(new Activity(activity));
		}

		this.relatedCategory = groupToCopy.relatedCategory;
	}


	public boolean isVerbose()
	{
		return verbose;
	}


	public void setVerbose(boolean verbose)
	{
		this.verbose = verbose;
	}


	public long getEndTime()
	{
		return endTime;
	}


	public void setEndTime(long endTime)
	{
		this.endTime = endTime;
	}


	public long getTimePeriod()
	{
		return timePeriod;
	}


	public void setTimePeriod(long timePeriod)
	{
		this.timePeriod = timePeriod;
	}


	public int getNumOfActivities()
	{
		return activities.size();
	}


	public ActivityCategory getRelatedCategory()
	{
		return relatedCategory;
	}


	public void setRelatedCategory(ActivityCategory relatedCategory)
	{
		this.relatedCategory = relatedCategory;
	}


	public LinkedList<Activity> getActivities()
	{
		return activities;
	}


	public void setActivities(LinkedList<Activity> activities)
	{
		this.activities = activities;
	}


	public long getStartTime()
	{
		return startTime;
	}


	public void setStartTime(long startTime)
	{
		this.startTime = startTime;
	}
	
	
	public void addActivity(Activity activity)
	{
		this.activities.add(activity);
	}


	public void addGroup(ActivitiesGroup groupToAdd, boolean recalcTimePeriod)
	{
		activities.addAll(groupToAdd.getActivities());
		endTime = groupToAdd.getEndTime();

		if (recalcTimePeriod)
		{
			timePeriod = endTime - startTime;
		}
		else
		{
			timePeriod += groupToAdd.getTimePeriod();
		}
	}


	public float getActivitiesPerMinute()
	{
		return (float) activities.size() / ((float) timePeriod / 1000.0f / 60.0f);
	}


	public String toString()
	{
		String str = "";

		if (verbose)
		{
			str += "GID: " + relatedCategory.getId() + newLine;
			str += "TimePeriod: " + timePeriod + newLine;
			str += "NumOfActivities: " + getNumOfActivities() + newLine;
			str += "ActivitiesPerMinute: " + getActivitiesPerMinute() + newLine;

			for (Activity activity : activities)
			{
				str += activity + newLine;
			}
		}
		else
		{
			str += "[" + relatedCategory.getId() + "," + timePeriod / 1000.0 + "]";
		}

		return str;
	}

}
