package utils;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;

public class ActivityCategory implements Comparable<ActivityCategory>
{
	private String id;
	private Color color;
	private HashSet<String> activities;
	private int order;


	public ActivityCategory(String id, Color color, int order)
	{
		super();
		this.id = id;
		this.color = color;
		this.order = order;
		this.activities = new HashSet<String>();
	}


	public String getId()
	{
		return id;
	}


	public void setId(String id)
	{
		this.id = id;
	}


	public Color getColor()
	{
		return color;
	}


	public void setColor(Color color)
	{
		this.color = color;
	}


	public int getOrder()
	{
		return order;
	}


	public void setOrder(int order)
	{
		this.order = order;
	}


	public HashSet<String> getActivities()
	{
		return activities;
	}


	public void setActivities(HashSet<String> activities)
	{
		this.activities = activities;
	}

    public void addActivity(String activity)
    {
        this.activities.add(activity);
    }

	public void addActivities(String[] activities)
	{
		this.activities.addAll(Arrays.asList(activities));
	}

    public void addActivities(HashSet<String> activities)
    {
        this.activities.addAll(activities);
    }
	
	public boolean containsActivity(String activityName)
	{
		return this.activities.contains(activityName);
	}


	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((color == null) ? 0 : color.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}


	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ActivityCategory other = (ActivityCategory) obj;
		if (color == null)
		{
			if (other.color != null)
				return false;
		}
		else if (!color.equals(other.color))
			return false;
		if (id == null)
		{
			if (other.id != null)
				return false;
		}
		else if (!id.equals(other.id))
			return false;
		return true;
	}


	@Override
	public String toString()
	{
		return id;
	}


	public static ArrayList<ActivityCategory> createSingleCategories(String[] activities)
	{
		ActivityCategory tempCategory;
		ArrayList<ActivityCategory> activityCategories = new ArrayList<ActivityCategory>();

		for (int i = 0; i < activities.length; i++)
		{
			String activity = activities[i];

			tempCategory = new ActivityCategory(activity, new Color((int) (Math.random() * 255),
					(int) (Math.random() * 255), (int) (Math.random() * 255)), i);
			tempCategory.addActivities(new String[] { activity });
			activityCategories.add(tempCategory);
		}

		return activityCategories;
	}


	@Override
	public int compareTo(ActivityCategory activityCategory)
	{
		if (this.order < activityCategory.getOrder())
				return -1;
		
		if (this.order > activityCategory.getOrder())
				return 1;
		
		return 0;
	}

    public static void mergeCategories(ActivityCategory activityCategory1, ActivityCategory activityCategory2) {
        activityCategory1.addActivities(activityCategory2.getActivities());
    }
}
