package utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;

public class LogFileMerger
{
	private LogFile mainFile;
	private ArrayList<LogFile> userFiles;

	private LinkedList<Activity> activities;
	
	public static int MAX_NUM_LOGFILES = 10;


	public LogFileMerger(final LogFile mainFile, final ArrayList<LogFile> userFiles)
	{
		this.mainFile = mainFile;
		this.userFiles = userFiles;

		this.activities = new LinkedList<Activity>();
	}


	public LogFile getMainFile()
	{
		return mainFile;
	}


	public void setMainFile(LogFile mainFile)
	{
		this.mainFile = mainFile;
	}


	public ArrayList<LogFile> getUserFiles()
	{
		return userFiles;
	}


	public void setUserFiles(ArrayList<LogFile> userFiles)
	{
		this.userFiles = userFiles;
	}


	public LinkedList<Activity> getActivities()
	{
		return activities;
	}


	public void collectLogMessages()
	{
		try
		{
			String line;
			
			BufferedReader mainFileReader = new BufferedReader(new FileReader(mainFile.getFile()));
			
			while ((line = mainFileReader.readLine()) != null)
			{
				if (line.startsWith("# timeshift"))
				{
					mainFile.setTimeshift(Integer.parseInt(line.substring(line.indexOf("=")+1).trim()));
				}
				else if (!line.endsWith("EOF") && !line.startsWith("#") && !line.trim().isEmpty())
				{
					Activity activity = new Activity(line);
					activity.shiftTimestamp(mainFile.getTimeshift());
					activities.add(new Activity(line));
				}

			}

			mainFileReader.close();
			
			for (LogFile userFile : userFiles)
			{
				BufferedReader userFileReader = new BufferedReader(new FileReader(userFile.getFile()));
				
				while ((line = userFileReader.readLine()) != null)
				{
					if (line.startsWith("# timeshift"))
					{
						userFile.setTimeshift(Integer.parseInt(line.substring(line.indexOf("=")+1).trim()));
					}
					else if (!line.endsWith("EOF") && !line.startsWith("#") && !line.trim().isEmpty())
					{
						Activity activity = new Activity(line);
						activity.shiftTimestamp(userFile.getTimeshift());
						activities.add(new Activity(line));
					}
				}

				userFileReader.close();
			}
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}


	public void sortLogMessages()
	{
		Collections.sort(activities);
	}


	public void saveLogMessages(String filePath)
	{
		try
		{
			BufferedWriter mergedLogFileWriter = new BufferedWriter(new FileWriter(filePath));

			for (Activity activity : activities)
			{
				mergedLogFileWriter.write(activity.getOriginalMessage());
				mergedLogFileWriter.newLine();
			}

			//mergedLogFileWriter.write(LogMessage.getEOFMessage(activities.getLast().getTimestamp()) + " EOF");

			mergedLogFileWriter.flush();
			mergedLogFileWriter.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}


	public static void main(String[] args)
	{
		if (args.length == 2)
		{
			int expNo = Integer.parseInt(args[0]);
			String logDir = args[1];
			
			LogFile mainFilePath = new LogFile(new File(logDir + "crewa" + expNo + ".log"));
	
			ArrayList<LogFile> userFiles = new ArrayList<LogFile>();
			
			for (int i=0; i < LogFileMerger.MAX_NUM_LOGFILES; i++)
			{
				File file = new File(logDir + "user" + expNo + "-" + i + ".log");
				
				if (file.exists())
				{
					userFiles.add(new LogFile(file));
				}		
			}
	
			String outputFilePath = logDir + "group" + expNo + ".log";
	
			LogFileMerger lfm = new LogFileMerger(mainFilePath, userFiles);
			
			lfm.collectLogMessages();
			System.out.println("Log messages collected");
			
			lfm.sortLogMessages();
			System.out.println("Log messages sorted");
			
			lfm.saveLogMessages(outputFilePath);
			System.out.println("Log messages saved to " + outputFilePath);
		}
	}

}
