package utils;

import java.util.LinkedList;
import java.util.List;

public class Pattern
{
	public static int highestCount = 1;

	private String fingerprint;
	private LinkedList<Activity[]> activityPatterns;

	private boolean includeHierarchyInfo = false;


	public Pattern()
	{
		this.activityPatterns = new LinkedList<Activity[]>();
	}


	public Pattern(Activity[] activitySequenz)
	{
		this.activityPatterns = new LinkedList<Activity[]>();
		this.activityPatterns.add(activitySequenz);
		this.fingerprint = computeFingerprint(activitySequenz, this.includeHierarchyInfo);
	}


	public Pattern(Activity[] activitySequenz, boolean includeHierarchyInfo)
	{
		this.activityPatterns = new LinkedList<Activity[]>();
		this.activityPatterns.add(activitySequenz);
		this.includeHierarchyInfo = includeHierarchyInfo;
		this.fingerprint = computeFingerprint(activitySequenz, this.includeHierarchyInfo);
	}


	public static String computeFingerprint(Activity[] activities, boolean includeHierarchyInfo)
	{
		String fingerprint = "|";

		for (Activity activity : activities)
		{
			fingerprint += activity.getDescription() + "|";

			if (includeHierarchyInfo)
			{
				fingerprint += "h" + activity.getFields().get(2) + "|";
			}
		}

		return fingerprint;
	}


	public String getFingerprint()
	{
		return fingerprint;
	}


	public void setFingerprint(String fingerprint)
	{
		this.fingerprint = fingerprint;
	}


	public List<Activity[]> getActivityPatterns()
	{
		return activityPatterns;
	}


	public void addActivitySequenz(Activity[] activitySequenz)
	{
		this.activityPatterns.add(activitySequenz);

		if (highestCount < this.activityPatterns.size())
		{
			highestCount = this.activityPatterns.size();
		}
	}


	public int getNumberOfPatterns()
	{
		return activityPatterns.size();
	}


	public boolean isIncludeHierarchyInfo()
	{
		return includeHierarchyInfo;
	}


	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((fingerprint == null) ? 0 : fingerprint.hashCode());
		return result;
	}


	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Pattern other = (Pattern) obj;
		if (fingerprint == null)
		{
			if (other.fingerprint != null)
				return false;
		}
		else if (!fingerprint.equals(other.fingerprint))
			return false;
		return true;
	}

}
