package utils;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.TreeMap;

public class TimeSpan
{
	private static String newLine = System.getProperties().getProperty("line.separator");
	public static int highestCount = 1;
	public static int totalNumOfActivities = 0;

	private long startTime;
	private long endTime;
	
	private TreeMap<ActivityCategory, Integer> counters;
	private LinkedList<Activity> activities;


	public TimeSpan(ArrayList<ActivityCategory> activityCategories)
	{
		counters = new TreeMap<ActivityCategory, Integer>();
		activities = new LinkedList<Activity>();
		
		for (ActivityCategory activityCategory : activityCategories)
		{
			counters.put(activityCategory, 0);
		}
	}


	public long getStartTime()
	{
		return startTime;
	}


	public void setStartTime(long startTime)
	{
		this.startTime = startTime;
	}


	public long getEndTime()
	{
		return endTime;
	}


	public void setEndTime(long end)
	{
		this.endTime = end;
	}


	public TreeMap<ActivityCategory, Integer> getCounters()
	{
		return counters;
	}


	public long getLength()
	{
		return endTime - startTime;
	}


	public void count(ActivityCategory category)
	{
		if (counters.containsKey(category))
		{
			Integer newValue = counters.get(category)+1;
			counters.put(category, newValue);
			
			if (highestCount < newValue)
			{
				highestCount = newValue;
			}
		}
		else
		{
			counters.put(category, 1);
		}
		
		totalNumOfActivities++;
	}
	
	public void addActivity(Activity event)
	{
		activities.add(event);
	}
	
	public String toString()
	{
		String str = "";

			str += "StartTime: " + startTime + newLine;
			
			for (Activity activity : activities)
			{
				str += activity + newLine;
			}
			
			str += "EndTime: " + endTime + newLine;
			
			str += "Counted activities per category:" + newLine;
			
			int totalNumOfActivities = 0;
			
			for (ActivityCategory key : counters.keySet())
			{	
				str += key +  ": " + counters.get(key) + newLine;
				totalNumOfActivities += counters.get(key);
			}
			
			str += "total: " + totalNumOfActivities + newLine;

		return str;
	}
	
	public String toCSV()
	{
		String str = "";
			
			for (ActivityCategory key : counters.keySet().toArray(new ActivityCategory[counters.keySet().size()]))
			{	
				str += key +  ": " + counters.get(key) + newLine;
				totalNumOfActivities += counters.get(key);
			}
			
			str += "total: " + totalNumOfActivities + newLine;

		return str;
	}
}
