package visualization;

import static main.LFAProperties.*;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import main.LFAProperties;
import utils.ActivitiesGroup;
import utils.ActivityCategory;

public class GroupedActivitiesVisualization extends JPanel implements MouseWheelListener, MouseListener
{
	private static final long serialVersionUID = -6100488094213598504L;

	private int width;
	private int height;
	private int gapX;
	private int gapY;

	private int squash = VIS_INITIAL_SQUASH;
	private int multiplier;
	private double sqrtFraction = 1.0 / 1.25;

	private LinkedList<ActivitiesGroup> groups = new LinkedList<ActivitiesGroup>();
	private LinkedList<ActivitiesGroup> groupsToDraw = new LinkedList<ActivitiesGroup>();
	private ArrayList<ActivityCategory> activityCategories;

	private BufferedImage imageToDraw;
	private int cropWidth = 0;
	private int cropHeight = 0;
	private String imagePath;


	public GroupedActivitiesVisualization(LinkedList<ActivitiesGroup> groups,
			ArrayList<ActivityCategory> activityCategories, String imagePath)
	{
		this.groups = groups;
		this.activityCategories = activityCategories;

		setLayoutProperties(VIS_WIDTH, VIS_HEIGHT, VIS_GAP_X, VIS_GAP_Y);

		// setSize(width, height * 4);
		setFocusable(true);
		addMouseWheelListener(this);

		imageToDraw = new BufferedImage(width, height * 4, BufferedImage.TYPE_INT_ARGB);
		this.imagePath = imagePath;

		addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e)
			{
				if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_S)
				{
					try
					{

						int x = gapX;
						int y = gapY - 1;
						int w = cropWidth - gapX;
						int h = cropHeight - (gapY - 1);
						
//						if (x + w > imageToDraw.getWidth())
//						{
//							w = imageToDraw.getWidth() - x;
//						}
						
						if (y + h > imageToDraw.getHeight())
						{
							h = imageToDraw.getHeight() - y;
						}

						File outputfile = new File(GroupedActivitiesVisualization.this.imagePath);
						ImageIO.write(imageToDraw.getSubimage(x, y, w, h), "png", outputfile);
						System.out.println("Image saved");
					}
					catch (IOException ex)
					{

					}
				}
			}
		});
	}


	public void setLayoutProperties(int width, int height, int gapX, int gapY)
	{
		this.width = width;
		this.height = height;
		this.gapX = gapX;
		this.gapY = gapY;

		switch (gaDrawType)
		{
			case LINES:
				break;
			case RECTS:
			case BARS:
				multiplier = height / (activityCategories.size() + 1);
				break;
		}
	}


	public void createImage()
	{
		Graphics2D graphics = (Graphics2D) imageToDraw.getGraphics();

		int locX = gapX;
		int locY = gapY;

		long totalDuration = 0;

		graphics.setColor(VIS_BACKGROUND_COLOR);
		graphics.fillRect(0, 0, getWidth(), getHeight());

		groupsToDraw.clear();

		// filter to small groups
		for (ActivitiesGroup group : groups)
		{
			int groupWidth = (int) group.getTimePeriod() / squash;

			if (groupWidth > GA_FILTER)
			{
				if (groupsToDraw.size() > 0)
				{
					ActivitiesGroup previousGroup = groupsToDraw.getLast();

					if (previousGroup.getRelatedCategory().getId().equals(group.getRelatedCategory().getId()))
					{
						previousGroup.addGroup(group, true);
					}
					else
					{
						groupsToDraw.add(new ActivitiesGroup(group));
					}
				}
				else
				{
					groupsToDraw.add(new ActivitiesGroup(group));
				}
			}
		}

		if (activityLines != ActivityLines.OFF)
		{
			for (ActivitiesGroup group : groupsToDraw)
			{
				switch (activityLines)
				{
					case PER_MINUTE:
					case PER_MINUTE_SQRT:
						if (group.getActivitiesPerMinute() > ActivitiesGroup.maxActivitiesPerMinute)
						{
							ActivitiesGroup.maxActivitiesPerMinute = group.getActivitiesPerMinute();
						}

						break;

					case ABSOLUTE:
						if (group.getNumOfActivities() > ActivitiesGroup.maxNumOfActivities)
						{
							ActivitiesGroup.maxNumOfActivities = group.getNumOfActivities();
						}

						break;
					default:
						break;
				}

				totalDuration += group.getTimePeriod();
			}
		}

		for (ActivitiesGroup group : groupsToDraw)
		{
			int groupWidth = (int) group.getTimePeriod() / squash;

			graphics.setColor(group.getRelatedCategory().getColor());

			if (locX + groupWidth >= width - gapX)
			{
				locX = gapX;
				locY += height + gapY;
			}

			switch (gaDrawType)
			{
				case LINES:
					graphics.fillRect(locX, locY, groupWidth, height);
					graphics.setColor(Color.BLACK);

					if (group.getRelatedCategory().getOrder() >= 0)
					{
						// int actLineLocY = locY + height - top -
						// group.getRelatedCategory().getOrder() * multiplier;
						if (activityLines != ActivityLines.OFF)
						{
							drawActivityLine(graphics, locX, locY, groupWidth, group);
						}
					}
					break;

				case RECTS:
					int rectLocY = locY + height - (group.getRelatedCategory().getOrder() + 1) * multiplier;

					graphics.fillRect(locX, rectLocY, groupWidth, multiplier);
					break;

				case BARS:
					int barLocY = locY + height - (group.getRelatedCategory().getOrder() + 1) * multiplier;
					int barHeight = (group.getRelatedCategory().getOrder() + 1) * multiplier;

					graphics.fillRect(locX, barLocY, groupWidth, barHeight);

					if (activityLines != ActivityLines.OFF)
					{
						drawActivityLine(graphics, locX, locY, groupWidth, group);

						// graphics.drawString(String.valueOf(group.getNumOfActivities()), locX +
						// groupWidth / 2, barLocY
						// + barHeight + (count % 2) * 15 + 15);
					}
					break;
			}

			locX += groupWidth;
		}

		drawTimeLines(graphics, LFAProperties.GA_MINUTE_STEPS, totalDuration);
		
		if (locY > height)
		{
			cropWidth = width - gapX;
		}
		else
		{
			cropWidth = locX;
		}
		
		cropHeight = locY + height;

		// Debugging
		// graphics.setColor(Color.BLACK);
		// graphics.drawLine(0, gapY, width, gapY);
		// graphics.drawLine(0, gapY + height, width, gapY + height);

	}


	private void drawActivityLine(Graphics2D graphics, int locX, int locY, int groupWidth, ActivitiesGroup group)
	{
		int actLineLocY = -1;

		switch (activityLines)
		{
			case PER_MINUTE:
				actLineLocY = locY
						+ height
						- (int) ((float) height * ((float) group.getActivitiesPerMinute() / (float) ActivitiesGroup.maxActivitiesPerMinute));
				break;

			case PER_MINUTE_SQRT:
				actLineLocY = locY
						+ height
						- (int) ((float) height * (Math.pow(
								1.0 / ActivitiesGroup.maxActivitiesPerMinute * group.getActivitiesPerMinute(),
								sqrtFraction)));
				break;

			case ABSOLUTE:
				actLineLocY = locY
						+ height
						- (int) ((float) height * ((float) group.getNumOfActivities() / (float) ActivitiesGroup.maxNumOfActivities));
			default:
				break;
		}

		graphics.setColor(Color.BLACK);
		graphics.fillRect(locX, actLineLocY - (int) Math.floor(GA_ACTIVITY_LINES_THICKNESS / 2.0f), groupWidth - 1,
				(int) Math.ceil(GA_ACTIVITY_LINES_THICKNESS / 2.0f));
	}


	public void paint(Graphics graphics)
	{
		createImage();
		graphics.drawImage(imageToDraw, 0, 0, this);
	}


	/**
	 * Achtung: Funktioniert momentan nur mit der initialen Skalierung!!!
	 */
	public void drawTimeLines(Graphics graphics, int minuteSteps, long totalDuration)
	{
		int locX = gapX;
		int locY = gapY + height;
		long currentTime = 0;

		graphics.setColor(Color.BLACK);

		while (currentTime < totalDuration)
		{
			graphics.drawLine(locX, locY, locX, locY + gapY / 4);

			currentTime += minuteSteps * 60000;
			locX += minuteSteps * (60000 / squash);

			if (locX >= width - gapX)
			{
				locX = gapX;
				locY += height + gapY;
			}
		}
	}


	@Override
	public void mouseWheelMoved(MouseWheelEvent e)
	{
		if (e.getWheelRotation() > 0)
		{
			squash = (int) ((float) (squash) * VIS_ZOOM_FACTOR);
		}
		else if (e.getWheelRotation() < 0)
		{
			squash = (int) ((float) (squash) / VIS_ZOOM_FACTOR);
		}

		repaint();
	}


	@Override
	public void mouseClicked(MouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON2)
		{
			GroupedActivitiesVisualization.this.squash = VIS_INITIAL_SQUASH;
			GroupedActivitiesVisualization.this.repaint();
		}
	}


	@Override
	public void mouseEntered(MouseEvent arg0)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mouseExited(MouseEvent arg0)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mousePressed(MouseEvent arg0)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mouseReleased(MouseEvent arg0)
	{
		// TODO Auto-generated method stub

	}
}
