package visualization;

import utils.ActivityCategory;
import utils.TimeSpan;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.Map.Entry;

import static main.LFAProperties.*;

public class TimeSpansVisualization extends JPanel implements MouseWheelListener, MouseListener
{

	private static final long serialVersionUID = -6100488094213598504L;

	private int width;
	private int height;
	private int gapX;
	private int gapY;

	private int squash = VIS_INITIAL_SQUASH;
	private int maxBarHeight = 0;

	// private int globalMax = 22;

	private LinkedList<TimeSpan> timeSpans = new LinkedList<TimeSpan>();

	private BufferedImage imageToDraw;
	private int fontHeight;
	private int cropWidth = 0;
	private int cropHeight = 0;
    private int highestCount;
	private String imagePath;


	public TimeSpansVisualization(LinkedList<TimeSpan> timeSpans, String imagePath)
	{
		this.timeSpans = timeSpans;

		setLayoutProperties(VIS_WIDTH, VIS_HEIGHT, VIS_GAP_X, VIS_GAP_Y);

		// setSize(this.width, this.height * 1);
		setFocusable(true);
		addMouseWheelListener(this);
		addMouseListener(this);

		this.imageToDraw = new BufferedImage(this.width, this.height * 8, BufferedImage.TYPE_INT_ARGB);
		this.imagePath = imagePath;
        this.highestCount = TimeSpan.highestCount;


		addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e)
			{
				if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_S)
				{
					try
					{
						int x;
						
						if (TS_SHOW_DESCRIPTION) 
						{
							x = TS_FONT_GAP - TS_SAVE_FRAME;
						} 
						else 
						{
							x = (gapX - 1) - TS_SAVE_FRAME; 
						}
						
						int y = gapY - TS_SAVE_FRAME;

						if (TS_NUMBER_STEPS != 0)
						{
							y -= fontHeight;
						}

						int w = cropWidth + 2 * TS_SAVE_FRAME;
						
						if (TS_SHOW_DESCRIPTION)
						{
							w -= (TS_FONT_GAP + 1);
						} 
						else
						{
							w -= gapX;
						}
						
						int h = cropHeight - (gapY - 1) + 2 * TS_SAVE_FRAME;

						if (x + w > imageToDraw.getWidth())
						{
							w = imageToDraw.getWidth() - x;
						}

						if (y + h > imageToDraw.getHeight())
						{
							h = imageToDraw.getHeight() - y;
						}

						File outputFile = new File(TimeSpansVisualization.this.imagePath);
						ImageIO.write(imageToDraw.getSubimage(x, y, w, h), "png", outputFile);
						System.out.println("Image saved");
					}
					catch (IOException ex)
					{

					}
				}
			}
		});
	}


	public void setLayoutProperties(int width, int height, int gapX, int gapY)
	{
		this.width = width;
		this.height = height;
		this.gapX = gapX;
		this.gapY = gapY;

		if (TS_NUMBER_STEPS != 0)
		{
            this.gapY += fontHeight;
		}

		if (tsDrawType == TimeSpansDrawType.BARS2)
		{
			maxBarHeight = this.height / this.timeSpans.getFirst().getCounters().size();
		}
	}


	public void createImage()
	{
		Graphics2D graphics = (Graphics2D) imageToDraw.getGraphics();
		FontMetrics metrics = graphics.getFontMetrics(graphics.getFont());
		fontHeight = metrics.getAscent();

		int locX = gapX;
		int locY = gapY;
		int count = 0;

		graphics.setColor(VIS_BACKGROUND_COLOR);
		graphics.fillRect(0, 0, getWidth(), getHeight());

		int descrLocY = locY;

		if (TS_SHOW_DESCRIPTION)
		{
			for (ActivityCategory activityCategory : timeSpans.getFirst().getCounters().keySet())
			{
				descrLocY += maxBarHeight;

				graphics.setColor(Color.BLACK);
				graphics.drawString(activityCategory.getId(), TS_FONT_GAP, descrLocY);

				int lengthInPixels = graphics.getFontMetrics().stringWidth(activityCategory.getId());
				if (gapX < lengthInPixels + 2 * TS_FONT_GAP)
				{
					gapX = lengthInPixels + 2 * TS_FONT_GAP;
				}
			}
		}

		for (TimeSpan timeSpan : timeSpans)
		{
			int timeSpanWidth = (int) timeSpan.getLength() / squash;

			if (locX + timeSpanWidth >= width - gapX)
			{
				locX = gapX;
				locY += height + gapY;

				if (TS_NUMBER_STEPS != 0)
				{
					locY += fontHeight;
				}
			}

			if (count % 2 == 0)
			{
				graphics.setColor(new Color(244, 244, 244));
				graphics.fillRect(locX, locY, timeSpanWidth, height);
			}

			if (TS_NUMBER_STEPS != 0 && count % TS_NUMBER_STEPS == 0)
			{
				graphics.setColor(Color.BLACK);

				String numberToDisplay = String.valueOf(count);
				graphics.drawString(numberToDisplay, locX + timeSpanWidth / 2 - metrics.stringWidth(numberToDisplay)
						/ 2, locY - 2);
			}

			int barLocY = locY;

			for (Entry<ActivityCategory, Integer> entry : timeSpan.getCounters().entrySet())
			{
				int barHeight;


				switch (tsDrawType)
				{
					case BARS:
						int barWidth = timeSpanWidth / timeSpan.getCounters().entrySet().size();
						barHeight = (int) ((float) entry.getValue().intValue() / (float) highestCount * height);
						barLocY += height - barHeight;

						graphics.setColor(entry.getKey().getColor());
						graphics.fillRect(locX, barLocY, barWidth, barHeight);

						locX += barWidth;
						break;
					case BARS2:

						barHeight = (int) (Math.log1p(entry.getValue().intValue() / TS_VERTICAL_STRETCH)
								/ Math.log1p((highestCount + 1) / TS_VERTICAL_STRETCH) * (maxBarHeight - 1));

						barLocY += maxBarHeight;

						graphics.setColor(entry.getKey().getColor());
						graphics.fillRect(locX, barLocY - barHeight, timeSpanWidth, barHeight);

						if (count % 2 == 0)
						{
							graphics.setColor(new Color(233, 233, 233));
							graphics.drawLine(locX, barLocY, locX + timeSpanWidth - 1, barLocY);
						}
						else
						{
							graphics.setColor(new Color(244, 244, 244));
							graphics.drawLine(locX - 1, barLocY, locX + timeSpanWidth, barLocY);
						}
				}
			}

			if (tsDrawType == TimeSpansDrawType.BARS2)
			{
				if (count % TS_GRID_LINES == 0)
				{
					graphics.setColor(new Color(222, 222, 222));
					graphics.drawLine(locX - 1, locY, locX - 1, locY + height);
				}

				locX += timeSpanWidth + 1;
			}
			count++;
		}

		if (locY > height)
		{
			cropWidth = width;
		}
		else
		{
			cropWidth = locX;
		}

		cropHeight = locY + height;

		if (TS_NUMBER_STEPS != 0)
		{
			cropHeight += fontHeight;
		}

		// Debugging
		// graphics.setColor(Color.BLACK);
		// graphics.drawLine(0, GAP, WIDTH, GAP);
		// graphics.drawLine(0, GAP + HEIGHT, WIDTH, GAP + HEIGHT);

	}


	public void paint(Graphics graphics)
	{
		createImage();
		graphics.drawImage(imageToDraw, 0, 0, this);
	}


	@Override
	public void mouseWheelMoved(MouseWheelEvent e)
	{
		if (e.getWheelRotation() > 0)
		{
			squash = (int) ((float) (squash) * VIS_ZOOM_FACTOR);
		}
		else if (e.getWheelRotation() < 0)
		{
			squash = (int) ((float) (squash) / VIS_ZOOM_FACTOR);
		}

		repaint();
	}


	@Override
	public void mouseClicked(MouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON2)
		{
			TimeSpansVisualization.this.squash = VIS_INITIAL_SQUASH;
			TimeSpansVisualization.this.repaint();
		}
	}


	@Override
	public void mouseEntered(MouseEvent e)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mouseExited(MouseEvent e)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mousePressed(MouseEvent e)
	{
		// TODO Auto-generated method stub

	}


	@Override
	public void mouseReleased(MouseEvent e)
	{
		// TODO Auto-generated method stub

	}


    public int getHighestCount() {
        return highestCount;
    }

    public void setHighestCount(int highestCount) {
        this.highestCount = highestCount;
    }
}
