package plg.overlay;/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

import com.intellij.openapi.util.TextRange;
import org.jetbrains.annotations.NotNull;

/**
 * Created by Peter on 05.07.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public enum IconAlignment {

    INNER_LEFT {
        int getOffset(@NotNull TextRange range) {
            return range.getStartOffset();
        }

        @Override
        int getAdditionalVisualOffset(int iconWidth) {
            return 0;
        }
    },
    INNER_RIGHT {
        int getOffset(@NotNull TextRange range) {
            return range.getEndOffset();
        }

        @Override
        int getAdditionalVisualOffset(int iconWidth) {
            return -iconWidth;
        }

        @Override
        boolean isLineWrapDependend() {
            return true;
        }
    },
    OUTER_RIGHT {
        int getOffset(@NotNull TextRange range) {
            return range.getEndOffset();
        }

        @Override
        int getAdditionalVisualOffset(int iconWidth) {
            return 0;
        }

        @Override
        boolean isLineWrapDependend() {
            return true;
        }
    },
    OUTER_LEFT {
        int getOffset(@NotNull TextRange range) {
            return range.getStartOffset();
        }

        @Override
        int getAdditionalVisualOffset(int iconWidth) {
            return -iconWidth;
        }
    }; //problem at the beginning of a line

    /**
     * Returns the Text-Offset drawing location for the supplied element
     *
     * @param range the textrange of the element that describes the location
     * @return the drawing Offset in Characters of the file
     */
    abstract int getOffset(@NotNull TextRange range);

    /**
     * Returns an additional visual offset for the horizontal column inside the editor
     *
     * @param iconWidth the width of the used icon
     * @return a modificator for the visual column
     */
    abstract int getAdditionalVisualOffset(int iconWidth);

    boolean isLineWrapDependend() {
        return false;
    }
}
