/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

//import com.intellij.ide.browsers.BrowsersConfiguration;

import com.intellij.ide.browsers.BrowserFamily;
import com.intellij.ide.browsers.WebBrowser;
import com.intellij.ide.browsers.WebBrowserManager;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.options.ShowSettingsUtil;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.MessageDialogBuilder;
import com.intellij.openapi.ui.Messages;
import com.intellij.ui.ColorChooserService;
import com.intellij.ui.ColorPickerListener;
import com.intellij.ui.JBColor;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.tools.slToolBox;
import plg.ui.SketchlinkColors;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.MessageFormat;
import java.util.ArrayList;

/**
 * Created by Peter on 29.01.14.
 */
public class SLConfigForm {
    private boolean color_edited = false;
    private JTabbedPane tabbedPane1;
    private JTextField serverURLtextfield;
    private JTextField clientNameTextField;
    private JPanel rootComponent;
    private JButton buttonCheckURL;
    private JLabel labelGroupName;
    private JLabel labelServerURL;

    private JCheckBox connectOnStartupCheckBox;
    private JCheckBox currentlyConnectedCheckBox;
    private JComboBox<WebBrowser> comboBox_defaultBrowser;
    private JButton buttonTestBrowser;
    private JCheckBox allowLocalNavigationFromCheckBox;


    private JLabel color_preview03;
    private JLabel color_preview01;
    private JLabel color_preview02;
    private JLabel color_preview04;
    private JLabel color_preview05;
    private JLabel color_preview06;
    private JLabel color_preview07;
    private JLabel color_preview08;
    private JLabel color_preview09;
    private JLabel color_preview10;
    private JLabel color_preview11;
    private JLabel color_preview12;
    private JCheckBox enableLoggingCheckbox;
    private JCheckBox keepSketchlinkDoctagsFromCheckBox;
    private JCheckBox protectSketchlinkDoctagsFromCheckBox;
    private JCheckBox SketchlinkTagShortFoldingName;
    private JComboBox foldingStyleComboBox;
    private JCheckBox showFindLinkAtTop;
    private JButton saveAsDefaultButton;
    private JButton loadDefaultButton;

    public SLConfigForm(@NotNull final Project project) {
        buttonTestBrowser.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (comboBox_defaultBrowser.getSelectedItem() instanceof WebBrowser) {
                    slToolBox.OpenBrowserTab("http://www.st.uni-trier.de", (WebBrowser) comboBox_defaultBrowser.getSelectedItem()); //NON-NLS
                }
            }
        });
        buttonCheckURL.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(@NotNull ActionEvent e) {
                AnAction action = ActionManager.getInstance().getAction("Sketchlink.ConnectToWebSocket");
                if (action == null) {
                    JOptionPane.showMessageDialog(rootComponent, "Error: Could not find connect Action");
                    return;
                }
//                System.out.println("Config: " + e.getClass() + " " + e.getSource() + " " + e);
                ActionManager.getInstance().tryToExecute(action, new MouseEvent(rootComponent, e.getID(), e.getWhen(), e.getModifiers(), 0, 0, 1, false), rootComponent, null, true);
            }
        });
        MouseAdapter colorPickOnClick = showColorPickOnClick();
        color_preview01.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview01.addMouseListener(colorPickOnClick);
        color_preview02.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview02.addMouseListener(colorPickOnClick);
        color_preview03.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview03.addMouseListener(colorPickOnClick);
        color_preview04.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview04.addMouseListener(colorPickOnClick);
        color_preview05.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview05.addMouseListener(colorPickOnClick);
        color_preview06.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview06.addMouseListener(colorPickOnClick);
        color_preview07.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview07.addMouseListener(colorPickOnClick);
        color_preview08.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview08.addMouseListener(colorPickOnClick);
        color_preview09.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview09.addMouseListener(colorPickOnClick);
        color_preview10.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview10.addMouseListener(colorPickOnClick);
        color_preview11.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview11.addMouseListener(colorPickOnClick);
        color_preview12.setBorder(BorderFactory.createLineBorder(JBColor.BLACK));
        color_preview12.addMouseListener(colorPickOnClick);

        saveAsDefaultButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                slProjectComp.getInstance(project).saveAsDefaultSettings();
            }
        });
        loadDefaultButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                slProjectComp.getInstance(project).loadDefaultSettings();
            }
        });
    }

    public static void showInvalidServerMessage(@NotNull final String message, final @NotNull Project project) {
        slProjectComp.getInstance(project).setConnectedToServer(false);

        UIUtil.invokeLaterIfNeeded(new Runnable() {
            @Override
            public void run() {
                MessageDialogBuilder.YesNo yesNo = MessageDialogBuilder.yesNo(slApplicationState.RESOURCE_BUNDLE.getString("invalid.server.url"), MessageFormat.format(slApplicationState.RESOURCE_BUNDLE.getString("the.url.to.the.server.is.invalid.do.you.want.to.change.it.now.n.nerror.01"), message));
                int result = yesNo.show();
                if (result == Messages.YES) {
                    ShowSettingsUtil.getInstance().showSettingsDialog(project, slProjectComp.getInstance(project));
                }
            }
        });

    }

    private MouseAdapter showColorPickOnClick() {
        return new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
            	@Nullable Color color = ColorChooserService.getInstance().showDialog(rootComponent, slApplicationState.RESOURCE_BUNDLE.getString("choose.color"), e.getComponent().getBackground(), true, new ArrayList<ColorPickerListener>(1), true); // IntelliJ 14
                // @Nullable Color color = ColorChooserService.getInstance().showDialog(rootComponent, slApplicationState.RESOURCE_BUNDLE.getString("choose.color"), e.getComponent().getBackground(), true, null, true); // IntelliJ 13
                if (color != null) {
                    e.getComponent().setBackground(color);
                    color_edited = true;
                }
            }
        };
    }

    public JComponent getRootComponent() {
        return rootComponent;
    }


    public void setData(@NotNull final slCurrentProjectSettings data) {
        color_edited = false;
        serverURLtextfield.setText(data.getServerURL());
        clientNameTextField.setText(data.getClientName());
        /**
         * Use special handling for PW field, because
         * 1. java.lang.IllegalStateException: Access from read action is not allowed, because it might lead to a deadlock.
         * 2. No guessing of PW through length
         */
        //passwordTextField.setText(FAKE_PW_DATA);

        //final String pw = data.getGroupPassword();
        color_preview01.setBackground(data.getColor(SketchlinkColors.PREVIEW_HIGHLIGHTED_STROKE));
        color_preview02.setBackground(data.getColor(SketchlinkColors.PREVIEW_HIGHLIGHTED_FILL));
        color_preview03.setBackground(data.getColor(SketchlinkColors.PREVIEW_LINKED_STROKE));
        color_preview04.setBackground(data.getColor(SketchlinkColors.PREVIEW_LINKED_FILL));
        color_preview05.setBackground(data.getColor(SketchlinkColors.PREVIEW_NORMAL_STROKE));
        color_preview06.setBackground(data.getColor(SketchlinkColors.PREVIEW_NORMAL_FILL));

        color_preview07.setBackground(data.getColor(SketchlinkColors.OVERLAY_TARGET));
        color_preview08.setBackground(data.getColor(SketchlinkColors.OVERLAY_TARGET_STROKE));
        color_preview09.setBackground(data.getColor(SketchlinkColors.PREVIEW_CODE_LINKED_BACKGROUND));
        color_preview10.setBackground(data.getColor(SketchlinkColors.PREVIEW_CODE_LINKED_STROKE));
        color_preview11.setBackground(data.getColor(SketchlinkColors.PREVIEW_CODE_AVAILABLE_BACKGROUND));
        color_preview12.setBackground(data.getColor(SketchlinkColors.PREVIEW_CODE_AVAILABLE_STROKE));

        enableLoggingCheckbox.setSelected(data.isServerLoggingEnabled());
        keepSketchlinkDoctagsFromCheckBox.setSelected(data.isSketchAnchorAlwaysFolded());
        protectSketchlinkDoctagsFromCheckBox.setSelected(data.isSketchAnchorsProtected());
        SketchlinkTagShortFoldingName.setSelected(data.isSketchAnchorsInMethodsUseFoldingStyle());

        connectOnStartupCheckBox.setSelected(data.isConnectOnStartup());
        currentlyConnectedCheckBox.setSelected(data.isConnected());
        allowLocalNavigationFromCheckBox.setSelected(data.isAllowRemoteNavigation());
        //comboBox_defaultBrowser.removeAllItems();
        String dataCurrentBrowserID = data.getBrowserID();
        WebBrowser browser = null;
        if (dataCurrentBrowserID != null) {
            browser = WebBrowserManager.getInstance().findBrowserById(dataCurrentBrowserID);
            comboBox_defaultBrowser.addItem(browser);
        }
        for (@NotNull WebBrowser webBrowser : WebBrowserManager.getInstance().getActiveBrowsers()) {
            if (webBrowser == browser || webBrowser.getFamily().equals(BrowserFamily.EXPLORER)) {
                continue;
            }
            comboBox_defaultBrowser.addItem(webBrowser);
        }
        //
//        currentlyConnectedCheckBox.setEnabled(false);
        setConnected(data.isConnected());
        foldingStyleComboBox.setSelectedIndex(data.getFoldingStyle());
        showFindLinkAtTop.setSelected(data.isShowFindForNavigationAtTop());
    }

    public void getData(@NotNull slCurrentProjectSettings data) {
        String url = serverURLtextfield.getText();
        int schemeIndex = url.indexOf("://");
        if (schemeIndex >= 0 && schemeIndex < url.length()) {
            url = url.substring(schemeIndex + 3);
        }
        data.setServerURL(url);
        data.setClientName(clientNameTextField.getText());

        data.setColor(SketchlinkColors.PREVIEW_HIGHLIGHTED_STROKE, color_preview01.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_HIGHLIGHTED_FILL, color_preview02.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_LINKED_STROKE, color_preview03.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_LINKED_FILL, color_preview04.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_NORMAL_STROKE, color_preview05.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_NORMAL_FILL, color_preview06.getBackground());

        data.setColor(SketchlinkColors.OVERLAY_TARGET, color_preview07.getBackground());
        data.setColor(SketchlinkColors.OVERLAY_TARGET_STROKE, color_preview08.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_CODE_LINKED_BACKGROUND, color_preview09.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_CODE_LINKED_STROKE, color_preview10.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_CODE_AVAILABLE_BACKGROUND, color_preview11.getBackground());
        data.setColor(SketchlinkColors.PREVIEW_CODE_AVAILABLE_STROKE, color_preview12.getBackground());
        data.setServerLoggingEnabled(enableLoggingCheckbox.isSelected());
        data.setSketchAnchorAlwaysFolded(keepSketchlinkDoctagsFromCheckBox.isSelected());
        data.setSketchAnchorsProtected(protectSketchlinkDoctagsFromCheckBox.isSelected());
        data.setSketchAnchorsInMethodsUseFoldingStyle(SketchlinkTagShortFoldingName.isSelected());
        data.setShowFindForNavigationAtTop(showFindLinkAtTop.isSelected());

        data.setConnectOnStartup(connectOnStartupCheckBox.isSelected());
        data.setAllowRemoteNavigation(allowLocalNavigationFromCheckBox.isSelected());
        data.setBrowserID(((WebBrowser) comboBox_defaultBrowser.getSelectedItem()).getId().toString());
        data.setFoldingStyle(foldingStyleComboBox.getSelectedIndex());
        color_edited = false;
    }

    public boolean isModified(@NotNull slCurrentProjectSettings data) {
        if (serverURLtextfield.getText() != null ? !serverURLtextfield.getText().equals(data.getServerURL()) : data.getServerURL() != null)
            return true;
        if (clientNameTextField.getText() != null ? !clientNameTextField.getText().equals(data.getClientName()) : data.getClientName() != null)
            return true;
        if (foldingStyleComboBox.getSelectedIndex() != data.getFoldingStyle()) return true;
        if (comboBox_defaultBrowser.getSelectedItem() != data.getBrowser()) return true;
        if (connectOnStartupCheckBox.isSelected() != data.isConnectOnStartup()) return true;
//        if (currentlyConnectedCheckBox.isSelected() != data.isConnected()) return true;
        if (allowLocalNavigationFromCheckBox.isSelected() != data.isAllowRemoteNavigation()) return true;
        if (color_edited) return true;
        if (enableLoggingCheckbox.isSelected() != data.isServerLoggingEnabled()) return true;
        if (protectSketchlinkDoctagsFromCheckBox.isSelected() != data.isSketchAnchorsProtected()) return true;
        if (keepSketchlinkDoctagsFromCheckBox.isSelected() != data.isSketchAnchorAlwaysFolded()) return true;
        if (SketchlinkTagShortFoldingName.isSelected() != data.isSketchAnchorsInMethodsUseFoldingStyle()) return true;
        if (data.isShowFindForNavigationAtTop() != showFindLinkAtTop.isSelected()) return true;
        return false;
    }

    public void setConnected(boolean connectedToServer) {
        serverURLtextfield.setEditable(!connectedToServer);
        serverURLtextfield.setEnabled(!connectedToServer);
        currentlyConnectedCheckBox.setSelected(connectedToServer);
        clientNameTextField.setEditable(!connectedToServer);
        clientNameTextField.setEnabled(!connectedToServer);

    }
}
