/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.command.CommandProcessor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.javadoc.PsiDocComment;
import com.intellij.psi.javadoc.PsiDocTag;
import org.jetbrains.annotations.NotNull;
import plg.overlay.OverlayElement;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;
import plg.ui.overlay.SketchlinkOverlayElementImpl;

import java.util.List;

/**
 * Created by Peter on 09.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class DeleteCodeAnchorAction extends AnAction {

    private static void deletePsiElement(@NotNull final PsiElement element) {
        CommandProcessor.getInstance().executeCommand(element.getProject(),
                new Runnable() {
                    @Override
                    public void run() {
                        ApplicationManager.getApplication().runWriteAction(new Runnable() {
                            @Override
                            public void run() {
                                element.delete();
                            }
                        });
                    }
                }, "Delete link anchor", null);
    }

    private static boolean deleteText(@NotNull LocalLinkAnchor anchor) {
        PsiDocTag anchorTag = anchor.getAnchorTag();
        if (anchorTag == null || !anchorTag.isValid()) {
            return false;
        }
        PsiDocComment containingComment = anchorTag.getContainingComment();
        if (containingComment == null || !containingComment.isValid()) {
            return false;
        }

        PsiElement deleteElement;
        PsiFile containingFile = anchorTag.getContainingFile();
        if (containingFile == null || !containingFile.isWritable()) {
            return false;
        }
        if (SketchlinkManager.TOOLS.checkCommentContainsSketchlink(containingComment, true)) {
            //delete whole comment
            deletePsiElement(containingComment);
        } else {
            //delete doctag only
            deletePsiElement(anchorTag);
        }
        return true;

    }

    public void actionPerformed(AnActionEvent e) {
        Project project = e.getProject();
        OverlayElement overlayElementProvided = e.getData(OverlayElement.OVERLAY_ELEMENT_DATA_KEY);
        SketchlinkOverlayElementImpl overlayElement = null;
        if (overlayElementProvided instanceof SketchlinkOverlayElementImpl) {
            overlayElement = (SketchlinkOverlayElementImpl) overlayElementProvided;
        }
        if (project == null || overlayElement == null)
            return;
        LocalLinkAnchor localAnchor = overlayElement.getLocalAnchor();
        deleteText(localAnchor);
        List<LinkAnchor> linksNotDeprecated = SketchlinkManager.getInstance(project).getLinksNotDeprecated(localAnchor);
        for (LinkAnchor linkAnchor : linksNotDeprecated) {
            //set true to update server
            SketchlinkManager.getInstance(project).addLink(localAnchor, linkAnchor, true, false);
        }
        SketchlinkManager.getInstance(project).removeAnchorFromCache(localAnchor);


    }
}
