/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.project.Project;
import com.kitfox.svg.ShapeElement;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.RemoteLinkAnchorM;
import plg.sketchlinks.RemoteLinkAnchorS;
import plg.sketchlinks.SketchlinkManager;
import plg.ui.preview.AnnotationEditDialog;
import plg.ui.preview.svgwrapper.SVGPanelExt;
import plg.websocket.slWebsocketManager;

import java.util.List;

/**
 * Created by Peter on 09.11.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class EditAnnotationAction extends AnAction {

    @Override
    public void update(@NotNull AnActionEvent e) {
        SVGPanelExt panelExt = e.getData(SVGPanelExt.DATA_KEY);
        Project project = e.getProject();
        if (panelExt == null || project == null) {
            return;
        }
        @NotNull List<ShapeElement> shapeElementsAt = panelExt.getShapeElementsAt(panelExt.getLastPoint());
        SketchlinkManager manager = SketchlinkManager.getInstance(project);
        e.getPresentation().setEnabled(slWebsocketManager.getInstance(project).isConnected());
        for (ShapeElement shapeElement : shapeElementsAt) {
            String id = shapeElement.getId();
            if (id == null) {
                continue;
            }
            @Nullable LinkAnchor knownAnchor = manager.getKnownAnchor(id);
            if (knownAnchor instanceof RemoteLinkAnchorS) {
                e.getPresentation().setText("Edit Sketch Annotation");
                return;
            } else {
                if (knownAnchor instanceof RemoteLinkAnchorM) {
                    e.getPresentation().setText("Edit Marker Annotation");
                    return;
                }
            }
        }
        e.getPresentation().setEnabled(false);
        e.getPresentation().setVisible(true);
    }

    public void actionPerformed(AnActionEvent e) {
        SVGPanelExt panelExt = e.getData(SVGPanelExt.DATA_KEY);
        Project project = e.getProject();
        if (panelExt == null || project == null) {
            return;
        }
        @NotNull List<ShapeElement> shapeElementsAt = panelExt.getShapeElementsAt(panelExt.getLastPoint());
        SketchlinkManager manager = SketchlinkManager.getInstance(project);
        for (ShapeElement shapeElement : shapeElementsAt) {
            String id = shapeElement.getId();
            if (id == null) {
                continue;
            }
            @Nullable LinkAnchor knownAnchor = manager.getKnownAnchor(id);
            if (knownAnchor instanceof RemoteLinkAnchorS) {
                AnnotationEditDialog.createAndShow((RemoteLinkAnchorS) knownAnchor);
                return;
            } else {
                if (knownAnchor instanceof RemoteLinkAnchorM) {
                    AnnotationEditDialog.createAndShow((RemoteLinkAnchorM) knownAnchor);
                    return;
                }
            }

        }
    }
}
