/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.psi.util.PsiTreeUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchlinkManager;
import plg.tools.slSearch;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by Peter on 09.06.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class LinkedArtifactsInFileAction extends AnAction {

    @NotNull
    public static Collection<PsiDocTag> getSketchlinkTagsInFile(@NotNull PsiFile file) {
        @NotNull Collection<PsiDocTag> docTags = PsiTreeUtil.findChildrenOfType(file, PsiDocTag.class);

        @NotNull Collection<PsiDocTag> sketchlinkTags = new ArrayList<>();
        for (@NotNull PsiDocTag docTag : docTags) {
            if (SketchlinkManager.TOOLS.isSketchlink(docTag)) {
                sketchlinkTags.add(docTag);
            }
        }
        return sketchlinkTags;

    }

    public void actionPerformed(@NotNull AnActionEvent e) {

        @Nullable PsiFile file = e.getData(DataKeys.PSI_FILE);
        if (file == null) {
            return;
        }
        @NotNull Collection<PsiDocTag> tagsInFile = getSketchlinkTagsInFile(file);
        @NotNull Set<PsiElement> allTargets = new HashSet<>(tagsInFile.size());
        for (@NotNull PsiDocTag docTag : tagsInFile) {
            allTargets.add(SketchlinkManager.TOOLS.getReferencedPsiElement(docTag));
        }
        slSearch.browseToLocal(allTargets);
    }
}
