/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.MessageDialogBuilder;
import com.intellij.openapi.ui.Messages;
import com.intellij.psi.*;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.psi.util.PsiUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchlinkManager;
import plg.slApplicationState;
import plg.tools.slGuiTools;
import plg.websocket.commands.builder.LoggingRequestBuilder;
import plg.websocket.slWebsocketManager;

import java.text.MessageFormat;
import java.util.concurrent.atomic.AtomicBoolean;


/**
 * Created by Peter on 07.02.14.
 */
public class createNewCodeAnchorAction extends AnAction {

    @Override
    public void update(@NotNull AnActionEvent e) {
        @Nullable Editor editor = e.getData(DataKeys.EDITOR);
        @Nullable Project project = e.getProject();
        if (editor == null || project == null) {
            // e.getPresentation().setEnabled(false);
            return;
        }
        int caretoffset = editor.getCaretModel().getOffset();
        @Nullable final PsiFile psiFile = PsiDocumentManager.getInstance(project).getPsiFile(editor.getDocument());
        if (psiFile == null) {
            return;
        }
        @NotNull PsiElement element = PsiUtil.getElementAtOffset(psiFile, caretoffset);
        @NotNull PsiElement topmostLinkableElement = SketchlinkManager.TOOLS.getTopmostLinkableElement(element);
        String typeName = topmostLinkableElement.getClass().getSimpleName();
        int start = typeName.startsWith("Psi") ? 3 : 0;
        int end = typeName.endsWith("Impl") ? typeName.length() - 4 : typeName.length(); //NON-NLS NON-NLS
        end = typeName.endsWith("PackageStatementImpl") ? end - 9 : end; //NON-NLS
        typeName = typeName.substring(start, end);
        e.getPresentation().setText(MessageFormat.format(slApplicationState.RESOURCE_BUNDLE.getString("create.a.new.link.anchor.for.0"), typeName));
    }

    public void actionPerformed(@NotNull AnActionEvent e) {
        @NotNull final AtomicBoolean foundErrors = new AtomicBoolean(false);
        @Nullable final Editor editor = e.getData(DataKeys.EDITOR);
        @Nullable final Project project = e.getProject();
        if (project == null || editor == null) {
            return;
        }
        if (!slWebsocketManager.getInstance(project).isConnected()) {
            MessageDialogBuilder.YesNoCancel yesNoCancel = MessageDialogBuilder.yesNoCancel(slApplicationState.RESOURCE_BUNDLE.getString("connect"),
                    slApplicationState.RESOURCE_BUNDLE.getString("to.keep.locally.existing.sketchanchors.synchronized.with.those.on.the.server.it.is.recommended.to.connect.to.a.sketchlink.server.first.n.nconnect.now"));
            int result = yesNoCancel.show();
            if (result == Messages.CANCEL) {
                return;
            } else {
                if (result == Messages.YES) {
                    AnAction action = ActionManager.getInstance().getAction("Sketchlink.ConnectToWebSocket");
                    if (action != null) {
                        action.actionPerformed(e);
                    }
                }
            }
        }
        int caretoffset = editor.getCaretModel().getOffset();
        @Nullable final PsiFile psiFile = PsiDocumentManager.getInstance(project).getPsiFile(editor.getDocument());
        if (psiFile == null) {
            return;
        }
        @Nullable final Document document = PsiDocumentManager.getInstance(project).getDocument(psiFile);
        if (document == null) {
            return;
        }
        //check for syntax errors
        @NotNull PsiRecursiveElementVisitor visitor = new PsiRecursiveElementVisitor() {
            @Override
            public void visitElement(PsiElement element) {
                if (element instanceof PsiErrorElement) {
                    if (!foundErrors.get()) {
                        slGuiTools.ShowNotificationAtBottom(slApplicationState.RESOURCE_BUNDLE.getString("can.t.insert.new.anchors.while.there.are.errors.in.the.file"), project);
                    }
                    foundErrors.set(true);

                }
            }
        };
        visitor.visitFile(psiFile);
        if (foundErrors.get()) {
            return;
        }
        @Nullable PsiElement elementAtCaretOffset = PsiUtil.getElementAtOffset(psiFile, caretoffset);
        //System.out.println("PsiElement under cursor: " + elementAtCaretOffset);
        if (elementAtCaretOffset instanceof PsiWhiteSpace) {
            int linestartoffset =
                    editor.getDocument().getLineStartOffset(editor.getDocument().getLineNumber(elementAtCaretOffset.getTextRange().getEndOffset()));
            @Nullable PsiElement sibling;
            if (linestartoffset <= caretoffset) {
                sibling = PsiTreeUtil.nextLeaf(elementAtCaretOffset, true);
            } else {
                sibling = PsiTreeUtil.prevLeaf(elementAtCaretOffset, true);
            }
            if (sibling != null) {
                elementAtCaretOffset = sibling;
            }
        }
        @Nullable String sketchlink = SketchlinkManager.getInstance(project).createSketchlink(elementAtCaretOffset);
        new LoggingRequestBuilder("created link anchor; " + sketchlink).send(project); //NON-NLS

        //ask the user to open the sketch inside the webview

//        if (slProjectComp.getInstance(project).getWebsocketManager().isConnected()) {
//            MessageDialogBuilder.YesNo yesNoCancel = MessageDialogBuilder.yesNo(slApplicationComp.RESOURCE_BUNDLE.getString("create.new.link"),
//                    "Open in WebView");
//            yesNoCancel.yesText(slApplicationComp.RESOURCE_BUNDLE.getString("open.webview.in.browser"));
//            yesNoCancel.noText(slApplicationComp.RESOURCE_BUNDLE.getString("do.it.later"));
//            int result = yesNoCancel.show();
//            if (result == Messages.YES && sketchlink != null) {
//                //TODO: Create Message to allow user to create a new link on a remote webview
//                PreviewManager.openAnchorInLocalTab(sketchlink);
//                SketchlinkManager.getInstance(project).cleanOldPreviews(true);
//            }
//        }
    }


}
