/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.vcs.*;
import com.intellij.openapi.vcs.changes.*;
import com.intellij.openapi.vcs.changes.ui.CommitChangeListDialog;
import com.intellij.openapi.vcs.history.VcsFileRevision;
import com.intellij.openapi.vcs.history.VcsHistorySession;
import com.intellij.openapi.vfs.VirtualFile;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Peter on 10.02.14.
 * Shows the current Status for the loaded File
 */
public class slGitControl extends AnAction {
    @Override
    public void update(@NotNull AnActionEvent e) {
        @Nullable Editor editor = e.getData(DataKeys.EDITOR);
        if (editor == null) {
            e.getPresentation().setText("Git Control: " + "Editor Object not available");
            return;
        }
        @Nullable VirtualFile file = e.getData(DataKeys.VIRTUAL_FILE);
        if (file == null) {
            e.getPresentation().setText("Git Control:" + "null or not loaded");
            return;
        }
        FileStatus status = FileStatusManager.getInstance(editor.getProject()).getStatus(file);
        e.getPresentation().setText("Git Control: File" + status.getText());

    }

    public void actionPerformed(@NotNull AnActionEvent e) {
        // TODO: insert action logic here
        @Nullable VirtualFile file = e.getData(DataKeys.VIRTUAL_FILE);
        @Nullable final Editor editor = e.getData(DataKeys.EDITOR);
        if (file == null || editor == null || editor.getProject() == null) {


            JOptionPane.showMessageDialog((editor != null ? editor.getComponent() : null), "Some problem with the file, editor or project");
            return;
        }
        @Nullable AbstractVcs abstractVcs = ProjectLevelVcsManager.getInstance(e.getProject()).getVcsFor(file);
        if (abstractVcs == null) {
            JOptionPane.showMessageDialog(editor.getComponent(), "No VCS found?");
            return;
        }
        @NotNull String info = "Name: " + abstractVcs.getName()
                + "\nType: " + abstractVcs.getType().name()
                + "\nInVCS: " + abstractVcs.fileExistsInVcs(new FilePathImpl(file))
                + "\nVCS Class: " + abstractVcs.getClass().getCanonicalName()
                + "\nFileStatus: " + FileStatusManager.getInstance(editor.getProject()).getStatus(file).getText()
                + "\nCommitExecutors: " + abstractVcs.getCommitExecutors().size()
                + "\nChangeList: " + ChangeListManager.getInstance(editor.getProject()).getStatus(file).getText()
                + "\nIsFile Changed: " + ChangeListManagerImpl.isFileChanged(editor.getProject(), file)
                + "\nCheckoutProvider: " + abstractVcs.getCheckoutProvider()
                + "\nMessage: " + (abstractVcs.getCheckoutProvider() == null ? "null" : abstractVcs.getCheckoutProvider().getVcsName())
                + "\nGetStatusEnvironment: " + abstractVcs.getStatusEnvironment() //null
                + "\nVcsHistoryProvider: " + abstractVcs.getVcsHistoryProvider()
                + "\nHistory available: " + (abstractVcs.getVcsHistoryProvider() == null ? "null" : abstractVcs.getVcsHistoryProvider().canShowHistoryFor(file));


        JOptionPane.showMessageDialog(editor.getComponent(), info, "slGitControl.java", JOptionPane.INFORMATION_MESSAGE);
        if (abstractVcs.getVcsHistoryProvider() != null && abstractVcs.getVcsHistoryProvider().canShowHistoryFor(file)) {
            @Nullable VcsHistorySession session;
            try {
                session = abstractVcs.getVcsHistoryProvider().createSessionFor(new FilePathImpl(file));
                if (session != null) {
                    for (@NotNull VcsFileRevision revisionList : session.getRevisionList()) {
                        System.out.println("List: " + revisionList.getBranchName() + " & " + revisionList.getAuthor() + " & " + revisionList.getRevisionNumber().asString() + " & " + revisionList.getRevisionDate());
                    }

                }


            } catch (VcsException e1) {
                e1.printStackTrace();
            }

        }

        //tries to clone vcs
//        if (abstractVcs.getCheckoutProvider()!=null) {
//            abstractVcs.getCheckoutProvider().doCheckout(editor.getProject(),new CheckoutProvider.Listener() {
//                @Override
//                public void directoryCheckedOut(File directory, VcsKey vcs) {
//                    System.out.println("slGitControl Listener: "+directory.getName()+" --- "+vcs.getName());
//                }
//
//                @Override
//                public void checkoutCompleted() {
//                    System.out.println("Checkout complete");
//
//                }
//            });
//        }

      /*  if (abstractVcs instanceof GitVcs) {
            @Nullable List<VirtualFile> filelist = GitPull.getGitRoots(editor.getProject(), (GitVcs) abstractVcs);
            if (filelist == null) {
                return;
            }
            for (@NotNull VirtualFile file1 : filelist) {
                System.out.println("Gitfiles: " + file1.getCanonicalPath() + "&" + file1.getName() + " " + file1.isInLocalFileSystem());
            }
        }*/

//        if ((abstractVcs instanceof GitVcs))
//        {
//            System.out.println("GitVcs found");
//            GitVcs gitVcs = (GitVcs) abstractVcs;
//            UpdateEnvironment environment= /*abstractVcs.getUpdateEnvironment();//*/ gitVcs.getUpdateEnvironment();
//            if (environment == null)
//            {
//                System.out.println("slgitcontrol: environment null");
//                return;
//            }
//
//            FilePath path = new FilePathImpl(file.getParent());
//            FileGroup fileGroup = UpdatedFiles.create().getGroupById(FileGroup.CHANGED_ON_SERVER_ID);
//            UpdatedFiles updatedFiles = UpdatedFiles.create();
//
//            for (FileGroup fileGroup2 : updatedFiles.getTopLevelGroups()) {
//                System.out.println("FileGroup: "+fileGroup2.getStatusName()+" & "+fileGroup2.getUpdateName()+" & "+fileGroup2.getUpdatedFiles().size());
//            }
//            SequentialUpdatesContext updatesContext = new SequentialUpdatesContext() {
//                @NotNull
//                @Override
//                public String getMessageWhenInterruptedBeforeStart() {
//                    return "Sequential sketchlink what the hell";
//                }
//
//                @Override
//                public boolean shouldFail() {
//
//                    return false;
//                }
//            };
//
//            Ref<SequentialUpdatesContext> updatesContextRef = Ref.create(updatesContext);
//
//            System.out.println("Starting updateDirectories now");
//            UpdateSession session= environment.updateDirectories(new FilePath[]{path}, updatedFiles, new ProgressIndicatorBase(), updatesContextRef);
//
//            for (VcsException vcsException : session.getExceptions()) {
//                System.out.println("Exception: "+vcsException.getMessage());
//            }
//            System.out.println("Canceled? "+session.isCanceled());
//
//        } else {
//            System.out.println("Abstractvs: "+abstractVcs.getClass().getCanonicalName());
//        }

        @NotNull List<Change> list = new ArrayList<>();
        @Nullable Change mychange = ChangeListManager.getInstance(editor.getProject()).getChange(file);
        if (mychange == null || abstractVcs.getCommitExecutors() == null) {
            System.out.println("slGitControl: mychange is null or commitexecutors: " + abstractVcs.getCommitExecutors());
            return;
        }
        list.add(mychange);

        @NotNull CommitResultHandler resultHandler = new CommitResultHandler() {
            @Override
            public void onSuccess(@NotNull String commitMessage) {
                JOptionPane.showMessageDialog(editor.getComponent(), "Commit was a success!\n" + commitMessage);
            }

            @Override
            public void onFailure() {
                JOptionPane.showMessageDialog(editor.getComponent(), "Commit failed!\n");

            }
        };
        @Nullable LocalChangeList changeList = ChangeListManager.getInstance(editor.getProject()).getChangeList("Sketchlink");
        if (changeList == null) {
            changeList = ChangeListManager.getInstance(editor.getProject()).addChangeList("Sketchlink", "Sketchlink comment");
        }

        System.out.println(changeList.getName() + " " + file.isValid() + " " + mychange.getBeforeRevision() + "&" + mychange.getAfterRevision());


        ChangeListManager.getInstance(editor.getProject()).commitChanges(changeList, list);
        assert (abstractVcs.getCommitExecutors() != null);
        @NotNull List<CommitExecutor> commitExecutorList = new ArrayList<>();
        CommitChangeListDialog.commitChanges(editor.getProject(),
                list,
                ChangeListManager.getInstance(editor.getProject()).getDefaultChangeList(),
                commitExecutorList,
                false,
                "Sketchlink comment here",
                resultHandler
        );


    }
}
