/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiElement;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.psi.util.PsiTreeUtil;
import com.intellij.refactoring.listeners.RefactoringElementListener;
import com.intellij.refactoring.listeners.UndoRefactoringElementListener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;
import plg.websocket.commands.builder.ChangeArtifactMetaDataRequestBuilder;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by Peter on 12.02.14.
 */
public class slRefactoringElementListener implements RefactoringElementListener, UndoRefactoringElementListener {
    private static final slRefactoringElementListener _instance = new slRefactoringElementListener();

    @NotNull
    public static slRefactoringElementListener getInstance() {
        return _instance;
    }


    private void sendUpdate(@NotNull PsiElement element) {
        @Nullable PsiDocTag docTag = SketchlinkManager.TOOLS.getSketchlinkForElement(element);
        if (docTag == null) {
            return;
        }
        @Nullable LocalLinkAnchor localSketchAnchor = SketchlinkManager.getAnchorOf(docTag);
        if (localSketchAnchor == null) {
            return;
        }
        new ChangeArtifactMetaDataRequestBuilder(localSketchAnchor).queue(localSketchAnchor.getProject(), null, true);
    }

    /**
     * Gets called AFTER refactoring.
     */
    @Override
    public void elementMoved(@NotNull PsiElement newElement) {
        sendUpdate(newElement);
    }

    @Override
    public void elementRenamed(@NotNull PsiElement newElement) {
        if (newElement instanceof PsiClass) {
            @NotNull Collection<PsiDocTag> psiDocTags = PsiTreeUtil.findChildrenOfType(newElement.getContainingFile(), PsiDocTag.class);
            @NotNull List<PsiDocTag> toChange = new ArrayList<>();
            for (@NotNull PsiDocTag psiDocTag : psiDocTags) {
                if (SketchlinkManager.TOOLS.isSketchlink(psiDocTag)) {
                    toChange.add(psiDocTag);
                }
            }
            SketchlinkManager.TOOLS.updateDocTagMD5(toChange);
            for (@NotNull PsiDocTag docTag : toChange) {
                sendUpdate(docTag);
            }
        } else {
            sendUpdate(newElement);
        }
    }

    @Override
    public void undoElementMovedOrRenamed(@NotNull PsiElement newElement, @NotNull String oldQualifiedName) {
        sendUpdate(newElement);
    }
}
