/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.actions.quicktesting;

import com.intellij.openapi.Disposable;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.editor.Editor;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.refactoring.RefactoringActionHandler;
import com.intellij.refactoring.RefactoringActionHandlerFactory;
import com.intellij.refactoring.listeners.RefactoringElementListenerProvider;
import com.intellij.refactoring.listeners.RefactoringEventData;
import com.intellij.refactoring.listeners.RefactoringEventListener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;
import plg.websocket.commands.builder.ChangeArtifactMetaDataRequestBuilder;

/**
 * Created by Peter on 12.02.14.
 */
public class slRefactoringTest extends AnAction {


    @Override
    public void update(@NotNull AnActionEvent e) {
        e.getPresentation().setText("Register Refactoring Listener (disabled)");
    }

    public void actionPerformed(@NotNull AnActionEvent e) {
        assert (e.getProject() != null);


        @Nullable Editor editor = e.getData(DataKeys.EDITOR);
        @Nullable PsiFile psiFile = e.getData(DataKeys.PSI_FILE);
        @Nullable PsiElement element = e.getData(DataKeys.PSI_ELEMENT);
        if (editor == null || psiFile == null) {
            System.out.println("slRefactorTest - null at beginning: " + editor + " " + psiFile + " " + element);
            return;
        }
        e.getProject().getMessageBus().connect(MyRefactorEventListener.getInstance()).subscribe(RefactoringEventListener.REFACTORING_EVENT_TOPIC, MyRefactorEventListener.getInstance());


//        RefactoringElementListenerProvider[] providers=  RefactoringElementListenerProvider.EP_NAME.getExtensions();
//
//        for (RefactoringElementListenerProvider provider : providers) {
//            System.out.println("Listeners: "+provider.getClass());
//        }
        System.out.println("Refactoring: " + RefactoringElementListenerProvider.EP_NAME);


        RefactoringActionHandler refactor = RefactoringActionHandlerFactory.getInstance().createRenameHandler();
        refactor.invoke(e.getProject(), new PsiElement[]{element}, e.getDataContext());


        //How to create "intelligent" undoable action
        /*UndoableAction action = new BasicUndoableAction() {
            @Override
            public void undo() {
                if (elementListener instanceof UndoRefactoringElementListener) {
                    ((UndoRefactoringElementListener)elementListener).undoElementMovedOrRenamed(myChangeInfo.getMethod(), fqn);
                }
            }

            @Override
            public void redo() {
            }
        };
        UndoManager.getInstance(myProject).undoableActionPerformed(action);*/
    }

    private static class MyRefactorEventListener implements RefactoringEventListener, Disposable {
        private static final MyRefactorEventListener instance = new MyRefactorEventListener();

        @NotNull
        public static MyRefactorEventListener getInstance() {
            return instance;
        }

        /**
         * Is fired when refactoring enters its write phase (find usages, conflict detection phases are passed already)
         *
         * @param refactoringId
         * @param beforeData
         */
        @Override
        public void refactoringStarted(@NotNull String refactoringId, @Nullable RefactoringEventData beforeData) {
            System.out.println("MyRefactorEventListener#refactoringStarted: " + refactoringId);

            if (beforeData == null) {

                return;
            }
            @Nullable PsiElement[] elements = beforeData.getUserData(RefactoringEventData.PSI_ELEMENT_ARRAY_KEY);
            if (elements == null) {
                System.out.println("MyRefactorEventListener#refactoringStarted: no element array - " + beforeData.getUserData(RefactoringEventData.PSI_ELEMENT_KEY));
                return;
            }
            for (@NotNull PsiElement element : elements) {

                System.out.println("Refact-Start: " + element.getTextRange() + " " + element.getContainingFile().getName() + " " + element.getNavigationElement().getText());

            }

        }

        /**
         * Is fired when refactoring is completed, probably with conflicts.
         *
         * @param refactoringId
         * @param afterData
         */
        @Override
        public void refactoringDone(@NotNull String refactoringId, @Nullable RefactoringEventData afterData) {
            System.out.println("MyRefactorEventListener#refactoringDone: " + refactoringId);
            if (afterData == null) {

                return;
            }

            @Nullable PsiElement[] elements = afterData.getUserData(RefactoringEventData.PSI_ELEMENT_ARRAY_KEY);
            @Nullable PsiElement userData = afterData.getUserData(RefactoringEventData.PSI_ELEMENT_KEY);
            if (elements == null) {
                if (userData == null) {
                    return;
                } else {
                    elements = new PsiElement[]{userData};
                }
            }
            for (@NotNull PsiElement element : elements) {
                @Nullable PsiDocTag sketchlinkForElement = SketchlinkManager.TOOLS.getSketchlinkForElement(element);
                if (sketchlinkForElement == null) {
                    continue;
                }
                @Nullable LocalLinkAnchor localSketchAnchor = SketchlinkManager.getAnchorOf(sketchlinkForElement);
                if (localSketchAnchor == null) {
                    continue;
                }
                new ChangeArtifactMetaDataRequestBuilder(localSketchAnchor).queue(localSketchAnchor.getProject(), null, true);
            }

        }

        /**
         * Is fired when conflicts are detected. If the next event comes from the same refactoring then conflicts were ignored.
         *
         * @param refactoringId
         * @param conflictsData should contain string representation of the conflicts
         */
        @Override
        public void conflictsDetected(@NotNull String refactoringId, @NotNull RefactoringEventData conflictsData) {
            System.out.println("MyRefactorEventListener#conflictsDetected: " + refactoringId);

            @Nullable PsiElement[] elements = conflictsData.getUserData(RefactoringEventData.PSI_ELEMENT_ARRAY_KEY);
            if (elements == null)
                return;
            for (@NotNull PsiElement element : elements) {
                System.out.println("Refact-Conflict: " + element.getTextRange() + " " + element.getContainingFile().getName() + " " + element.getNavigationElement().getText());

            }

        }

        /**
         * Is fired when undoable action created on refactoring execution is undone.
         *
         * @param refactoringId
         */
        @Override
        public void undoRefactoring(@NotNull String refactoringId) {
            System.out.println("MyRefactorEventListener#undoRefactoring: " + refactoringId);
            //save stack

        }

        @Override
        public void dispose() {

        }
    }
}
