/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package plg.sketchlinks;

import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.slApplicationState;
import plg.tools.slToolBox;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import javax.swing.*;
import java.security.InvalidParameterException;
import java.text.MessageFormat;

/**
 * Created by Peter on 22.03.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public final class LinkAnchorImpl implements LinkAnchor {
    @NotNull
    private final Project project;
    private String anchorId;
    private ImageIcon myImageIcon;
    private Icon oldIcon;

    public LinkAnchorImpl(@NotNull String anchorId, @NotNull Project project) {
        if (!SketchlinkManager.TOOLS.isValidID(anchorId)) {
            throw new InvalidParameterException("The format of the LinkAnchor is invalid: " + anchorId);
        }
        this.anchorId = anchorId;
        oldIcon = slApplicationState.getInstance().getOverlayIcon();
        myImageIcon = new ImageIcon(slToolBox.iconToImage(oldIcon));
        this.project = project;
    }

    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        @NotNull LinkAnchorImpl that = (LinkAnchorImpl) o;

        if (!anchorId.equals(that.anchorId)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return anchorId.hashCode();
    }

    @NotNull
    @Override
    public Project getProject() {
        return project;
    }

    @NotNull
    @Override
    public String getAnchorId() {
        return anchorId;
    }

    @Override
    public void setAnchorId(@NotNull String id) {
        if (SketchlinkManager.TOOLS.isValidID(id)) {
            this.anchorId = id;
        }
    }

    @NotNull
    @Override
    public String getDescription() {
        return MessageFormat.format(slApplicationState.RESOURCE_BUNDLE.getString("an.anchor.of.type.0"), getAnchorType().name());
    }

    @NotNull
    @Override
    public AnchorTypes getAnchorType() {
        switch (anchorId.charAt(0)) {
            case 'c':
                return AnchorTypes.Code_Anchor;
            case 'u':
                return AnchorTypes.URL_Anchor;
            case 'm':
                return AnchorTypes.SVG_Rectangle_Anchor;
            case 's':
                return AnchorTypes.SVG_Anchor;
        }
        return AnchorTypes.URL_Anchor;
    }

    @NotNull
    @Override
    public ImageIcon getImageIcon() {
        if (oldIcon != slApplicationState.getInstance().getOverlayIcon()) {
            oldIcon = slApplicationState.getInstance().getOverlayIcon();
            myImageIcon = new ImageIcon(slToolBox.iconToImage(oldIcon));
        }
        return myImageIcon;
    }

    @Override
    public void navigateTo() {
        throw new NotImplementedException();
    }
}
