/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.sketchlinks;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.ui.preview.PreviewItem;

import javax.swing.*;
import java.net.URL;

/**
 * Created by noxin on 15.07.14.
 */
public interface PreviewableAnchor extends LinkAnchor {
    /**
     * The URL for the Preview of the sketch or null, if it's not known yet or the anchor is of type CodeAnchor
     *
     * @return
     */
    @Nullable
    URL getPreviewLocation();

    void setPreviewLocation(@NotNull URL previewURL);

    /**
     * Gets the Preview Panel for this Anchor, might be a placeholder if no actual Preview has been loaded yet.
     * If the actual data is loaded while showing the placeholder, the panel will be automatically updated.
     *
     * @return the preview for this Anchor or a placeholder until the final preview is available
     */
    @NotNull
    JPanel getPreview();

    @NotNull
    PreviewItem getPreviewItem();

    /**
     * Try to unload Preview Data. The Preview Data gets unloaded if it hasn't been accessed for more than
     * a certain time. Does not clear the PreviewLocation.
     * <p/>
     * After using this Method, use preparePreview before calling getPreview again.
     *
     * @param forceUnload ignore last access time and force unloading
     * @return true if the data has been unloaded or is not available, false otherwise
     */
    boolean clearPreviewDataIfOld(boolean forceUnload);

    /**
     * Tries to load ImageData from it's PreviewLocation. Should be called before getPreview()
     */
    void preparePreview();


}
