/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.sketchlinks;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.ui.preview.PreviewItem;
import plg.websocket.commands.builder.GetArtifactMetaDataRequestBuilder;

import javax.swing.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class RemoteLinkAnchorM implements RemoteLinkAnchor<SketchMetadataTypeM>, LinkAnchorParentOwner, PreviewableAnchor {
    @NotNull
    private final LinkAnchor myLinkAnchor;
    @NotNull
    private final RemoteLinkAnchorS myParentAnchor;
    @NotNull
    private final Project project;
    private SketchMetadataTypeM myMetadata;
    //property support
    @NotNull
    private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    public RemoteLinkAnchorM(@NotNull String anchorId, @NotNull RemoteLinkAnchorS parentAnchor, @NotNull Project project) {
        this.myLinkAnchor = new LinkAnchorImpl(anchorId, project);
        this.myParentAnchor = parentAnchor;
        this.project = project;
        ApplicationManager.getApplication().executeOnPooledThread(new Runnable() {
            @Override
            public void run() {
                if (getMetadata() == null) {
                    new GetArtifactMetaDataRequestBuilder(RemoteLinkAnchorM.this).send(getProject(), null);

                }
            }
        });
    }


    @NotNull
    @Override
    public Project getProject() {
        return project;
    }

    @NotNull
    public RemoteLinkAnchor getParentAnchor() {
        return this.myParentAnchor;
    }

    @Override
    @Nullable
    public URL getPreviewLocation() {
        return myParentAnchor.getPreviewLocation();
    }

    @Override
    public void setPreviewLocation(@NotNull URL previewURL) {
        myParentAnchor.setPreviewLocation(previewURL);
    }

    @Nullable
    @Override
    public SketchMetadataTypeM getMetadata() {
        return myMetadata;
    }

    @Override
    public void setMetadata(@NotNull SketchMetadataTypeM metadata) {
        if (metadata.equals(this.myMetadata))
            return;
        SketchMetadataTypeM oldValue = this.myMetadata;
        this.myMetadata = metadata;
        propertyChangeSupport.firePropertyChange(SketchMetadata.METADATA_Property, oldValue, this.myMetadata);
    }

    @NotNull
    @Override
    public JPanel getPreview() {
        return myParentAnchor.getPreview();
    }

    @Override
    @NotNull
    public PreviewItem getPreviewItem() {
        return myParentAnchor.getPreviewItem();
    }

    @Override
    public boolean clearPreviewDataIfOld(boolean forceUnload) {
        return myParentAnchor.clearPreviewDataIfOld(forceUnload);
    }

    @Override
    public void preparePreview() {
        myParentAnchor.preparePreview();
        ApplicationManager.getApplication().executeOnPooledThread(new Runnable() {
            @Override
            public void run() {
                if (getMetadata() == null) {
                    new GetArtifactMetaDataRequestBuilder(RemoteLinkAnchorM.this).send(getProject(), null);
                }
            }
        });
    }

    @Override
    @NotNull
    public String getAnchorId() {
        return myLinkAnchor.getAnchorId();
    }

    @Override
    public void setAnchorId(@NotNull String id) {
        myLinkAnchor.setAnchorId(id);
    }

    @Override
    @NotNull
    public String getDescription() {
        return (myMetadata != null) ? myMetadata.getAnnotation() : myLinkAnchor.getDescription();

    }

    @Override
    @NotNull
    public AnchorTypes getAnchorType() {
        return myLinkAnchor.getAnchorType();
    }

    @Override
    @NotNull
    public ImageIcon getImageIcon() {
        return myLinkAnchor.getImageIcon();
    }

    @Override
    public void navigateTo() {
        myParentAnchor.navigateTo();
    }


    @NotNull
    @Override
    public LinkAnchor getParent() {
        return myParentAnchor;
    }

    @Override
    public void addPropertyChangeListener(@NotNull PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    @Override
    public void removePropertyChangeListener(@NotNull PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    @NotNull
    @Override
    public List<PropertyChangeListener> getPropertyChangeListeners() {
        //seems redundant, but Arrays.asList returns an immutable list throwing errors when removing items manually
        return new ArrayList<>(Arrays.asList(propertyChangeSupport.getPropertyChangeListeners()));
    }
}
