/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.sketchlinks;

import com.google.gson.GsonBuilder;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class SketchMetadataTypeS implements SketchMetadata {
    @NotNull
    private String timestamp = "";
    @NotNull
    private String authors = "";
    @NotNull
    private String annotation = "";
    @NotNull
    private String title = "no title set";


    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        @NotNull SketchMetadataTypeS that = (SketchMetadataTypeS) o;

        if (annotation != null ? !annotation.equals(that.annotation) : that.annotation != null) return false;
        if (authors != null ? !authors.equals(that.authors) : that.authors != null) return false;
        if (timestamp != null ? !timestamp.equals(that.timestamp) : that.timestamp != null) return false;
        if (title != null ? !title.equals(that.title) : that.title != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = timestamp != null ? timestamp.hashCode() : 0;
        result = 31 * result + (authors != null ? authors.hashCode() : 0);
        result = 31 * result + (annotation != null ? annotation.hashCode() : 0);
        result = 31 * result + (title != null ? title.hashCode() : 0);
        return result;
    }

    @NotNull
    public String getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(@NotNull String timestamp) {
        this.timestamp = timestamp;
    }

    @NotNull
    public String getAuthors() {
        return authors;
    }

    public void setAuthors(@NotNull String authors) {
        this.authors = authors;
    }

    @NotNull
    public String getAnnotation() {
        return annotation;
    }

    public void setAnnotation(@NotNull String annotation) {
        this.annotation = annotation;
    }

    @NotNull
    public String getTitle() {
        return title;
    }

    public void setTitle(@NotNull String title) {
        this.title = title;
    }

    public boolean isNeverNull() {
        return title != null && annotation != null && authors != null && timestamp != null;
    }

    @Override
    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(this);
    }


}
