/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

import com.intellij.openapi.components.*;
import com.intellij.openapi.util.IconLoader;
import com.intellij.util.xmlb.XmlSerializerUtil;
import com.intellij.util.xmlb.annotations.Transient;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

/**
 * Save default values for new projects
 */
@State(
        name = slApplicationState.COMPONENT_NAME,
        storages = {
                @Storage(id = "default", file = StoragePathMacros.APP_CONFIG + "/sketchlink2.xml")
        }
)
public class slApplicationState implements PersistentStateComponent<SketchlinkState>, ApplicationComponent {
    public static final String COMPONENT_NAME = "slApplicationState";
    public static final String SKETCHLINK_TAGNAME = "sketchlink"; //the Tag used inside comments
    public static final ResourceBundle RESOURCE_BUNDLE = PropertyResourceBundle.getBundle("plg.SLPlugin");
    public static final int FOLDINGSTYLE_HIDDEN = 0;
    public static final int FOLDINGSTYLE_PREFIX = 1;
    public static final int FOLDINGSTYLE_POSTFIX = 2;
    @NotNull
    static Icon myDefaultIcon = IconLoader.getIcon("/plg/ui/icons/whiteboard_icon.png");
    @NotNull
    static Icon myOverlayIcon = IconLoader.getIcon("/plg/ui/icons/icon14x17_small0.png");

    private SketchlinkState myState;

    public slApplicationState() {
        myState = new SketchlinkState();
    }

    public static slApplicationState getInstance() {
        return ServiceManager.getService(slApplicationState.class);
    }

    @Nullable
    public SketchlinkState getState() {
        return myState;
    }

    /**
     * Sets the current default values to state
     */
    public void loadState(SketchlinkState state) {
        XmlSerializerUtil.copyBean(state, myState);
    }

    @Override
    public void initComponent() {

    }

    @Override
    public void disposeComponent() {

    }

    @NotNull
    @Override
    public String getComponentName() {
        return COMPONENT_NAME;
    }


    //default values


    @NotNull
    @Transient
    public Icon getOverlayIcon() {
        return myOverlayIcon;
    }
}
