/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.RangeMarker;
import com.intellij.openapi.project.Project;
import com.intellij.util.containers.ConcurrentHashSet;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Set;

/**
 * Created by Peter on 10.11.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class slCodeBlockGuard {
    private final Set<RangeMarker> protectedAreas = new ConcurrentHashSet<>(30);
    private final Project project;

    public slCodeBlockGuard(@NotNull Project project) {
        this.project = project;
    }

    public void addGuardedBlock(@Nullable RangeMarker guardedBlock) {
        if (guardedBlock == null) {
            return;
        }
        protectedAreas.add(guardedBlock);
    }

    public void removeGuardedBlocks() {
        if (protectedAreas.isEmpty()) {
            return;
        }
        ApplicationManager.getApplication().runReadAction(new Runnable() {
            @Override
            public void run() {
                for (@NotNull RangeMarker protectedArea : protectedAreas) {
                    if (!protectedArea.isValid()) {
                        continue;
                    }
                    @NotNull Document document = protectedArea.getDocument();
                    document.removeGuardedBlock(protectedArea);
                }
            }
        });
    }

}
