/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

import com.intellij.openapi.components.*;
import com.intellij.openapi.project.Project;
import com.intellij.util.xmlb.XmlSerializerUtil;
import com.intellij.util.xmlb.annotations.Transient;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Created by Peter on 10.11.2014.
 * <p/>
 * Bean class with all settings on a Project level. Important Entries are contained in the SketchlinkState class,
 * which can be saved application wide.
 */
@State(
        name = "plg.slCurrentProjectSettings",
        storages = {
                @Storage(id = "other", file = "$PROJECT_FILE$"),
                @Storage(id = "dir", file = "$PROJECT_CONFIG_DIR$/sketchlink.xml", scheme = StorageScheme.DIRECTORY_BASED)})
public class slCurrentProjectSettings extends SketchlinkState implements PersistentStateComponent<slCurrentProjectSettings> {
    private boolean connected = false;

    /**
     * Default constructor for the PersistentStateComponent.
     * NEEDS TO EXIST
     */
    public slCurrentProjectSettings() {
        loadApplicationDefaults();
    }

    public static slCurrentProjectSettings getInstance(@NotNull Project project) {
        return ServiceManager.getService(project, slCurrentProjectSettings.class);
    }

    public boolean loadApplicationDefaults() {
        SketchlinkState state = slApplicationState.getInstance().getState();
        if (state == null) {
            return false;
        }
        copyFrom(state);
        return true;
    }

    public void saveAsApplicationDefault() {
        SketchlinkState state = slApplicationState.getInstance().getState();
        if (state != null) {
            state.copyFrom(this);
        }
    }

    @Transient
    public boolean isConnected() {
        return connected;
    }

    public void setConnected(boolean connected) {
        this.connected = connected;
    }


    @Nullable
    @Override
    public slCurrentProjectSettings getState() {
        return this;
    }

    @Override
    public void loadState(slCurrentProjectSettings state) {
        XmlSerializerUtil.copyBean(state, this);
    }
}
