/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

import com.intellij.codeInsight.folding.impl.JavaFoldingBuilder;
import com.intellij.lang.ASTNode;
import com.intellij.lang.folding.CustomFoldingBuilder;
import com.intellij.lang.folding.FoldingDescriptor;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.FoldingGroup;
import com.intellij.openapi.editor.RangeMarker;
import com.intellij.openapi.util.TextRange;
import com.intellij.psi.PsiElement;
import com.intellij.psi.javadoc.PsiDocComment;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.psi.util.PsiTreeUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * Created by Peter on 07.02.14.
 */
public class slFoldingBuilder extends CustomFoldingBuilder {
    public static final FoldingGroup myGroup = FoldingGroup.newGroup("sketchlink");

    /**
     * Implement this message to build language folding regions besides custom folding regions.
     *
     * @param descriptors The list of folding descriptors to store results to.
     * @param root        The root node for which the folding is requested.
     * @param document    The document for which folding is built.
     * @param quick       whether the result should be provided as soon as possible without reference resolving
     */
    @Override
    protected void buildLanguageFoldRegions(@NotNull List<FoldingDescriptor> descriptors, @NotNull PsiElement root, @NotNull Document document, boolean quick) {
        @NotNull Collection<PsiDocTag> psiDocTags = PsiTreeUtil.findChildrenOfType(root, PsiDocTag.class);
        slCurrentProjectSettings projectSettings = slCurrentProjectSettings.getInstance(root.getProject());

        if (!projectSettings.isSketchAnchorsProtected()) {
            slProjectComp.getInstance(root.getProject()).getGuardedBlocks().removeGuardedBlocks();
        }
        for (@NotNull PsiDocTag docTag : psiDocTags) {
            @Nullable LocalLinkAnchor anchor = SketchlinkManager.getAnchorOf(docTag);
            if (anchor == null) {
                continue; //no id in doctag
            }
            PsiDocComment docComment = docTag.getContainingComment();
            if (docComment == null || !docComment.isValid()) {
                continue; //the docTag has been replaced and no longer has a parent
            }
            boolean isDefaultLocation = SketchlinkManager.TOOLS.isDefaultDocCommentLocation(docComment);
            @NotNull TextRange foldingRange = anchor.getAnchorRange();
            if (projectSettings.isSketchAnchorsProtected()) {
                @NotNull RangeMarker guardedBlock = document.createGuardedBlock(foldingRange.getStartOffset(), foldingRange.getEndOffset());

                guardedBlock.setGreedyToLeft(false);
                guardedBlock.setGreedyToRight(false);
                slProjectComp.getInstance(root.getProject()).getGuardedBlocks().addGuardedBlock(guardedBlock);
            }

            if (isDefaultLocation) {
                descriptors.add(new FoldingDescriptor(docTag.getNode(), foldingRange, myGroup, Collections.emptySet(), projectSettings.isSketchAnchorAlwaysFolded()));
                // System.out.println("");
            } else {
                //Comment is inside a Method
                if (SketchlinkManager.TOOLS.checkCommentContainsSketchlink(docTag.getContainingComment(), true)) {
                    //is pure sketchlink-comment, fold completely
                    descriptors.add(new FoldingDescriptor(docComment.getNode(), docComment.getTextRange(), myGroup, Collections.emptySet(), projectSettings.isSketchAnchorAlwaysFolded()));
                    //System.out.println("PURE");
                } else {
                    descriptors.add(new FoldingDescriptor(docComment.getNode(), docComment.getTextRange()));
                    //System.out.println("MIXED");

                }
            }

        }

//        Collection<PsiDocComment> psiDocComments = PsiTreeUtil.findChildrenOfType(root, PsiDocComment.class);
//
//        for (PsiDocComment comment : psiDocComments) {
//
//            if (SketchlinkManager.TOOLS.checkCommentContainsSketchlink(comment, false)) {
//                //don't add DocComments describing a Method, even if pure
//                //don't add DocComments who are between Methods, generates Error
//                boolean isDefaultLocation = SketchlinkManager.TOOLS.isDefaultDocCommentLocation(comment);
//                //int usedEndOffset;
//                for (PsiDocTag docTag : SketchlinkManager.TOOLS.getSketchlinkTags(comment)) {
//                    //use the lesser offset between the end of the doctag or the end of line, the doctag started
////                    usedEndOffset = document.getLineEndOffset(document.getLineNumber(docTag.getTextRange().getStartOffset()));
////                    usedEndOffset = (usedEndOffset < docTag.getTextRange().getEndOffset() ? usedEndOffset : docTag.getTextRange().getEndOffset());
//
//                    TextRange foldingRange = SketchlinkManager.TOOLS.getTextRangeOfAnchor(docTag);
//                    if (PROTECT_SKETCHLINKS) {
//                        document.createGuardedBlock(foldingRange.getStartOffset(), foldingRange.getEndOffset());
//                    }
//                    if (isDefaultLocation) {
//                        //TextRange foldingRange = new TextRange(docTag.getTextRange().getStartOffset(), usedEndOffset);
//                        descriptors.add(new FoldingDescriptor(docTag.getNode(), foldingRange, myGroup, Collections.emptySet(), DISALLOW_EXPAND));
//                    }
//                }
//                if (isDefaultLocation) {
//                    continue;
//                }
//                //Comment is inside a Method
//                if (SketchlinkManager.TOOLS.checkCommentContainsSketchlink(comment, true)) {
//                    //is pure sketchlink-comment, fold completely
//                    descriptors.add(new FoldingDescriptor(comment.getNode(), comment.getTextRange(), myGroup, Collections.emptySet(), DISALLOW_EXPAND));
//                } else {
//                    descriptors.add(new FoldingDescriptor(comment.getNode(), comment.getTextRange()));
//                }
//
//
////                for (PsiDocTag docTag : SketchlinkManager.getSketchlinkTags(comment)) {
////                    int lineEndOffset = document.getLineEndOffset(document.getLineNumber(docTag.getTextRange().getStartOffset()));
////
////                    TextRange foldingRange = new TextRange(docTag.getTextRange().getStartOffset(),
////                                            (lineEndOffset < docTag.getTextRange().getEndOffset() ? lineEndOffset : docTag.getTextRange().getEndOffset()));
////                    descriptors.add(new FoldingDescriptor(docTag.getNode(), foldingRange, myGroup, Collections.emptySet(), DISALLOW_EXPAND));
////                }
//
//            }
//        }

        //for (PsiDocTag comment : result_new) {
        //  document.createGuardedBlock(comment.getTextRange().getStartOffset(),comment.getTextRange().getEndOffset());
//
//            /**
//             * TODO: What about comments like these? Inside a Block comment
//             * @sketchlink 123
//             */
//
//            int lineEndOffset = document.getLineEndOffset(document.getLineNumber(comment.getTextRange().getStartOffset()));
//            //.getLineStartOffset(   offsetTo VisualPosition(element.getTextRange().getStartOffset(), editor).line);
//            TextRange foldingRange = new TextRange(comment.getTextRange().getStartOffset(), (lineEndOffset < comment.getTextRange().getEndOffset() ? lineEndOffset : comment.getTextRange().getEndOffset()));
//            descriptors.add(new FoldingDescriptor(comment.getNode(), foldingRange, myGroup, Collections.emptySet(), DISALLOW_EXPAND));
//        }

    }

    @Nullable
    @Override
    protected String getLanguagePlaceholderText(@NotNull ASTNode node, @NotNull TextRange range) {
        PsiElement psiElement = node.getPsi();
        boolean alwaysHide = false;
        if (psiElement instanceof PsiDocComment) {
            if (!SketchlinkManager.TOOLS.checkCommentContainsSketchlink((PsiDocComment) psiElement, true)) {
                return new JavaFoldingBuilder().getPlaceholderText(node, range);
            }
            alwaysHide = !SketchlinkManager.TOOLS.isDefaultDocCommentLocation((PsiDocComment) psiElement);
        }


        String foldedText;

        int foldingStyle = slCurrentProjectSettings.getInstance(psiElement.getProject()).getFoldingStyle();
        if (foldingStyle == slApplicationState.FOLDINGSTYLE_HIDDEN || alwaysHide) {
            foldedText = "  ";
        } else {
            //icon directly in front of tag
            if (foldingStyle == slApplicationState.FOLDINGSTYLE_PREFIX) {
                foldedText = "  @" + slApplicationState.SKETCHLINK_TAGNAME;
            } else {
                //icon aftter the tag, with a whitespace inbetween
                foldedText = "@" + slApplicationState.SKETCHLINK_TAGNAME + " ";
            }
        }

        return foldedText;
        //return "  @sketchlink";
    }

    /**
     * Returns the default collapsed state for the folding region related to the specified node.
     *
     * @param node the node for which the collapsed state is requested.
     * @return true if the region is collapsed by default, false otherwise.
     */
    @Override
    protected boolean isRegionCollapsedByDefault(@NotNull ASTNode node) {
        return true;
    }
}
