/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.components.ServiceManager;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.project.Project;
import com.intellij.util.xmlb.annotations.Transient;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchlinkManager;
import plg.ui.overlay.CodeHighlighter;
import plg.ui.preview.PreviewManager;
import plg.websocket.slWebsocketManager;

import javax.swing.*;

/**
 * Created by Peter on 01.02.14.
 * <p/>
 * Method Frame for saving and configuring project wide settings
 */

public class slProjectComp implements ProjectComponent, Configurable {
    public final static String COMPONENT_NAME = "com.sketchlink.idea.slProjectComp";
    private final Project project;
    private PreviewManager previewManager;
    private CodeHighlighter myCodeHighlighter;
    private slCodeBlockGuard myGuardedBlocks;

    //    private SketchlinkState projectSettings;
    private slCurrentProjectSettings currentState;
    private SLConfigForm form;
//    private boolean currentlyConnected;

    public slProjectComp(Project project) {

        this.project = project;
        previewManager = new PreviewManager(project);
        myCodeHighlighter = new CodeHighlighter(project);
        myGuardedBlocks = new slCodeBlockGuard(project);
//        currentState = new slCurrentProjectSettings();

    }

    public static slProjectComp getInstance(@NotNull Project targetProject) {
        return ServiceManager.getService(targetProject, slProjectComp.class);
    }

    public void initComponent() {
        // TODO: insert component initialization logic here
        currentState = slCurrentProjectSettings.getInstance(project);
    }


    public void disposeComponent() {

    }

    @NotNull
    public String getComponentName() {
        return COMPONENT_NAME;
    }

    public void projectOpened() {
        // called when project is opened
        if (currentState.isConnectOnStartup()) {
            slWebsocketManager.getInstance(project).connect();
        }
    }

    public void projectClosed() {
        slWebsocketManager.getInstance(project).disconnect();
        SketchlinkManager.getInstance(project).clearAllData();
    }


    @NotNull
    @Transient
    public PreviewManager getPreviewManager() {
        return previewManager;
    }

    @Transient
    @NotNull
    public CodeHighlighter getCodeHighlighter() {
        return myCodeHighlighter;
    }

    @NotNull
    @Transient
    public slCodeBlockGuard getGuardedBlocks() {
        return this.myGuardedBlocks;
    }

    @NotNull
    public String getServerURL() {
        return slCurrentProjectSettings.getInstance(project).getServerURL();
    }

    //----Configuration Interface stuff

    @Nls
    @Override
    public String getDisplayName() {
        return "Sketchlink";
    }

    @Nullable
    @Override
    public String getHelpTopic() {
        return null;
    }

    @Nullable
    @Override
    public JComponent createComponent() {
        if (form == null) {
            form = new SLConfigForm(project);
        }
        return form.getRootComponent();
    }

    @Override
    public boolean isModified() {
        return form != null && form.isModified(currentState);
    }

    @Override
    public void apply() throws ConfigurationException {
        if (form != null)
            form.getData(currentState);
    }

    @Override
    public void reset() {
        if (form != null) {
            form.setData(currentState);
        }
    }

    @Override
    public void disposeUIResources() {
        form = null;
    }


    //---Persistence

    /**
     * Tries loading the current default applicationwide settings for this project.
     *
     * @return true, if successful, false otherwise
     */
    public boolean loadDefaultSettings() {
        boolean worked = currentState.loadApplicationDefaults();
        if (form != null) {
            form.setData(currentState);
        }
        return worked;
    }

    /**
     * Saves the current Project settings (server, colors, etc) as the default settings for all new projects.
     */
    public void saveAsDefaultSettings() {
        if (form != null) {
            form.getData(currentState);
        }
        currentState.saveAsApplicationDefault();
    }

    public boolean isConnectedToServer() {
        return slWebsocketManager.getInstance(project).isConnected();
    }

    /**
     * Enables / Disables the Websocket connection.
     */
    public void setConnectedToServer(boolean active) {
        if (active) {
            slWebsocketManager.getInstance(project).connect();
        } else {
            slWebsocketManager.getInstance(project).disconnect();
        }
        slCurrentProjectSettings.getInstance(project).setConnected(active);
        if (form != null) {
            form.setConnected(active);
        }

    }


}
