/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.tools;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.MessageType;
import com.intellij.openapi.ui.popup.Balloon;
import com.intellij.openapi.ui.popup.JBPopupFactory;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.ui.awt.RelativePoint;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Created by Peter on 27.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class slGuiTools {
    public static void ShowNotificationAtBottom(@NotNull final String text, @NotNull Project project) {
        ShowNotificationAtBottom(text, project, MessageType.INFO);
    }

    public static void ShowNotificationAtBottom(@NotNull final String text, @NotNull Project project, @NotNull MessageType type) {
        ShowNotificationAtBottom(text, project, type, 3500);
    }

    public static void ShowNotificationAtBottom(@NotNull final String text, @NotNull final RelativePoint relativePoint, @NotNull final MessageType type, final int fadeOutTime) {
        UIUtil.invokeLaterIfNeeded(new Runnable() {
            @Override
            public void run() {
                JBPopupFactory.getInstance()
                        .createHtmlTextBalloonBuilder(text, type, null)
                        .setFadeoutTime(fadeOutTime)
                        .createBalloon()
                        .show(relativePoint, Balloon.Position.above);
            }
        });
    }

    /**
     * Shows a notification popup at the bottom, that hides again after a certain amount of time.
     *
     * @param text        The text the notification should display, HTML format supported, e.g. <p></p>
     * @param project     The Project this notification should appear at
     * @param type        the Messagetype of this window, e.g. MessageType.INFO
     * @param fadeOutTime the time after which the MessageWindow should vanish, in milliseconds
     */
    public static void ShowNotificationAtBottom(@NotNull final String text, @NotNull Project project, @NotNull final MessageType type, final int fadeOutTime) {
        @Nullable final StatusBar statusBar2 = WindowManager.getInstance()
                .getStatusBar(project);
        if (statusBar2 != null) {
            UIUtil.invokeLaterIfNeeded(new Runnable() {
                @Override
                public void run() {
                    JBPopupFactory.getInstance()
                            .createHtmlTextBalloonBuilder(text, type, null)
                            .setFadeoutTime(fadeOutTime)
                            .createBalloon()
                            .show(RelativePoint.getCenterOf(statusBar2.getComponent()),
                                    Balloon.Position.atRight);
                }
            });
        }
    }
}
