/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package plg.tools;


import com.google.common.base.Strings;
import com.intellij.ide.browsers.BrowserLauncher;
import com.intellij.ide.browsers.WebBrowser;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleUtil;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.ui.MessageType;
import com.intellij.openapi.util.Computable;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.psi.*;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.slCurrentProjectSettings;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.security.MessageDigest;


/**
 * Created by Peter on 22.01.14.
 * Toolbox-Class for checking and extracting information from PsiDocComments.
 */
public class slToolBox {
    private static final Logger LOG = Logger.getInstance(slToolBox.class);

    /**
     * Converts an Icon to an Image
     *
     * @param icon the Icon to be converted
     * @return a image object of the supplied icon
     */
    //based on example code from: https://groups.google.com/forum/#!topic/comp.lang.java.programmer/OI_IdebPL68
    @NotNull
    public static Image iconToImage(@NotNull Icon icon) {
        if (icon instanceof ImageIcon) {
            return ((ImageIcon) icon).getImage();
        } else {
            int w = icon.getIconWidth();
            int h = icon.getIconHeight();
            GraphicsEnvironment ge =
                    GraphicsEnvironment.getLocalGraphicsEnvironment();
            GraphicsDevice gd = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gd.getDefaultConfiguration();
            BufferedImage image = gc.createCompatibleImage(w, h);
            Graphics2D g = image.createGraphics();
            icon.paintIcon(null, g, 0, 0);
            g.dispose();
            return image;
        }
    }


    public static void OpenBrowserTab(@NotNull String url, @NotNull WebBrowser browser) {
        //NOTE: boolean launchbrowser doesn't return if a browser successfully launched, have to check via http-server if opening worked
        BrowserLauncher.getInstance().browse(url, browser);
    }

    public static void OpenBrowserTab(@NotNull String url, @NotNull Project project) {
        WebBrowser browser = slCurrentProjectSettings.getInstance(project).getBrowser();
        if (browser == null) {
            slGuiTools.ShowNotificationAtBottom("No valid default browser set.<br>Please change the browser in Sketchlink\'s configuration.", project, MessageType.ERROR);
            return;
        }
        OpenBrowserTab(url, browser);
    }


    @NotNull
    public static ImageIcon scalePic(@NotNull ImageIcon base, int maxWidth, int maxHeight, boolean upscaleIfPossible, boolean keepAspectRatio) {
        int h = base.getIconHeight();
        int w = base.getIconWidth();
        double scaleX = 1.d * w / maxWidth;
        double scaleY = 1.d * h / maxHeight;
        //resize Picture if necessary,
        //TODO: resize is very slow, needs improved performance!
        //System.out.println("Resized "+maxWidth+" "+maxHeight+" "+w+" "+h+" "+scaleX+" "+scaleY);
        if (((h == maxHeight) && (w == maxWidth)) || (!upscaleIfPossible && keepAspectRatio && (scaleX < 1) && (scaleY < 1))) {
            return base; //nothing to do
        }
        if (!keepAspectRatio) {
            return new ImageIcon(base.getImage().getScaledInstance(maxWidth, maxHeight, Image.SCALE_SMOOTH));
        }

        if (scaleX > 1 || scaleY > 1 || upscaleIfPossible) {
            if (scaleX > scaleY) {
                return new ImageIcon(base.getImage().getScaledInstance(maxWidth, -1, Image.SCALE_SMOOTH));
            } else {
                return new ImageIcon(base.getImage().getScaledInstance(-1, maxHeight, Image.SCALE_SMOOTH));
            }
        }
        return base;
    }

    @NotNull
    public static ImageIcon scalePic(@NotNull ImageIcon base, int maxWidth, int maxHeight, boolean upscaleIfPossible) {
        return scalePic(base, maxWidth, maxHeight, upscaleIfPossible, true);

    }

    /**
     * This Method returns the Type of the generic Parameter T on position pos.
     * <p/>
     * Code example and explanation from http://rgomes-info.blogspot.co.uk/2013/12/using-typetokens-to-retrieve-generic.html
     * retrieved: 18.02.2014
     * <p/>
     * However, original code had two Bugs. Below mine fixed Version.
     * <p/>
     * Important: extendedClass has to be instantiated as an anonymous class if it should work with this Method.
     * Example:
     * correct: List<foo> list = new List<>() {}; //the {} are important
     * wrong: List<foo> list = new List<>();
     *
     * @param extendedClass an anonymous Instance or extended Class of an generic Class
     * @param pos           the position of the Generic to lookup
     * @return the Type of the superclass of klass
     */

    public static Type getType(@NotNull final Class<?> extendedClass, final int pos) {
        // obtain anonymous, if any, class for 'this' instance
        final Type superclass = extendedClass.getGenericSuperclass();
        // test if this class is really an anonymous class or extending a class
        //FIXED: switched true/false
        if (!(superclass instanceof ParameterizedType)) {
            throw new RuntimeException("This instance should belong to an anonymous class.");
        }
        // obtain RTTI of all generic parameters
        final Type[] types = ((ParameterizedType) superclass).getActualTypeArguments();
        // test if enough generic parameters were passed
        //FIXED: error if pos was less than length
        if (pos > types.length - 1) {
            throw new RuntimeException(String.format(
                    "Could not find generic parameter %d because only %d parameters were passed",
                    pos, types.length));
        }
        // return the type descriptor of the requested generic parameter
        return types[pos];
    }

    /**
     * Calculates the optimal Size of a Picture using <code>prefSize</code> to fit into an Object with
     * maximum Sizes of maxWidth and maxHeight, while keeping the Aspect ratio intact.
     */
    @NotNull
    public static Dimension calculatePictureSize(@NotNull Dimension prefSize, int maxWidth, int maxHeight) {
        float scaleX = 1.f * maxWidth / prefSize.width;
        float scaleY = 1.f * maxHeight / prefSize.height;
        if (scaleX < scaleY) {

            return new Dimension(maxWidth, Math.round(prefSize.height * scaleX));
        }
        return new Dimension(Math.round(prefSize.width * scaleY), maxHeight);
    }


    /**
     * Calculates the MD5 hash for the given base-String.
     *
     * @param base
     * @return the md5 hash for the string or "0" if calculation wasn't possible
     */
    @Contract("null -> null")
    @Nullable
    public static String calculateMD5(@Nullable String base) {
        if (base == null)
            return null;

//        String digest = null;
//        try {
//            MessageDigest md = MessageDigest.getInstance("MD5");
//            byte[] hash = md.digest(base.getBytes("UTF-8"));
//
//            //converting byte array to Hexadecimal String
//            StringBuilder sb = new StringBuilder(2*hash.length);
//            for(byte b : hash){
//                sb.append(String.format("%02x", b&0xff));
//            }
//            digest = sb.toString();
//        } catch (UnsupportedEncodingException | NoSuchAlgorithmException ex) {
//            LOG.error(ex);
//        }
//        return digest;

        //had to be changed because IntelliJ removed Apache Commons Library in 13.1
        MessageDigest digest = DigestUtils.getMd5Digest();
        digest.reset();
        byte[] md5;
        try {
            md5 = digest.digest(base.getBytes("UTF-8"));
        } catch (UnsupportedEncodingException e) {
            LOG.error(e);
            return "0";
        }
        return new String(Hex.encodeHex(md5));
    }

    /**
     * Computes the relative Filepath to the module root for the given PsiElement.
     *
     * @return a Stringrepresentation of the relative Filepath, normalized to forward slash,
     * null if either the path for the psiElement or the containing Module path doesn't exist
     */
    @Nullable
    public static String getRelativePath(@NotNull final PsiElement psiElement) {
        @NotNull Computable<String> result = new Computable<String>() {
            @Nullable
            @Override
            public String compute() {
                @NotNull Project project = psiElement.getProject();
                @Nullable Module module = ModuleUtil.findModuleForPsiElement(psiElement.getContainingFile());
                @NotNull StringBuilder path = new StringBuilder("");
                path.append(psiElement.getContainingFile().getVirtualFile().getPresentableUrl());
                if (module == null || path.toString().equals("null")) {
                    return null;
                }
                for (int i = 0; i < path.length(); i++) {
                    if (path.charAt(i) == '\\') {
                        path.setCharAt(i, '/');
                    }
                }
                ModuleRootManager rootManager = ModuleRootManager.getInstance(module);
                //don't use this, causes exception on shutdown
                //ModifiableRootModel modifiableModel = rootManager.getModifiableModel();
                @NotNull VirtualFile[] contentRoots = rootManager.getSourceRoots(true);
                int length = -1;
                @NotNull String usedPrefixPath = "";
                for (@NotNull VirtualFile compare : contentRoots) {
                    @Nullable PsiDirectory directory = PsiManager.getInstance(project).findDirectory(compare);
                    if (directory == null) {
                        LOG.debug("A PsiDirectory for a found Module Source root is empty.\nPlease notify the Developer with your Module configuration");
                        continue;
                    }
                    @Nullable PsiPackage psiPackage = JavaDirectoryService.getInstance().getPackage(directory);
                    @NotNull StringBuilder directoryPackageNameToPath = new StringBuilder("");
                    if (psiPackage != null) {
                        directoryPackageNameToPath.append("/");
                        directoryPackageNameToPath.append(psiPackage.getQualifiedName().replace('.', '/'));
                    }
                    @NotNull StringBuilder path2 = new StringBuilder(compare.getPresentableUrl());
                    if (path2.toString().equals("null")) {
                        continue;
                    }
                    for (int i = 0; i < path2.length(); i++) {
                        if (path2.charAt(i) == '\\') {
                            path2.setCharAt(i, '/');
                        }
                    }
                    path2.append(directoryPackageNameToPath);
                    //path is dependent on local file system, switch backslashes to slashes for uniformity
                    //path2 = path2.replace('\\', '/') + directoryPackageNameToPath;
                    int differentPosition = Strings.commonPrefix(path.toString(), path2.toString()).length();
                    if (differentPosition > length) {
                        length = differentPosition;
                        usedPrefixPath = directoryPackageNameToPath.toString();
                    }
                }
                if (length > 0) {
                    path.replace(0, length, "");
                    path.insert(0, usedPrefixPath.length() > 1 ? "/" : "");
                    path.insert(0, usedPrefixPath);
                    //path = usedPrefixPath + (usedPrefixPath.length() > 1 ? "/" : "") + path.substring(length);
                }
                //path normally starts with a '/'
                //TODO: maybe remove '/' if path starts with it
                return path.toString();
            }
        };
        return ApplicationManager.getApplication().runReadAction(result);
    }

//    private static int commonPrefixLength(@NotNull String s1, @NotNull String s2) {
//        for (int i = 0;i<s1.length();i++) {
//            if (i >= s2.length() || s1.charAt(i) != s2.charAt(i)) {
//                return i;
//            }
//        }
//        return s1.length();
//    }
}
