/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.overlay;

import com.intellij.ide.DataManager;
import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.ui.components.JBList;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Created by Peter on 26.03.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class RightClickContextAction {
    private static final RightClickContextAction instance = new RightClickContextAction();
    @NotNull
    private final MouseMotionListener myMouseMotionListener;
    @NotNull
    private final MouseListener myMouseListener;

    private RightClickContextAction() {
        myMouseMotionListener = new MouseMotionAdapter() {
            @Override
            public void mouseMoved(@NotNull MouseEvent e) {
                @NotNull Point pointMouse = e.getPoint();
                if (!(e.getComponent() instanceof JBList)) {
                    return;
                }
                @NotNull JBList jBList = (JBList) e.getComponent();
                int newIndex = jBList.locationToIndex(pointMouse);
                if (newIndex == jBList.getSelectedIndex()) {
                    return;
                }
                jBList.clearSelection();
                jBList.setSelectedIndex(newIndex);
            }
        };

        myMouseListener = new MouseAdapter() {
            @Override
            public void mouseClicked(@NotNull MouseEvent e) {
                if (!(e.getComponent() instanceof JBList)) {
                    return;
                }
                @NotNull JBList list = (JBList) e.getComponent();
                Object object = list.getSelectedValue();
                if (!(object instanceof AnAction)) {
                    return;
                }
                @NotNull AnAction action = (AnAction) object;
                @NotNull AnActionEvent event = new AnActionEvent(e,
                        DataManager.getInstance().getDataContext(e.getComponent()),
                        ActionPlaces.EDITOR_POPUP,
                        action.getTemplatePresentation(),
                        ActionManager.getInstance(),
                        e.getModifiers());
                action.actionPerformed(event);
            }

            @Override
            public void mouseEntered(@NotNull MouseEvent e) {
                if (!(e.getComponent() instanceof JBList)) {
                    return;
                }
                @NotNull JBList list = (JBList) e.getComponent();
                @NotNull Point pointMouse = e.getPoint();
                int newIndex = list.locationToIndex(pointMouse);
                list.setSelectedIndex(newIndex);
            }
        };
    }

    @NotNull
    public static RightClickContextAction getInstance() {
        return instance;
    }

    public void show(@NotNull JComponent parentComponent) {
//        System.out.println("RightClickContextAction#show");
        ActionGroup group;
        if (ActionManager.getInstance().isGroup("Sketchlink.OverlayContext")) { //NON-NLS
            group = (ActionGroup) ActionManager.getInstance().getAction("Sketchlink.OverlayContext");
        } else {
            Logger.getInstance(RightClickContextAction.class)
                    .warn("Expected Actiongroup for Sketchlink.OverlayContext, but received only normal action");
            return;
        }
        ActionPopupMenu actionPopupMenu = ActionManager.getInstance().createActionPopupMenu(ActionPlaces.EDITOR_POPUP, group);
        actionPopupMenu.getComponent().show(parentComponent, parentComponent.getWidth(), 0);
    }
}
