/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.overlay;

import com.intellij.psi.PsiElement;
import com.intellij.psi.javadoc.PsiDocComment;
import com.intellij.psi.javadoc.PsiDocTag;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.overlay.OverlayElementImpl;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;

import javax.swing.*;

/**
 * Created by Peter on 06.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class SketchlinkOverlayElementImpl extends OverlayElementImpl<PsiElement> {

    @NotNull
    private LocalLinkAnchor localAnchor;
    private LocalLinkAnchor positionAnchor;


    public SketchlinkOverlayElementImpl(@NotNull LocalLinkAnchor localAnchor, @NotNull PsiElement positionElement, @NotNull ImageIcon icon, boolean allowLineWrapsForPosition) {
        super(positionElement, icon, allowLineWrapsForPosition);
        this.localAnchor = localAnchor;
        this.positionAnchor = localAnchor;
    }

    public SketchlinkOverlayElementImpl(@NotNull LocalLinkAnchor localAnchor, @NotNull PsiElement positionElement, @NotNull ImageIcon icon, boolean allowLineWrapsForPosition, @NotNull LocalLinkAnchor positionAnchor) {
        super(positionElement, icon, allowLineWrapsForPosition);
        this.localAnchor = localAnchor;
        this.positionAnchor = positionAnchor;


    }



    @Nullable
    @Override
    public PsiElement getPositionalElement() {
        boolean insideMethod = false;
        PsiDocTag anchorTag = positionAnchor.getAnchorTag();
        if (anchorTag == null) {
            return null;
        }
        PsiDocComment containingComment = anchorTag.getContainingComment();
        if (containingComment != null) {
            insideMethod = SketchlinkManager.TOOLS.checkCommentContainsSketchlink(containingComment, true)
                    && !SketchlinkManager.TOOLS.isDefaultDocCommentLocation(containingComment);
        }
        return insideMethod ? containingComment : anchorTag;
    }

    @NotNull
    public LocalLinkAnchor getLocalAnchor() {
        return localAnchor;
    }

    public void setLocalAnchor(@NotNull LocalLinkAnchor anchor) {
        if (localAnchor.getDocument() != anchor.getDocument()) {
            return;
        }
        localAnchor = anchor;

    }

    public void setAnchorForPosition(@NotNull LocalLinkAnchor anchor) {
        if (positionAnchor.getDocument() != anchor.getDocument()) {
            return;
        }
        positionAnchor = anchor;
    }

    /**
     * Gets the ImageIcon that will be drawn. Might be resized earlier.
     */
    @NotNull
    @Override
    public ImageIcon getImageIcon() {
        if (super.getImageIcon() != getLocalAnchor().getImageIcon() && localAnchor == positionAnchor) {
            setImageIcon(getLocalAnchor().getImageIcon());
        }
        return super.getImageIcon();
    }

    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;

        @NotNull SketchlinkOverlayElementImpl that = (SketchlinkOverlayElementImpl) o;

        if (!localAnchor.equals(that.localAnchor)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + localAnchor.hashCode();
        return result;
    }
}
