/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.overlay;

import com.intellij.psi.PsiElement;
import com.intellij.psi.javadoc.PsiDocComment;
import com.intellij.psi.javadoc.PsiDocTag;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.overlay.IconAlignment;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.SketchlinkManager;
import plg.slApplicationState;
import plg.slCurrentProjectSettings;
import plg.ui.preview.PreviewManager;

import javax.swing.*;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by Peter on 05.07.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class slOverlayElementBuilder {
    private SketchlinkOverlayElementImpl myOverlayElement;


    public slOverlayElementBuilder(@NotNull final LocalLinkAnchor highlightAnchor, @NotNull PsiElement positionElement, @NotNull ImageIcon icon, @NotNull LocalLinkAnchor positionAnchor) {
        myOverlayElement = new SketchlinkOverlayElementImpl(highlightAnchor, positionElement, icon, false, positionAnchor);
        @NotNull DragSourceAdapter dragSourceAdapter = createDragSourceListener();
        @NotNull DragGestureListener myDragGestureListener = createDragGestureListener(highlightAnchor, dragSourceAdapter);
        @NotNull TransferHandler myTransferhandler = createTransferhander(highlightAnchor);
        @NotNull MouseListener myMouseListener = createMouseListener(highlightAnchor);

        myOverlayElement.setDragsourceAdapter(dragSourceAdapter);
        myOverlayElement.setDragGestureListener(myDragGestureListener);
        myOverlayElement.setTransferHandler(myTransferhandler);
        myOverlayElement.setMouseListener(myMouseListener);

        @Nullable PsiDocTag anchorTag = positionAnchor.getAnchorTag();
        boolean insideMethod = false;
        if (anchorTag != null) {
            PsiDocComment containingComment = anchorTag.getContainingComment();
            if (containingComment != null) {
                insideMethod = SketchlinkManager.TOOLS.checkCommentContainsSketchlink(containingComment, true)
                        && !SketchlinkManager.TOOLS.isDefaultDocCommentLocation(containingComment);
            }
        }

        if (slCurrentProjectSettings.getInstance(positionElement.getProject()).getFoldingStyle() == slApplicationState.FOLDINGSTYLE_POSTFIX && !insideMethod) {
            myOverlayElement.setIconAlignment(IconAlignment.OUTER_RIGHT);
        }

    }

    public slOverlayElementBuilder(@NotNull final LocalLinkAnchor anchor, @NotNull PsiElement anchorElement) {
        myOverlayElement = new SketchlinkOverlayElementImpl(anchor, anchorElement, anchor.getImageIcon(), false);
        @NotNull DragSourceAdapter dragSourceAdapter = createDragSourceListener();
        @NotNull DragGestureListener myDragGestureListener = createDragGestureListener(anchor, dragSourceAdapter);
        @NotNull TransferHandler myTransferhandler = createTransferhander(anchor);
        @NotNull MouseListener myMouseListener = createMouseListener(anchor);

        myOverlayElement.setDragsourceAdapter(dragSourceAdapter);
        myOverlayElement.setDragGestureListener(myDragGestureListener);
        myOverlayElement.setTransferHandler(myTransferhandler);
        myOverlayElement.setMouseListener(myMouseListener);
        @Nullable PsiDocTag anchorTag = anchor.getAnchorTag();
        boolean insideMethod = false;
        if (anchorTag != null) {
            PsiDocComment containingComment = anchorTag.getContainingComment();
            if (containingComment != null) {
                insideMethod = SketchlinkManager.TOOLS.checkCommentContainsSketchlink(containingComment, true)
                        && !SketchlinkManager.TOOLS.isDefaultDocCommentLocation(containingComment);
            }
        }

        if (slCurrentProjectSettings.getInstance(anchorElement.getProject()).getFoldingStyle() == slApplicationState.FOLDINGSTYLE_POSTFIX && !insideMethod) {
            myOverlayElement.setIconAlignment(IconAlignment.OUTER_RIGHT);
        }

    }

    @NotNull
    private static DragSourceAdapter createDragSourceListener() {
        return new DragSourceAdapter() {
            //nothing?
        };
    }

    @NotNull
    private static DragGestureListener createDragGestureListener(
            @NotNull final LinkAnchor anchor, @NotNull final DragSourceListener dragSourceListener) {
        return new DragGestureListener() {
            @Override
            public void dragGestureRecognized(@NotNull DragGestureEvent dge) {
                dge.getDragSource().startDrag(dge,
                        DragSource.DefaultLinkDrop,
                        new SketchAnchorTransferable(anchor),
                        dragSourceListener);
            }
        };
    }

    @NotNull
    private MouseListener createMouseListener(@NotNull final LocalLinkAnchor anchor) {
        return new MouseAdapter() {
            @Override
            public void mouseEntered(MouseEvent e) {
                highlightCode(anchor);
                //maybe update all linked Elements
                //SketchlinkManager.getInstance().lookupAnchor(anchor,null);

            }

            @Override
            public void mouseExited(MouseEvent e) {
                unHighlightCode(anchor);
            }

            @Override
            public void mouseClicked(@NotNull MouseEvent e) {
                if (!(e.getComponent() instanceof JComponent)) {
                    return;
                }
                @NotNull JComponent component = (JComponent) e.getComponent();
                if (!component.isEnabled()) {
                    return;
                }
                if (e.getButton() == MouseEvent.BUTTON1) {
//                    PreviewHandler.getInstance(anchor.getProject()).showPopupList(anchor, component);
                    PreviewManager.getInstance(anchor.getProject()).showPreview(anchor, component);
                }
                if (e.getButton() == MouseEvent.BUTTON3) { //right click
                    RightClickContextAction.getInstance().show(component);
                }
            }
        };
    }

    private void unHighlightCode(@NotNull LocalLinkAnchor localAnchor) {
        CodeHighlighter.getInstance(localAnchor.getProject()).unmarkAllRanges();
    }

    private void highlightCode(@NotNull LocalLinkAnchor localAnchor) {
        @NotNull Set<LocalLinkAnchor> hashSet = new HashSet<>();
        hashSet.add(localAnchor);
        CodeHighlighter.getInstance(localAnchor.getProject()).markCode(hashSet);
    }

    @NotNull
    private TransferHandler createTransferhander(@NotNull final LinkAnchor anchor) {
        return new TransferHandler("sketchAnchor") {
            @Override
            public boolean importData(@NotNull TransferSupport support) {
                if (!support.isDataFlavorSupported(SketchAnchorTransferable.SketchFlavor)) {
                    System.out.println("Sketchflavor on importData not supported");
                    return false;
                }
                LinkAnchor data;
                try {
                    data = (LinkAnchor) support.getTransferable()
                            .getTransferData(SketchAnchorTransferable.SketchFlavor);
                } catch (@NotNull UnsupportedFlavorException | IOException e) {
                    e.printStackTrace();
                    return false;
                }
                System.out.println("Got a drop event!\n\tTarget: "
                        + anchor.getAnchorId()
                        + "\n\tDropped: " + data.getAnchorId());
                SketchlinkManager.getInstance(anchor.getProject()).addLink(anchor, data, false, true);
                return true;
            }

            @Override
            public boolean canImport(@NotNull TransferSupport support) {
                return support.isDataFlavorSupported(SketchAnchorTransferable.SketchFlavor);
            }
        };
    }

    public SketchlinkOverlayElementImpl build() {
        return myOverlayElement;
    }
}
