/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import org.jetbrains.annotations.NotNull;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.RemoteLinkAnchorC;
import plg.sketchlinks.RemoteLinkAnchorM;
import plg.sketchlinks.RemoteLinkAnchorS;

/**
 * Created by Peter on 05.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class AvailableViewImpl implements AvailableViews {
    private String identifier;
    private String type;
    private String displayName;
    private String ipAddress;

    public AvailableViewImpl(@NotNull String displayName, @NotNull String type, @NotNull String identifier) {
        this.displayName = displayName;
        this.type = type;
        this.identifier = identifier;

    }


    @NotNull
    @Override
    public String getDisplayName() {
        return displayName;
    }

    @Override
    public void setDisplayName(@NotNull String displayName) {
        this.displayName = displayName;
    }

    @NotNull
    @Override
    public String getType() {
        return type;
    }

    @Override
    public void setType(@NotNull String type) {
        this.type = type;
    }

    @NotNull
    @Override
    public String getIdentifier() {
        return identifier;
    }

    @Override
    public void setIdentifier(@NotNull String identifier) {
        this.identifier = identifier;
    }

    @Override
    public boolean isWebview() {
        return "webview".equals(type); //NON-NLS NON-NLS
    }

    @Override
    public void addViewableArtifactID(@NotNull LinkAnchor anchor) {
        @NotNull LinkAnchor linkAnchor = anchor;
        if (anchor instanceof RemoteLinkAnchorM) {
            linkAnchor = ((RemoteLinkAnchorM) anchor).getParentAnchor();
        }

    }

    @Override
    public void removeViewableArtifactID(@NotNull LinkAnchor anchor) {
        @NotNull LinkAnchor linkAnchor = anchor;
        if (anchor instanceof RemoteLinkAnchorM) {
            linkAnchor = ((RemoteLinkAnchorM) anchor).getParentAnchor();
        }

    }

    @Override
    public boolean canShow(@NotNull LinkAnchor anchor) {
        @NotNull LinkAnchor linkAnchor = anchor;
        if (anchor instanceof RemoteLinkAnchorM) {
            linkAnchor = ((RemoteLinkAnchorM) anchor).getParentAnchor();
        }
        if (linkAnchor instanceof RemoteLinkAnchorS && "webview".equals(getType())) { //NON-NLS
            return true;
        }
        if (linkAnchor instanceof RemoteLinkAnchorC && "ide".equals(getType())) { //NON-NLS NON-NLS
            return true;
        }
        return false;
    }

    @Override
    public String getIpAddress() {
        return ipAddress;
    }

    @Override
    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    @NotNull
    private String makeWhiteSpace(int count) {
        @NotNull StringBuilder sb = new StringBuilder(" ");
        for (int i = 0; i < count; i++) {
            sb.append(" ");
        }
        return sb.toString();
    }

    @NotNull
    @Override
    public String toString() {
        return displayName + (ipAddress != null ? makeWhiteSpace(15 - displayName.length()) + "\t[" + ipAddress + "]" : "");
    }


}
