/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.ui.JBColor;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.font.TextAttribute;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;

/**
 * Created by Peter on 09.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class EmptyPreviewOptions {
    private JLabel uploadLabel;
    private JLabel showRecentSketchesLabel;
    private JPanel rootPanel;

    public EmptyPreviewOptions(@NotNull final PreviewManager previewManager) {
        setJLabelUnderlined(uploadLabel);
        uploadLabel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        setJLabelUnderlined(showRecentSketchesLabel);
        showRecentSketchesLabel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));


        showRecentSketchesLabel.addMouseListener(new MouseAdapter() {
            Color oldColor;

            @Override
            public void mouseClicked(MouseEvent e) {
                previewManager.showUploadedSketches();
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                oldColor = showRecentSketchesLabel.getForeground();
                showRecentSketchesLabel.setForeground(JBColor.BLUE);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                if (oldColor != null) {
                    showRecentSketchesLabel.setForeground(oldColor);
                }
            }
        });

        uploadLabel.addMouseListener(new MouseAdapter() {
            Color oldColor;

            @Override
            public void mouseClicked(MouseEvent e) {
                try {
                    previewManager.showUploadSketchWindow(null, null);
                } catch (ExecutionException | InterruptedException ignored) {

                }
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                oldColor = uploadLabel.getForeground();
                uploadLabel.setForeground(JBColor.BLUE);
            }

            @Override
            public void mouseExited(MouseEvent e) {
                if (oldColor != null) {
                    uploadLabel.setForeground(oldColor);
                }
            }
        });
    }

    public static void setJLabelUnderlined(@NotNull JLabel label) {
        Font font = label.getFont();
        Map<TextAttribute, Object> map =
                new HashMap<TextAttribute, Object>(font.getAttributes());
        map.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
        label.setFont(font.deriveFont(map));
    }

    public JPanel getRootPanel() {
        return rootPanel;
    }
}
