/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.openapi.editor.Document;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.popup.JBPopupFactory;
import com.intellij.openapi.ui.popup.ListPopup;
import com.intellij.openapi.ui.popup.PopupStep;
import com.intellij.openapi.ui.popup.util.BaseListPopupStep;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.util.TextRange;
import com.intellij.psi.PsiElement;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.ui.awt.RelativePoint;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.overlay.OverlayElement;
import plg.sketchlinks.*;
import plg.slCurrentProjectSettings;
import plg.tools.slToolBox;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import javax.swing.*;
import java.beans.PropertyChangeListener;
import java.util.*;

/**
 * Created by Peter on 28.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class LocalNavigationPopup {

    private final Project project;
    private final RelativePoint location;
    private final RemoteLinkAnchorS sketchAnchor;

    public LocalNavigationPopup(@NotNull Project project,
                                @NotNull RelativePoint location,
                                @NotNull RemoteLinkAnchorS sketchAnchor,
                                @NotNull Collection<LocalLinkAnchor> targets,
                                @NotNull Collection<LinkAnchor> sourceOfTargets) {
        this.project = project;
        this.location = location;
        this.sketchAnchor = sketchAnchor;


        List<LocalLinkAnchor> showList = new ArrayList<>(targets);
        Collections.sort(showList);
        if (slCurrentProjectSettings.getInstance(project).isShowFindForNavigationAtTop()) {
            showList.add(0, new MyFakeLocalLinkAnchor(project, sourceOfTargets));
        } else {
            showList.add(new MyFakeLocalLinkAnchor(project, sourceOfTargets));
        }

        MyListPopupStep myListPopupStep = new MyListPopupStep(null, showList);
        ListPopup popup = JBPopupFactory.getInstance().createListPopup(myListPopupStep);
        popup.setRequestFocus(true);
        popup.show(location);


    }


    private class MyListPopupStep extends BaseListPopupStep<LocalLinkAnchor> {

        private MyListPopupStep(@Nullable String aTitle, LocalLinkAnchor[] aValues) {
            super(aTitle, aValues);
        }

        private MyListPopupStep(@Nullable String aTitle, java.util.List<? extends LocalLinkAnchor> aValues) {
            super(aTitle, aValues);
        }

        private MyListPopupStep(@Nullable String aTitle, LocalLinkAnchor[] aValues, Icon[] aIcons) {
            super(aTitle, aValues, aIcons);
        }

        private MyListPopupStep(@Nullable String aTitle, @NotNull List<? extends LocalLinkAnchor> aValues, Icon aSameIcon) {
            super(aTitle, aValues, aSameIcon);
        }

        private MyListPopupStep(@Nullable String aTitle, @NotNull List<? extends LocalLinkAnchor> aValues, List<Icon> aIcons) {
            super(aTitle, aValues, aIcons);
        }

        private MyListPopupStep() {
        }

        @Override
        public PopupStep onChosen(LocalLinkAnchor selectedValue, boolean finalChoice) {
            selectedValue.navigateTo();
            if (selectedValue instanceof MyFakeLocalLinkAnchor) {
                PreviewManager.getInstance(project).getPreviewHandler().setHighlighting(new HashSet<String>()); //disable highlighting

            } else {
                PreviewManager.getInstance(project).getPreviewHandler().SetContentFromAnchor(selectedValue, sketchAnchor.getPreviewItem());
            }
//            System.out.println("BaseListPopupStep: "+selectedValue.getDescription()+" "+finalChoice);
            return FINAL_CHOICE;
        }

        @Override
        public Icon getIconFor(LocalLinkAnchor aValue) {
            return aValue.getImageIcon();
        }

        @NotNull
        @Override
        public String getTextFor(LocalLinkAnchor value) {
            String fileText = " in ";
            PsiDocTag anchorTag = value.getAnchorTag();
            if (anchorTag != null) {
                fileText += anchorTag.getContainingFile().getName();
            } else {
                fileText = "";
            }
            return value.getDescription() + fileText;
        }
    }

    private class MyFakeLocalLinkAnchor implements LocalLinkAnchor {
        private final Project project;
        private ImageIcon icon = new ImageIcon(slToolBox.iconToImage(IconLoader.getIcon("/plg/ui/icons/icon_overlayhide.png")));
        private Collection<LinkAnchor> selectedMarkers;

        public MyFakeLocalLinkAnchor(@NotNull Project project, @NotNull Collection<LinkAnchor> selectedMarkers) {
            this.project = project;
            this.selectedMarkers = selectedMarkers;

        }

        @Nullable
        @Override
        public PsiDocTag getAnchorTag() {
            return null;
        }

        @NotNull
        @Override
        public TextRange getAnchorRange() {
            return new TextRange(0, 0);
        }

        @NotNull
        @Override
        public TextRange getTargetRange() {
            return new TextRange(0, 0);
        }

        @Nullable
        @Override
        public Document getDocument() {
            return null;
        }

        @NotNull
        @Override
        public String getSavedMD5() {
            return "";
        }

        @Override
        public void updateMD5() {

        }

        @Override
        public void updateAnchorTag(@NotNull PsiDocTag newAnchor) {

        }

        @Nullable
        @Override
        public PsiElement getTargetElement() {
            return null;
        }

        @Override
        public boolean isTargetUnchanged(boolean update) {
            return true;
        }

        @Override
        public boolean isValid() {
            return false;
        }

        @Override
        public void invalidate() {

        }

        @Override
        public boolean isAnchorEqualTo(@NotNull PsiDocTag docTag) {
            return false;
        }

        @Nullable
        @Override
        public SketchMetadataTypeC getCurrentMetaData() {
            return null;
        }

        @NotNull
        @Override
        public OverlayElement getOverlayElement() {
            throw new NotImplementedException();
        }

        @Nullable
        @Override
        public SketchMetadataTypeC isCurrentMetaDataUpdated() {
            return null;
        }

        @NotNull
        @Override
        public Project getProject() {
            return project;
        }

        @NotNull
        @Override
        public String getAnchorId() {
            throw new NotImplementedException();

        }

        @Override
        public void setAnchorId(@NotNull String id) {
            throw new NotImplementedException();
        }

        @NotNull
        @Override
        public String getDescription() {
            return "Show in Find View";
        }

        @NotNull
        @Override
        public AnchorTypes getAnchorType() {
            throw new NotImplementedException();
        }

        @NotNull
        @Override
        public ImageIcon getImageIcon() {
            return icon;
        }

        @Override
        public void navigateTo() {

//            System.out.println("Navigation");
            Set<LocalLinkAnchor> localLinkAnchors = SketchlinkManager.getInstance(project).browseToLocalAnchorsLinkedTo(selectedMarkers, null, null);

        }

        @Nullable
        @Override
        public LinkAnchor getParent() {
            return null;
        }

        @Nullable
        @Override
        public SketchMetadataTypeC getMetadata() {
            return null;
        }

        @Override
        public void setMetadata(@NotNull SketchMetadataTypeC metadata) {

        }

        @Override
        public void addPropertyChangeListener(@NotNull PropertyChangeListener listener) {

        }

        @Override
        public void removePropertyChangeListener(@NotNull PropertyChangeListener listener) {

        }

        @NotNull
        @Override
        public List<PropertyChangeListener> getPropertyChangeListeners() {
            throw new NotImplementedException();
        }

        @Override
        public int compareTo(LocalLinkAnchor o) {
            return -1;
        }
    }
}
