/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.ui.awt.RelativePoint;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;

import java.util.List;
import java.util.Set;

/**
 * Created by Peter on 14.09.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public interface PreviewHandler {


    public abstract void copyFrom(@NotNull PreviewHandler handler);

    public void addContent(@NotNull LinkAnchor anchor);

    /**
     * A list of currently previewed LinkAnchors in their current order.
     */
    @NotNull
    public List<LinkAnchor> getContent();

    public void setContent(@NotNull List<LinkAnchor> anchors);

    @Nullable
    public PreviewItem getPinnedContent();

    /**
     * Sets the PreviewItem pinned at the top.
     * If the PreviewHandler supports it, the pinnedItem stays visible at the top as long
     * it is not set to null.
     * If set to null, it will only be removed on the next call of setContent.
     *
     * @param pinnedItem the PreviewItem that should be always visible at the top
     */
    public void setPinnedContent(@Nullable PreviewItem pinnedItem);

    public void addHighlighting(@NotNull String id);

    @NotNull
    public Set<String> getHighlighting();

    /**
     * Set the ids to "linked" highlighting.
     */
    public void setHighlighting(@NotNull Set<String> linkedIDs);

    /**
     * Sets the highlighting to all ids linked to the following anchor, only uses not-deprecated ids.
     * Unsets possible pinned items.
     */
    public void SetContentFromAnchor(@NotNull LinkAnchor source, @Nullable PreviewItem pinnedItemAtTop);


    @NotNull
    public String getTitle();

    /**
     * If supported, sets the title of this PreviewHandler.
     * Currently, only shown as part of the Toolwindow.
     */
    public void setTitle(@NotNull String title);

    @Nullable
    public LinkAnchor getContentAnchor();

    @NotNull
    public RelativePoint getRelativeLocation();

    public void setRelativeLocation(@NotNull RelativePoint point);

    public boolean isVisible();

    //    public void showPreview(@NotNull LinkAnchor source,@NotNull Collection<PreviewableAnchor> linkedAnchors,@NotNull JComponent parentLocation);
    public void setVisible(boolean showing);

    public void unregister();

}
