/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.util.containers.ConcurrentHashSet;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.RemoteLinkAnchorM;
import plg.sketchlinks.SketchlinkManager;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * Created by Peter on 07.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public abstract class PreviewHandlerBase implements PreviewHandler {

    @Nullable
    protected PreviewItem myPinnedAnchor;
    @Nullable
    private LinkAnchor highlightingSource = null;

    /**
     * Sets the content to all LinkAnchors linked to the source anchor,
     * updates highlighting and replaces all previous manual settings.
     */
    @Override
    public void SetContentFromAnchor(@NotNull LinkAnchor source, @Nullable PreviewItem pinnedAtTop) {
        List<LinkAnchor> list = SketchlinkManager.getInstance(source.getProject()).getLinksNotDeprecated(source);
        @NotNull Set<LinkAnchor> content = new ConcurrentHashSet<>(list.size());
        @NotNull Set<String> highlight = new ConcurrentHashSet<>(list.size());

        //use parent s-anchors instead of marker anchors.
        for (LinkAnchor anchor : list) {
            if (anchor instanceof RemoteLinkAnchorM) {
                content.add(((RemoteLinkAnchorM) anchor).getParentAnchor());
            } else {
                content.add(anchor);
            }
            highlight.add(anchor.getAnchorId());
        }
        @NotNull List<LinkAnchor> contentList = new ArrayList<>(content);
        Collections.sort(contentList, new SketchAnchorComparable());
        setPinnedContent(pinnedAtTop);
        setContent(contentList);
        setHighlighting(highlight);
        setTitle("Linked Sketches for " + source.getDescription());
        this.highlightingSource = source;
    }

    @Nullable
    @Override
    public LinkAnchor getContentAnchor() {
        return highlightingSource;
    }


    @Override
    public void copyFrom(@NotNull PreviewHandler handler) {
        if (handler.getContentAnchor() != null) {
            SetContentFromAnchor(handler.getContentAnchor(), handler.getPinnedContent());
        }
        setPinnedContent(handler.getPinnedContent());
        setContent(handler.getContent());
        setHighlighting(handler.getHighlighting());
        setRelativeLocation(handler.getRelativeLocation());
        setTitle(getTitle());
    }

    @Override
    @Nullable
    public PreviewItem getPinnedContent() {
        return myPinnedAnchor;
    }

    @Override
    public void setPinnedContent(@Nullable PreviewItem pinnedItem) {
        myPinnedAnchor = pinnedItem;
    }
}
