/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.openapi.actionSystem.DataKey;
import com.intellij.openapi.actionSystem.DataProvider;
import org.jetbrains.annotations.NotNull;
import plg.sketchlinks.LinkAnchor;

import javax.swing.*;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Set;

/**
 * Created by Peter on 13.03.14.
 * Part of Sketchlink Plugin for IntelliJ
 */
public interface PreviewItem extends DataProvider {
    public static final DataKey<PreviewItem> DATA_KEY = DataKey.create("sketchlink.previewItem");

    //    /**
//     * Used to detect which Anchor is currently active, markers linked to this Anchor could be highlighted.
//     *
//     * @return the referenced anchor, if available
//     */
//    @Nullable
//    LinkAnchor getReferencedAnchor();
//    public static final String REFERENCED_ANCHOR_PROPERTY = "referencedAnchor";
    public static final String HIGHLIGHTED_IDS_PROPERTY = "highlightedIDs";


    //    /**
//     * Set the referenced anchor that decides highlighting.
//     */
//    void setReferencedAnchor(@NotNull LinkAnchor anchor);
    public static final String TIME_OF_LAST_ACCESS_PROPERTY = "timeOfLastAccess";

    /**
     * Gets the highlighted ids that were set previously.
     */
    @NotNull
    public Set<String> getHighlightedIDs();

    /**
     * Tells the PreviewItem to highlight the following ids.
     * Using an empty set disables highlighting.
     */
    public void setHighlightedIDs(@NotNull Set<String> ids);

    /**
     * Gets the IDs that are highlighted in this item, returns a subset of <code>getHighlightedIDs()</code>
     */
    @NotNull
    public Set<String> getHighlightedIDsInThisItem();

    /**
     * The Anchor of the current Sketch
     */
    @NotNull
    public LinkAnchor getAnchorOfSketch();

    /**
     * the actual preview
     */
    @NotNull
    public JPanel getPanel();

    /**
     * Updates the URL to load it's data from
     *
     * @param url the new URL location
     */
    public void setRemoteLocation(@NotNull URL url);

    /**
     * Returns the time in System.currentTimeMillis for the last access to getPanel.
     */
    public long getTimeOfLastAccess();

    /**
     * Removes cached data
     */
    public void clear();

    public void addPropertyChangeListener(PropertyChangeListener listener);

    public void removePropertyChangeListener(PropertyChangeListener listener);

    /**
     * Sets the width of the preview.
     * It should calculate the height automatically based on the aspect ratio.
     * If the scrollBarWidth is >0, then the scrollbar is visible and will hide the [scrollBarwWdth] rightmost pixel of the panel.
     * The complete_width includes the scrollbarwidth, so the actual width for content might be smaller.
     * ScrollbarWidth should only be considered when toolWindowMode is set to true.
     */
    public void setWidth(int complete_width, int scrollBarWidth);

    /**
     * If true, the PreviewItem is shown inside a toolwindow, which might have a scrollbar on the right side.
     * False, hide the scrollbar.
     */
    void setToolWindowMode(boolean active);

    /**
     * True: If possible, allow the user to edit the content of this PreviewItem.
     * False: Default mode, only highlighting.
     */
    void setEditMode(boolean active);
}
