/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.*;

import java.util.Comparator;

/**
 * Created by Peter on 06.10.2014.
 * Imposes an ordering of anchors for preview purposes. Newer sketches first
 */
public class SketchAnchorComparable implements Comparator<LinkAnchor> {

    @Nullable
    private String getMetadataDate(@NotNull LinkAnchor anchorIn) {
        @NotNull LinkAnchor anchor = anchorIn;
        //use date of the sketch for marker anchors
        if (anchor instanceof RemoteLinkAnchorM) {
            anchor = ((RemoteLinkAnchorM) anchor).getParentAnchor();
        }
        @Nullable SketchMetadata metadata = null;
        if (anchor instanceof MetadataOwner) {
            metadata = ((MetadataOwner) anchor).getMetadata();
        }
        if (metadata == null)
            return null; //either metadata not availabe or anchor has no metadata

        if (metadata instanceof SketchMetadataTypeS) {
            return ((SketchMetadataTypeS) metadata).getTimestamp();
        }
        return null;
    }

    /**
     * Use the project name and the path for source code anchors (local and remote).
     * Sort code anchors after sketch anchors by prefixing them with 0's
     */
    @NotNull
    private String getPathMetadataDate(@NotNull LinkAnchor anchor) {
        @Nullable SketchMetadata metadata = null;
        if (anchor instanceof LocalLinkAnchor) {
            return "0000" + ((LocalLinkAnchor) anchor).getCurrentMetaData().getProjectName() + "/" + ((LocalLinkAnchor) anchor).getCurrentMetaData().getPath();
        }
        if (anchor instanceof MetadataOwner) {
            metadata = ((MetadataOwner) anchor).getMetadata();
        }
        if (metadata instanceof SketchMetadataTypeC) {
            return "0000" + ((SketchMetadataTypeC) metadata).getProjectName() + "/" + ((SketchMetadataTypeC) metadata).getPath();
        }
        //either metadata not availabe or anchor has no metadata
        return "";
    }


    /**
     * Sort anchors, based on creation date (for sketches, descending order=newest first),
     * code anchors are last, sorted on project name, the path to their file and their id
     */
    @Override
    public int compare(@NotNull LinkAnchor o1, @NotNull LinkAnchor o2) {
        if (o1.equals(o2))
            return 0;
        @Nullable String s1 = getMetadataDate(o1);
        @Nullable String s2 = getMetadataDate(o2);
        if (s1 == null) {
            s1 = getPathMetadataDate(o1) + o1.getAnchorId();
        }
        if (s2 == null) {
            s2 = getPathMetadataDate(o2) + o2.getAnchorId();
        }
        return s2.compareTo(s1);
    }
}
