/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview;

import com.intellij.openapi.actionSystem.ActionGroup;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.ActionPlaces;
import com.intellij.openapi.actionSystem.ActionPopupMenu;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.*;
import plg.ui.preview.svgwrapper.SVGPanelExt;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.HashSet;
import java.util.List;

/**
 * Created by Peter on 19.09.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class SketchEditMouseListener implements MouseListener, MouseMotionListener {

    @NotNull
    private final Project project;

    public SketchEditMouseListener(@NotNull Project project) {
        this.project = project;
    }

    @Override
    public void mouseClicked(@NotNull MouseEvent e) {
        @Nullable final SVGPanelExt panel = getPanel(e.getComponent());
        if (panel == null) {
            return;
        }
        if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() >= 2) {
            @Nullable String currentID = panel.getSVGPreviewContainer().getCurrentID();
            if (currentID == null)
                return;
            @Nullable LinkAnchor knownAnchor = SketchlinkManager.getInstance(project).getKnownAnchor(currentID);
            if (knownAnchor instanceof RemoteLinkAnchorS) {
                AnnotationEditDialog.createAndShow((RemoteLinkAnchorS) knownAnchor);
            } else {
                if (knownAnchor instanceof RemoteLinkAnchorM) {
                    AnnotationEditDialog.createAndShow((RemoteLinkAnchorM) knownAnchor);
                }
            }
            return;
        }

        if (e.getButton() != MouseEvent.BUTTON3) {
            return;
        }

        ActionGroup group;
        if (ActionManager.getInstance().isGroup("Sketchlink.SketchEditContext")) { //NON-NLS
            group = (ActionGroup) ActionManager.getInstance().getAction("Sketchlink.SketchEditContext");
        } else {
            Logger.getInstance(SketchEditMouseListener.class)
                    .warn("Expected Actiongroup for Sketchlink.SVGDeleteMarker, but received only normal action");
            return;
        }
        ActionPopupMenu actionPopupMenu = ActionManager.getInstance().createActionPopupMenu(ActionPlaces.UNKNOWN, group);
        actionPopupMenu.getComponent().show(panel, e.getPoint().x, e.getPoint().y);

    }


    @Override
    public void mousePressed(@NotNull MouseEvent e) {
        @Nullable SVGPanelExt panel = getPanel(e.getComponent());
        if (panel == null) {
            return;
        }
        if (e.getButton() == MouseEvent.BUTTON1) {
            panel.lockAction(e.getPoint());
        }

//        if (e.getButton() != MouseEvent.BUTTON1) {
//            return;
//        }
//        synchronized (this) {
//                lastPoint = e.getPoint();
//        }
//        Component component = e.getComponent();
//        if (!(component instanceof SVGPanelExt)) {
//            return;
//        }
//        movingMarker = null;
//        SVGPanelExt svgPanelExt = (SVGPanelExt)component;
//        for (String id : svgPanelExt.getShapeElementIDsAt(e.getPoint())) {
//            if (id.startsWith("m-")) {
//                movingMarker = id;
//                return;
//            }
//        }
    }

    private void createNewEntry(@Nullable String id, @NotNull SVGPanelExt panel) {
        if (id == null)
            return;
        //create entry in database
        @NotNull SketchMetadataTypeM meta = new SketchMetadataTypeM();
        meta.setParentID(panel.getDiagramEx().getSketchID());
        meta.setAnnotation("Created with Sketchlink plugin");
        @Nullable LinkAnchor knownAnchor = SketchlinkManager.getInstance(project).getKnownAnchor(panel.getDiagramEx().getSketchID());
        if (knownAnchor instanceof RemoteLinkAnchorS) {
            SketchlinkManager.getInstance(project).createMarker(id, (RemoteLinkAnchorS) knownAnchor, meta, true);
        }
    }

    @Override
    public void mouseReleased(@NotNull MouseEvent e) {
        unsetAction(e);
    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(@NotNull MouseEvent e) {
        unsetAction(e);
    }

    private void unsetAction(@NotNull MouseEvent e) {
        @Nullable SVGPanelExt panel = getPanel(e.getComponent());
        if (panel == null) {
            return;
        }
        if (e.getButton() == MouseEvent.BUTTON1) {
            String id = panel.unlockAction();
            if (id != null) {
                createNewEntry(id, panel);
            }
        }
        panel.setCursor(Cursor.getDefaultCursor());
    }

    @Override
    public void mouseDragged(@NotNull MouseEvent e) {

        @Nullable SVGPanelExt panel = getPanel(e.getComponent());
        if (panel == null) {
            return;
        }
        if (panel.isActionLocked()) {
            panel.action(e.getPoint());
        } else {
            panel.setCursor(Cursor.getDefaultCursor());
        }


//        if (!(e.getComponent().contains(e.getPoint()))) {
//            mouseExited(e);
//            return;
//        }
//        synchronized (this) {
//            if (movingMarker==null || lastPoint == null) {
//                return;
//            }
//            Component component = e.getComponent();
//            if (!(component instanceof SVGPanelExt)) {
//                return;
//            }
//            SVGPanelExt svgPanelExt = (SVGPanelExt) component;
//            Point point = e.getPoint();
//            Point delta = new Point(point.x - lastPoint.x,point.y - lastPoint.y);
//            svgPanelExt.moveRectangle(movingMarker,delta);
//            lastPoint = point;
//        }
    }

    @Override
    public void mouseMoved(@NotNull MouseEvent e) {
        @Nullable SVGPanelExt panel;
        if ((panel = getPanel(e.getComponent())) == null) {
            return;
        }
        @NotNull List<String> list = panel.getShapeTargetIDsAtPoint(e.getPoint());
//        List<ShapeElement> list = panel.highlightMarkerAtPoint(e.getPoint());
        @NotNull HashSet<String> set = new HashSet<>();
        if (list.size() > 1) {
            for (@NotNull String s : list) {
                if (!s.startsWith("s-"))
                    set.add(s);
            }
        } else {
            set.addAll(list);
        }

//        System.out.println("Edit: shapeIDS "+list.size());
        panel.setHighlightedIDs(set);

        switch (panel.getActionForPoint(e.getPoint())) {
            case MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                break;
            case TOP_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
                break;
            case LEFT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
                break;
            case RIGHT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
                break;
            case BOTTOM_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
                break;
            case TOPLEFT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
                break;
            case TOPRIGHT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
                break;
            case BOTTOMLEFT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
                break;
            case BOTTOMRIGHT_MOVE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
                break;
            case ROTATE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR)); //nothing better found yet
                break;
            case NONE:
                panel.setCursor(Cursor.getDefaultCursor());
                break;
            case CREATE:
                panel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        }

    }

    @Nullable
    private SVGPanelExt getPanel(@NotNull Component component) {
        if (component instanceof SVGPanelExt) {
            return (SVGPanelExt) component;
        }
        return null;
    }

}
