/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.preview.svgwrapper;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.project.Project;
import com.kitfox.svg.SVGElementException;
import com.kitfox.svg.ShapeElement;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchlinkManager;

import java.util.List;

/**
 * Created by Peter on 22.09.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class SVGDeleteMarker extends AnAction {

    @Override
    public void update(@NotNull AnActionEvent e) {
        @Nullable SVGPanelExt data = e.getData(SVGPanelExt.DATA_KEY);
        @Nullable Project project = e.getProject();
        if (data == null || project == null) {
            return;
        }
        @NotNull List<ShapeElement> shapeElementsAt = data.getShapeElementsAt(data.getLastPoint());
        boolean hasMarkers = false;
        for (@NotNull ShapeElement shapeElement : shapeElementsAt) {
            if (shapeElement.getId() != null && shapeElement.getId().startsWith("m-")) {
                hasMarkers = true;
                break;
            }
        }
        e.getPresentation().setEnabled(hasMarkers);
    }

    @Override
    public void actionPerformed(@NotNull AnActionEvent e) {
        @Nullable SVGPanelExt data = e.getData(SVGPanelExt.DATA_KEY);
        @Nullable Project project = e.getProject();
        if (data == null || project == null) {

            return;
        }
//        System.out.println("Last point: "+data.getLastPoint());
//        System.out.println("Sketch size: "+data.getDiagramEx().getSize());
        @NotNull List<ShapeElement> shapeElementsAt = data.getShapeElementsAt(data.getLastPoint());
        for (@NotNull ShapeElement shapeElement : shapeElementsAt) {
//            System.out.println("ID at point: "+shapeElement.getId());
            if (shapeElement.getId() == null || shapeElement.getId().startsWith("s-"))
                continue;
            try {
                data.getDiagramEx().removeShapeElement(shapeElement, false);
                SketchlinkManager.getInstance(project).deleteAnchor(shapeElement.getId(), false);
            } catch (SVGElementException e1) {
                e1.printStackTrace();
            }

        }


    }
}
