/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.sketchupload;

import com.intellij.ui.JBColor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;

/**
 * Created by Peter on 28.09.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class SingleSketchUpload {
    private JPanel rootPanel;
    private JTextField titleField;
    private JTextArea annotationField;
    private JTextField authorField;
    private JLabel sketchPreview;
    //    private JLabel sketchLocation;
    private JTextField fileLocationTextField;
    private JButton selectFileButton;
    private JLabel titleLabel;
    private JLabel annotationlabel;
    private JLabel authorLabel;
    private ImageIcon selectedImage;
    private String filePath;
    private File selectedFile;
    @NotNull
    private PropertyChangeSupport propertyChangeSupport =
            new PropertyChangeSupport(this);
    private boolean lastValid;
    private JBColor warnColor = new JBColor(new Color(53, 30, 0), new Color(255, 180, 0));


    public SingleSketchUpload() {
        authorField.setText(System.getProperty("user.name", ""));
        titleField.getDocument().addDocumentListener(createDocumentListener(titleField, titleLabel));
        authorField.getDocument().addDocumentListener(createDocumentListener(authorField, authorLabel));
        sketchPreview.setMaximumSize(new Dimension(100, 100));
        //System.out.println("property: "+user+" - "+System.getenv("USERNAME")+" - "+new com.sun.security.auth.module.NTSystem().getName());
        sketchPreview.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                showFileChooser();
            }
        });
        selectFileButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                showFileChooser();
            }
        });
    }

    private void showFileChooser() {
        String text = fileLocationTextField.getText();
        if (text == null || text.trim().isEmpty())
            text = System.getProperty("user.home");
        System.out.println("Using directory: " + text);
        @NotNull JFileChooser openChoser = new JFileChooser(text);
        @NotNull FileFilter imageFileFilter = new FileFilter() {
            @Override
            public boolean accept(@NotNull File f) {
                return f.canRead() && (f.getName().toLowerCase().endsWith(".png") || f.getName().toLowerCase().endsWith(".jpg") || f.getName().toLowerCase().endsWith(".jpeg") || f.isDirectory());
            }

            @NotNull
            @Override
            public String getDescription() {
                return "*.jpg | *.png image files";
            }
        };
//                @NotNull FileFilter pngFileFilter = new FileFilter() {
//                    @Override
//                    public boolean accept(@NotNull File file) {
//                        return file.canRead() && file.getName().toLowerCase().endsWith(".png");
//                    }
//
//                    @NotNull
//                    @Override
//                    public String getDescription() {
//                        return "*.png image files";
//                    }
//                };
//                openChoser.addChoosableFileFilter(imageFileFilter);
//                openChoser.addChoosableFileFilter(pngFileFilter);
        openChoser.setFileFilter(imageFileFilter);

        openChoser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        int result = openChoser.showOpenDialog(rootPanel);
        if (result == JFileChooser.CANCEL_OPTION) {
//                    System.out.println("FileChooser: Canceled");
            return;
        }
        if (result != JFileChooser.APPROVE_OPTION) {
//                    System.out.println("Not Approve: " + result);
            return;
        }
        File selectedFile = openChoser.getSelectedFile();
        if (selectedFile == null) {
//                    System.out.println("No File selected");
            return;
        }
        setSelectedFile(selectedFile);
    }

    public void setSelectedFile(@NotNull File selectedFile) {
        if (this.selectedFile == selectedFile) {
            return;
        }
        File oldValue = this.selectedFile;
        this.selectedFile = selectedFile;
//        sketchLocation.setText(selectedFile.getName());
        fileLocationTextField.setText(selectedFile.getAbsolutePath());
        propertyChangeSupport.firePropertyChange("file", oldValue, selectedFile);
        setSelectedImage(new ImageIcon(selectedFile.getAbsolutePath()));
        checkValid();
    }

    @Nullable
    public ImageIcon getSelectedImage() {
        return selectedImage;
    }

    public void setSelectedImage(@Nullable ImageIcon image) {
        ImageIcon oldValue = selectedImage;
        selectedImage = image;
        if (selectedImage != null) {
            int width = Math.round(100.f * selectedImage.getIconWidth() / selectedImage.getIconHeight());
            //maximum size: as large as the titleField, minimum size: 50px. height constant at 100px
            width = titleField.getWidth() < width ? titleField.getWidth() : width < 50 ? 50 : width;
            sketchPreview.setIcon(new ImageIcon(selectedImage.getImage().getScaledInstance(width, 100, Image.SCALE_SMOOTH)));
        } else {
            sketchPreview.setIcon(new ImageIcon(this.getClass().getResource("emptyPreview.png")));
        }
        propertyChangeSupport.firePropertyChange("image", oldValue, selectedImage);
        checkValid();
    }


    @NotNull
    private DocumentListener createDocumentListener(@NotNull final JTextField comp, @NotNull final JLabel colorLabel) {
        return new DocumentListener() {
            @Nullable
            private Color original = null;

            @NotNull
            public DocumentListener runOnce() {
                changed();
                return this;
            }

            @Override
            public void insertUpdate(DocumentEvent e) {
                changed();
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                changed();
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                changed();
            }

            private void changed() {
                checkValid();
//                if (comp.getText().trim().isEmpty()) {
//                    if (original == null)
//                        original = colorLabel.getForeground();
//                    colorLabel.setForeground(warnColor);
//                } else {
//                    if (original != null) {
//                        colorLabel.setForeground(original);
//                        original = null;
//                    }
//                }
            }
        }.runOnce();
    }

    public String getTitle() {
        return titleField.getText();
    }

    public String getAuthors() {
        return authorField.getText();
    }

    public String getAnnotation() {
        return annotationField.getText();
    }

    public JPanel getRootPanel() {
        return rootPanel;
    }

    private void checkValid() {
        boolean valid = isValid();
        if (lastValid == valid) {
            return;
        }
        lastValid = valid;
        propertyChangeSupport.firePropertyChange("valid", !valid, valid);

    }

    public boolean isValid() {
        return !(selectedImage == null || titleField.getText().trim().isEmpty() || authorField.getText().trim().isEmpty());
    }

    public String getFilePath() {
        return filePath;

    }

    public File getFile() {
        return selectedFile;
    }


    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }


    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }
}
