/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.sketchupload;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.slProjectComp;
import plg.tools.SketchlinkServerRESTUtil;
import plg.ui.preview.svgwrapper.SVGDiagram2XML;
import plg.ui.preview.svgwrapper.SVGPanelExt;
import plg.websocket.slWebsocketManager;

import java.io.IOException;

/**
 * Created by Peter on 23.09.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class UploadCurrentSVGAction extends AnAction {

    @Override
    public void update(@NotNull AnActionEvent e) {
        Project project = e.getProject();
        if (project == null)
            return;
        boolean connected = slWebsocketManager.getInstance(project).isConnected();
        e.getPresentation().setEnabled(connected);
    }

    @Override
    public void actionPerformed(@NotNull AnActionEvent e) {
        @Nullable SVGPanelExt data = e.getData(SVGPanelExt.DATA_KEY);
        Project project = e.getProject();
        if (data == null || project == null) {
            return;
        }
//        System.out.println("Recovered XML file:\n-------------------");
        @Nullable String exportSVG = SVGDiagram2XML.export(data.getDiagramEx());
//        System.out.println(exportSVG);
        if (exportSVG == null) {
//            System.out.println("No output, export SVG failed");
            return;
        }
        String serverURL = slProjectComp.getInstance(project).getServerURL();
        try {
            SketchlinkServerRESTUtil.updateSVG(serverURL, data.getDiagramEx().getSketchID(), exportSVG, null);
        } catch (IOException e1) {
            e1.printStackTrace();
        }

    }
}
