/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.ui.sketchupload;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.overlay.OverlayComponent;
import plg.overlay.OverlayElement;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.RemoteLinkAnchorS;
import plg.ui.overlay.SketchlinkOverlayElementImpl;
import plg.ui.preview.PreviewManager;
import plg.websocket.slWebsocketManager;

import java.util.concurrent.ExecutionException;

/**
 * Created by Peter on 06.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class UploadNewSketchAction extends AnAction {

    @Override
    public void update(AnActionEvent e) {
        OverlayElement data = e.getData(OverlayElement.OVERLAY_ELEMENT_DATA_KEY);
        Project project = e.getProject();
        boolean setEnabled = true;
        if (data != null) {
            e.getPresentation().setText("Import and Link Sketch from File");
            if (project != null && !slWebsocketManager.getInstance(project).getSendQueue().isSendDirect()) {
                setEnabled = false;
            }
        } else {
            e.getPresentation().setText("Import Sketch from File");
        }
        if (project == null)
            return;
        boolean connected = slWebsocketManager.getInstance(project).isConnected();
        setEnabled = setEnabled && connected;

        e.getPresentation().setEnabled(setEnabled);
        e.getPresentation().setVisible(true);
    }

    public void actionPerformed(@NotNull AnActionEvent e) {
        // TODO: insert action logic here
        @Nullable Project project = e.getProject();
        if (project == null)
            return;

        @Nullable OverlayElement overlayElement = e.getData(OverlayElement.OVERLAY_ELEMENT_DATA_KEY);


        @Nullable LocalLinkAnchor linkWithID = null;
        if (overlayElement instanceof SketchlinkOverlayElementImpl) {
            linkWithID = ((SketchlinkOverlayElementImpl) overlayElement).getLocalAnchor();
        }

        PreviewManager previewManager = PreviewManager.getInstance(project);
        try {
            RemoteLinkAnchorS remoteLinkAnchorS = previewManager.showUploadSketchWindow(e.getData(OverlayComponent.DATA_KEY), linkWithID);
            if (remoteLinkAnchorS != null) {
                remoteLinkAnchorS.preparePreview();
            } else {
                return;
            }
        } catch (ExecutionException | InterruptedException e1) {
            e1.printStackTrace();
            return;
        }
        previewManager.showUploadedSketches();

//        @NotNull SketchUpload sketchUpload = new SketchUpload(project, serverURL, linkWithID);
//        sketchUpload.pack();
//        sketchUpload.setSize(500, 400);
//        @Nullable OverlayComponent overlayComponent = e.getData(OverlayComponent.DATA_KEY);
//        if (overlayComponent != null) {
//            sketchUpload.setLocationRelativeTo(overlayComponent);
//        } else {
//            @NotNull Window activeWindow = UIUtil.getActiveWindow();
//            Dimension size = activeWindow.getSize();
//            Point onScreen = activeWindow.getLocationOnScreen();
//            onScreen.x += size.width / 2;
//            onScreen.y += size.height / 2;
//            onScreen.x -= onScreen.x < 250 ? 0 : 250;
//            onScreen.y -= onScreen.y < 200 ? 0 : 200;
//            sketchUpload.updateLocationAndSize(onScreen);
//        }
//        sketchUpload.run();
//        @Nullable String result = null;
//        try {
//            result = sketchUpload.get();
//        } catch (@NotNull InterruptedException | ExecutionException e1) {
//            e1.printStackTrace();
//            return;
//        } finally {
//            sketchUpload.dispose();
//        }
//        if (result == null || result.isEmpty()) {
//            return;
//        }
//        PreviewManager.getInstance(project).setToolwindowMode(true);
//        @NotNull LinkAnchor linkAnchor = SketchlinkManager.getInstance(project).addRemote(result, null, project);
//        @NotNull PreviewHandler previewHandler = PreviewManager.getInstance(project).getPreviewHandler();
//        @NotNull List<LinkAnchor> list = new ArrayList<>();
//        list.add(linkAnchor);
//        previewHandler.setContent(list);
//        previewHandler.setVisible(true);

    }
}
