/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.group.ChannelGroup;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collection;
import java.util.List;

/**
 * Created by Peter on 22.02.14.
 */
public abstract class ChannelManager {
    @NonNls
    public static final String DEFAULT_GROUP = "default"; //NON-NLS
    public static final int DEFAULT_PORT = 8080;

    /**
     * Broadcasts the supplied Text-Data to all connected Clients inside the supplied Groupnames.
     * Each Channel only broadcasts the message once, even if part of several groups.
     *
     * @param textData the Data to Broadcast
     * @param groups   the Name of the Groups who to broadcast on
     */
    public abstract void broadcastText(@NotNull String textData, String... groups);

    /**
     * Returns the Channelgroup with the supplied name or creates one if none existed previously.
     *
     * @param name the name of the Channelgroup
     * @return the Group with the supplied name
     */
    @NotNull
    public abstract ChannelGroup getChannelGroup(@NotNull String name);

    /**
     * Tries to remove the Channelgroup with <code>name</code>. Only the Group is removed, the
     * containing Channels are not changed in any way and are still accessible through <ref>getAllChannels</ref>.
     *
     * @param name identifier of the group to remove
     */
    public abstract void removeChannelGroup(@NotNull String name);

    @Nullable
    public abstract SketchlinkChannel getSketchlinkChannel(@NotNull Channel channel);

    @NotNull
    public abstract ChannelGroup getDefaultGroup();


    @NotNull
    public abstract Collection<ChannelGroup> getChannelGroups();

    public abstract void addChannelToGroup(@NotNull SketchlinkChannel channel, @NotNull String groupname);

    public abstract void removeChannelFromGroup(@NotNull SketchlinkChannel channel, @NotNull String groupname);


    public abstract void broadcastDisconnectRequest(int statuscode, @Nullable String reason);

    public abstract void closeAll(int statuscode, @Nullable String reason);

    public abstract void close(@NotNull Channel channel, int statuscode, @Nullable String reason);

    public abstract void close(@NotNull SketchlinkChannel channel, int statuscode, @Nullable String reason);

    public abstract void close(@NotNull Channel channel);

    public abstract SketchlinkChannel createChannel(@NotNull URI uri, @NotNull String description);

    @NotNull
    public abstract List<SketchlinkChannel> getChannelByDescription(@NotNull String description);

    @NotNull
    public abstract List<SketchlinkChannel> getSketchlinkChannelByTarget(@NotNull URI uri);

    @NotNull
    public abstract List<SketchlinkChannel> getAllChannels();

    @NotNull
    protected abstract Bootstrap getBootstrab();


    public abstract void addChannelToDefaultGroup(@NotNull SketchlinkChannel channel);

    public abstract void connectToDefault() throws URISyntaxException;
}
