/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket;

import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * Created by Peter on 18.02.14.
 */
public abstract class CommandsManager {
    @NotNull
    private final slWebsocketManager myManager;

    public CommandsManager(@NotNull slWebsocketManager websocketManager) {
        this.myManager = websocketManager;
    }

    /**
     * Looks up the Project the supplied RunCmd is registered to.
     *
     * @param runCmd the RunCmd to look up
     * @return the Project used with the RunCmd, null if not found
     */
    @Nullable
    public static Project getProjectForCommand(@NotNull RunCmd runCmd) {
        for (@NotNull Project project1 : ProjectManager.getInstance().getOpenProjects()) {
            @NotNull CommandsManager mgr = slWebsocketManager.getInstance(project1).getCommandsManager();
            @Nullable RunCmd compare = mgr.getCommand(runCmd.getCommandIdentifier());
            if (runCmd.equals(compare)) {
                //we found the place where our runCmd is registered to.
                //There should only be one place, so return the first
                return project1;
            }
        }
        return null;
    }

//    public static void returnErrorMessage(@NotNull String errorType, @NotNull String payload, @NotNull Channel channel) {
//
//        Gson json = new GsonBuilder().serializeNulls().create();
//        String[] errordata = new String[2];
//        errordata[0] = errorType;
//        errordata[1] = payload;
//        String[] data = new String[2];
//        data[0] = "sketchlink.error";
//        data[1] = json.toJson(errordata);
//        channel.writeAndFlush(new TextWebSocketFrame(json.toJson(data)));
//    }

//    @NotNull
//    public static CommandsManager getInstance(@NotNull Project project) {
//        return slWebsocketManager.getInstance(project).getCommandsManager();
//    }

    @NotNull
    public Project getProject() {
        return myManager.getProject();
    }

    @NotNull
    public slWebsocketManager getWebsocketManager() {
        return myManager;
    }


//    public static CommandsManager getInstance() {
//        return ServiceManager.getService(CommandsManager.class);
//    }

//    /**
//     * Registers a new Command in this Manager.
//     *
//     * @param runCmd the Command to register
//     * @param commandIdentifier the command name to register under, is compared to the "message" field of incoming websocket messages
//     * @return true if registration was successful,
//     * false if this command is already known.
//     */
//     abstract boolean registerCmd(@NotNull RunCmd runCmd,@NotNull String commandIdentifier);


    @Nullable
    public abstract RunCmd getCommand(@NotNull String commandIdentifier);

    /**
     * Return a list of all known commands.
     *
     * @return a not-null list of registered commands.
     */
    @NotNull
    public abstract List<String> getAllKnownCommandNames();

    @NotNull
    public abstract List<RunCmd> getAllKnownCommands();


    /**
     * Executes the command <code>commandIdentifier</code> with <code>parameters</code> as input.
     * Does nothing if commandIdentifier is unknown.
     *
     * @param toplevel       the command to run
     * @param inboundChannel the channel the command was received from
     */
    public abstract void executeCommand(@NotNull CommandTopParameterReceive toplevel, @NotNull Channel inboundChannel, @NotNull String receivedCommand);

    /**
     * Executes the command contained in the Text from the TextWebsocket.
     *
     * @param receivedCommand the received TextWebSocketFrame of a Sketchlink Session
     * @param inboundChannel  the channel the command was received from
     */

    public abstract void executeCommand(@NotNull String receivedCommand, @NotNull Channel inboundChannel);

    @Nullable
    public abstract RunCmd unregisterCmd(@NotNull String commandIdentifier);


    /**
     * Gets the registered command with this identifier.
     * If it does not exist, the command is instantiated, initialized and registered.
     *
     * @param cmdClass
     */
    @NotNull
    public abstract RunCmd getCommandOrInstantiate(@NotNull Class<? extends RunCmd> cmdClass) throws IllegalAccessException, InstantiationException;
}
