/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket;


import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.UUID;

/**
 * The basic Interface all Sketchlink-commands have to implement.
 *
 * @see RunCmdStub if you don't want to convert the JSON formatted parameters yourself.
 */
public interface RunCmd {


    public void init(@NotNull CommandsManager manager);

    /**
     * Executes the current command
     *
     * @param params         The parameters needed to execute this command in JSon Format
     * @param inboundChannel the Channel this Command was received from, may be null if message is called manually
     */
    public void executeCmd(@NotNull CommandTopParameterReceive params, @NotNull Channel inboundChannel, @NotNull String receivedCommand);

    /**
     * if CommandIdentifier is the same as the incoming command, run <code>executeCommand</code> with it's data
     *
     * @return the CommandIdentifier of this Method
     */
    @NotNull
    @NonNls
    public String getCommandIdentifier();


    public void sendRequest(@Nullable Payload data, @Nullable Channel channel, @Nullable UUID uuid, boolean sendAsBroadCast);

    /**
     * Sends a response payload with the correct MessageID and type. If channel is null, should Broadcast the date to all clients.
     */
    public void sendResponse(@NotNull Payload data, @Nullable Channel channel, @NotNull UUID uuid);


}
