/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket;

import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.net.URI;
import java.net.URL;

/**
 * Created by Peter on 22.02.14.
 */
public interface SketchlinkChannel {

    @Nullable
    public Channel getChannel();

    /**
     * Sends the textFrameData over this Channel as a TextWebMessage.
     *
     * @param textFrameData the String to send
     */
    public boolean send(@NotNull String textFrameData);

    /**
     * Gets this channels Manager
     *
     * @return the Channelmanager this Channel is registered to.
     */
    @Nullable
    public ChannelManager getManager();


    /**
     * Sends a Close request and waits for the endpoint to acknowledge this close
     *
     * @param statuscode the statuscode to send, as per RFC 6455
     * @param reason     an optional readable Reason text
     */
    public void requestClosing(int statuscode, @Nullable String reason);

    /**
     * Notifies the endpoint and then closes this connection without waiting for an answer.
     *
     * @param statuscode the statuscode to send, as per RFC 6455
     * @param reason     an optional readable Reason text
     */
    public void forceClosing(int statuscode, @Nullable String reason);

    /**
     * Checks if this channel is able to send or receive Data.
     *
     * @return false if this channel has been closed
     */
    public boolean isOpen();

    /**
     * Returns a descriptive Name of this channel.
     *
     * @return a String representation of this Name
     */
    @NotNull
    public String getName();

    public boolean isConnectedTo(@NotNull URI uri);

    public boolean isConnectedTo(@NotNull String uri);

    @NotNull
    public URI getEndpointUri();

    @Nullable
    public URL getEndpointAsHTTP();

    @NotNull
    public String getMyName();

    void open() throws Exception;

    //void run() throws Exception;
}
