/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.EmptyPayload;
import plg.websocket.commands.parameter.LinkRequestParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 05.07.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class CreateLinkCMD extends RunCmdStub<LinkRequestParameter, EmptyPayload> {
    public final static String COMMAND = "createLink";

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    private static class requestWrap implements PayloadWithData<LinkRequestParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public LinkRequestParameter data;

        @Override
        public LinkRequestParameter getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<EmptyPayload> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public EmptyPayload data;

        public EmptyPayload getData() {
            return this.data;
        }
    }

}
