/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.*;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.ArtifactIDParameter;
import plg.websocket.commands.parameter.ArtifactMetaDataParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetArtifactMetaDataCMD extends RunCmdStub<ArtifactIDParameter, ArtifactMetaDataParameter> {
    @NonNls
    public final static String COMMAND = "getArtifactMetaData";

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    @Override
    public void response(@NotNull ArtifactMetaDataParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
        //JOptionPane.showMessageDialog(null, "getArtifactMetadataCMD received metadata " + params.getArtifactID() + "\n" + params.metaData);

        @Nullable Project project = CommandsManager.getProjectForCommand(this);
        if (project == null) {
            System.out.println("Command not registered! " + getCommandIdentifier());
            return;
        }
        @Nullable LinkAnchor knownAnchor = SketchlinkManager.getInstance(project).getKnownAnchor(params.getArtifactID());
        @Nullable SketchMetadata convertedMetaData = params.getConvertedMetaData();
        if (convertedMetaData == null) {
            return;
        }
        if (knownAnchor instanceof LocalLinkAnchor) {
            return;
        }
        if (knownAnchor instanceof RemoteLinkAnchorS && convertedMetaData instanceof SketchMetadataTypeS) {
            ((RemoteLinkAnchorS) knownAnchor).setMetadata((SketchMetadataTypeS) convertedMetaData);
        } else {
            if (knownAnchor instanceof RemoteLinkAnchorM && convertedMetaData instanceof SketchMetadataTypeM) {
                ((RemoteLinkAnchorM) knownAnchor).setMetadata((SketchMetadataTypeM) convertedMetaData);
            } else {
                if (knownAnchor instanceof RemoteLinkAnchorC && convertedMetaData instanceof SketchMetadataTypeC) {
                    ((RemoteLinkAnchorC) knownAnchor).setMetadata((SketchMetadataTypeC) convertedMetaData);
                }
            }
        }

    }

    @NotNull
    @NonNls
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    private static class requestWrap implements PayloadWithData<ArtifactIDParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ArtifactIDParameter data;

        @Override
        public ArtifactIDParameter getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<ArtifactMetaDataParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ArtifactMetaDataParameter data;

        public ArtifactMetaDataParameter getData() {
            return this.data;
        }
    }


}
