/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.project.Project;
import com.intellij.psi.JavaPsiFacade;
import com.intellij.psi.PsiClass;
import com.intellij.psi.javadoc.PsiDocTag;
import com.intellij.psi.search.GlobalSearchScope;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.actions.quicktesting.LinkedArtifactsInFileAction;
import plg.sketchlinks.SketchlinkManager;
import plg.tools.slGuiTools;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.builder.GetContainedAnchorsResponseBuilder;
import plg.websocket.commands.parameter.ContainedAnchorsRequestParameter;
import plg.websocket.commands.parameter.ContainedAnchorsResponseParameter;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetContainedAnchorsCMD extends RunCmdStub<ContainedAnchorsRequestParameter, ContainedAnchorsResponseParameter> {
    @NonNls
    public static final String COMMAND = "getContainedAnchors";

    @Override
    public void request(@NotNull final ContainedAnchorsRequestParameter params, @NotNull final UUID receivedUUID, @NotNull final Channel inboundChannel) {

//        JOptionPane.showMessageDialog(null, "GetContainedAnchorsCMD was asked to check for all anchors in " + params.path);

        if (params.path == null || params.path.trim().isEmpty() || !params.path.toLowerCase().endsWith(".java")) { //NON-NLS
            return;
        }
        final String location = params.path.substring(0, params.path.length() - 5).replaceAll("/", ".");
        @Nullable final Project project = CommandsManager.getProjectForCommand(this);
        if (project == null)
            return;
        ApplicationManager.getApplication().runReadAction(new Runnable() {
            @Override
            public void run() {
                @Nullable PsiClass psiClass = JavaPsiFacade.getInstance(project).findClass(location, GlobalSearchScope.allScope(project));
                if (psiClass == null) {
                    slGuiTools.ShowNotificationAtBottom("getContainedAnchors error: File not found <br> " + params.path, project);
//                    JOptionPane.showMessageDialog(null, "getContainedAnchors: File not found");
                    return;
                }
                @NotNull Collection<PsiDocTag> docTags = LinkedArtifactsInFileAction.getSketchlinkTagsInFile(psiClass.getContainingFile());
                @NotNull GetContainedAnchorsResponseBuilder builder = new GetContainedAnchorsResponseBuilder(params.path);
                @NotNull Collection<String> result = new ArrayList<>();
                for (@NotNull PsiDocTag docTag : docTags) {
                    @Nullable String anchorID = SketchlinkManager.TOOLS.getSketchlinkTagID(docTag);
                    if (anchorID != null) {
                        result.add(anchorID);
                    }
                }
                builder.addAll(result);
                builder.send(GetContainedAnchorsCMD.this, inboundChannel, receivedUUID);
            }
        });


    }

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }


    private static class requestWrap implements PayloadWithData<ContainedAnchorsRequestParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ContainedAnchorsRequestParameter data;

        @Override
        public ContainedAnchorsRequestParameter getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<ContainedAnchorsResponseParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ContainedAnchorsResponseParameter data;

        public ContainedAnchorsResponseParameter getData() {
            return this.data;
        }
    }
}
