/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.SketchlinkManager;
import plg.tools.slGuiTools;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.ArtifactIDParameter;
import plg.websocket.commands.parameter.LinkedArtifactsResponseParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetLinkedArtifactsCMD extends RunCmdStub<ArtifactIDParameter, LinkedArtifactsResponseParameter> {
    @NonNls
    public static final String COMMAND = "getLinkedArtifacts";

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    @Override
    public void response(@NotNull LinkedArtifactsResponseParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
//        JOptionPane.showMessageDialog(null, "Received " + COMMAND + " für " + params.getArtifactID());
        @Nullable Project project = CommandsManager.getProjectForCommand(this);


        if (project == null) {
//            JOptionPane.showMessageDialog(null, "A not registered Command received data " + COMMAND + " " + params.getArtifactID());
            return;
        }

        @Nullable LinkAnchor sideB = SketchlinkManager.getInstance(project).getKnownAnchor(params.getArtifactID());
        if (sideB == null) {
            slGuiTools.ShowNotificationAtBottom("Received an unknown Anchor-ID as main link " + COMMAND + " " + params.getArtifactID(), project);
//            JOptionPane.showMessageDialog(null, "Received an unknown Anchor-ID as main link " + COMMAND + " " + params.getArtifactID());
            return;
        }

        //System.out.println("LinkedArtifact: " + (params.linkedArtifacts != null ? params.linkedArtifacts.length : "null"));
        /*
         * Add the new ArtifactID as remote anchor, add a link between this and the other artifactID and then ask the server for metadata of the new id
         */
        for (@NotNull LinkedArtifactsResponseParameter.ArtifactLink link : params.linkedArtifacts) {
            //System.out.println("\t\tGetLinkedArtifacts: " + link.getArtifactID() + " " + link.deprecated);

            @NotNull LinkAnchor sideA = SketchlinkManager.getInstance(project).addRemote(link.getArtifactID(), link.parentID, project);
            SketchlinkManager.getInstance(project).addLink(sideA, sideB, link.deprecated, false);
//            if (!(sideA instanceof RemoteLinkAnchor && ((RemoteLinkAnchor) sideA).getMetadata() != null)) {
//                new GetArtifactMetaDataRequestBuilder(sideA).send(project, inboundChannel);
//            }

        }


    }

    private static class requestWrap implements PayloadWithData<ArtifactIDParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ArtifactIDParameter data;

        @Override
        public ArtifactIDParameter getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<LinkedArtifactsResponseParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public LinkedArtifactsResponseParameter data;

        public LinkedArtifactsResponseParameter getData() {
            return this.data;
        }
    }
}
