/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchlinkManager;
import plg.slCurrentProjectSettings;
import plg.tools.slGuiTools;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.builder.LoggingRequestBuilder;
import plg.websocket.commands.parameter.RegisterRequestParameter;
import plg.websocket.commands.parameter.RegisterResponseParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class RegisterCMD extends RunCmdStub<RegisterRequestParameter, RegisterResponseParameter> {
    @NonNls
    public final static String COMMAND = "register";

    @Override
    public void response(@NotNull final RegisterResponseParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
        @Nullable Project project = CommandsManager.getProjectForCommand(this);
        if (project == null) {
            return;
        }
        slCurrentProjectSettings.getInstance(project).setClientName(params.displayName);
        String notificationText = "Registration to Sketchlink Server successful.<p/>Displayname: " + params.displayName + "<p/>ID: " + params.identifier;
        slGuiTools.ShowNotificationAtBottom(notificationText, project);
        SketchlinkManager.getInstance(project).lookupAnchors(true, project, inboundChannel);
        new LoggingRequestBuilder("Websocket registered;" + project.getName()).send(project);
    }


    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }


    private static class requestWrap implements PayloadWithData<RegisterRequestParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public RegisterRequestParameter data;

        @Nullable
        @Override
        public RegisterRequestParameter getData() {
            return null;
        }
    }

    private static class responseWrap implements PayloadWithData<RegisterResponseParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public RegisterResponseParameter data;

        public RegisterResponseParameter getData() {
            return this.data;
        }
    }
}
