/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.project.DumbService;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.MessageType;
import com.intellij.openapi.ui.popup.Balloon;
import com.intellij.openapi.ui.popup.JBPopupFactory;
import com.intellij.openapi.wm.StatusBar;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.ui.awt.RelativePoint;
import com.intellij.util.ui.UIUtil;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.slApplicationState;
import plg.slCurrentProjectSettings;
import plg.tools.slSearch;
import plg.ui.preview.PreviewManager;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.EmptyPayload;
import plg.websocket.commands.parameter.ShowRequestParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class ShowCMD extends RunCmdStub<ShowRequestParameter, EmptyPayload> {
    @NonNls
    public static final String COMMAND = "show";

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    @Override
    public void request(@NotNull final ShowRequestParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
        //JOptionPane.showMessageDialog(null, "Show received request to go to " + params.getArtifactID());
        @Nullable final Project project = CommandsManager.getProjectForCommand(this);

        if (project == null) {
            return;
        }
        @Nullable final StatusBar statusBar2 = WindowManager.getInstance()
                .getStatusBar(project);
        if (statusBar2 != null) {
            UIUtil.invokeLaterIfNeeded(new Runnable() {
                @Override
                public void run() {
                    JBPopupFactory.getInstance()
                            .createHtmlTextBalloonBuilder("Received show request for<p/>" + params.getArtifactID(), MessageType.INFO, null)
                            .setFadeoutTime(1500)
                            .createBalloon()
                            .show(RelativePoint.getCenterOf(statusBar2.getComponent()),
                                    Balloon.Position.atRight);

                }
            });
        }
        //on empty id, show all local code anchors to user
        if (params.getArtifactID() == null || params.getArtifactID().isEmpty()) {
            DumbService.getInstance(project).smartInvokeLater(new Runnable() {
                @Override
                public void run() {
                    new slSearch(project).runSearchForIDs(null, true, true);
                }
            });
            return;
        }

        if (params.getAnchorType() == LinkAnchor.AnchorTypes.Code_Anchor) {
            if (slCurrentProjectSettings.getInstance(project).isAllowRemoteNavigation()) {
                DumbService.getInstance(project).smartInvokeLater(new Runnable() {
                    @Override
                    public void run() {
                        new slSearch(project).runSearchForIDs(new String[]{params.getArtifactID()}, true, true);
                    }
                });
            } else {
                //insert tooltip for blocked search
                @Nullable final StatusBar statusBar = WindowManager.getInstance()
                        .getStatusBar(project);
                if (statusBar == null) {
                    return;
                }
                UIUtil.invokeLaterIfNeeded(new Runnable() {
                    @Override
                    public void run() {
                        JBPopupFactory.getInstance()
                                .createHtmlTextBalloonBuilder(
                                        slApplicationState.RESOURCE_BUNDLE.getString("a.remote.request.for.local.navigation.has.been.blocked.p.because.this.feature.has.been.disabled.in.the.sketchlink.configuration.be.the.user"), MessageType.INFO, null)
                                .setFadeoutTime(3500)
                                .createBalloon()
                                .show(RelativePoint.getCenterOf(statusBar.getComponent()),
                                        Balloon.Position.atRight);
                    }
                });
                // myTooltip.show(new RelativePoint(e.getComponent(),point));
            }
        } else
            PreviewManager.openAnchorInLocalTab(params.getArtifactID(), project);

    }

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    private static class requestWrap implements PayloadWithData<ShowRequestParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ShowRequestParameter data;

        @Override
        public ShowRequestParameter getData() {
            return data;
        }

    }

    private static class responseWrap implements PayloadWithData<EmptyPayload> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public EmptyPayload data;

        public EmptyPayload getData() {
            return this.data;
        }
    }
}
