/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;

import com.google.gson.reflect.TypeToken;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.ArtifactIDParameter;
import plg.websocket.commands.parameter.ArtifactMetaDataParameter;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class UpdateArtifactMetaDataCMD extends RunCmdStub<ArtifactMetaDataParameter, ArtifactIDParameter> {
    @NonNls
    public static final String COMMAND = "updateArtifactMetaData";

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    @Override
    public void response(@NotNull ArtifactIDParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
        //JOptionPane.showMessageDialog(null, "UpdateArtifactMetadata.response received " + params.getArtifactID());
        //only id as "check"?

    }

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    private static class requestWrap implements PayloadWithData<ArtifactMetaDataParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ArtifactMetaDataParameter data;

        @Override
        public ArtifactMetaDataParameter getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<ArtifactIDParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public ArtifactIDParameter data;

        public ArtifactIDParameter getData() {
            return this.data;
        }
    }
}
