/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.google.gson.GsonBuilder;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.sketchlinks.LocalLinkAnchor;
import plg.sketchlinks.RemoteLinkAnchor;
import plg.sketchlinks.SketchMetadata;
import plg.websocket.RunCmd;
import plg.websocket.commands.CreateArtifactMetaDataCMD;
import plg.websocket.commands.UpdateArtifactMetaDataCMD;
import plg.websocket.commands.parameter.ArtifactMetaDataParameter;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashSet;

/**
 * Created by Peter on 04.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class ChangeArtifactMetaDataRequestBuilder implements slApiRequestBuilder {
    @NotNull
    private final ArtifactMetaDataParameter parameter;
    private final LinkAnchor anchor;
    private boolean isUpdate = false;


    public ChangeArtifactMetaDataRequestBuilder(@NotNull LinkAnchor anchor) {
        this.anchor = anchor;
        parameter = new ArtifactMetaDataParameter();
        parameter.setArtifactID(anchor.getAnchorId());
        if (anchor instanceof LocalLinkAnchor) {
            parameter.setMetaData(((LocalLinkAnchor) anchor).getCurrentMetaData());

        } else {
            if (anchor instanceof RemoteLinkAnchor) {
                @Nullable SketchMetadata metadata = ((RemoteLinkAnchor) anchor).getMetadata();
                parameter.setMetaData(metadata != null ? metadata : null);
            }
        }
    }

    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(parameter);
    }

    public void queue(@NotNull Project project, @Nullable Channel channel, boolean isUpdate) {
        this.isUpdate = isUpdate;
        slWebsocketManager.getInstance(project).getSendQueue().sendRequest(this, channel);
    }

    private void sendAsUpdate(@NotNull Project project, @Nullable Channel channel) {
        if (anchor instanceof LocalLinkAnchor && !((LocalLinkAnchor) anchor).isValid()) {
//            System.out.println("ChangeArtifactMetaDataRequestBuilder: InvalidUpdate, did not send - "+anchor.getDescription()+" "+anchor.getAnchorId());
            return;
        }
//        System.out.println("ChangeArtifactMetaDataRequestBuilder: Sendupdate: "+anchor.getDescription()+" "+anchor.getAnchorId());
        @Nullable RunCmd cmd = slWebsocketManager.getInstance(project).getCommandsManager().getCommand(UpdateArtifactMetaDataCMD.COMMAND);
        if (cmd instanceof UpdateArtifactMetaDataCMD) {
            cmd.sendRequest(parameter, channel, null, false);
        }
    }


    private void sendAsCreate(@NotNull Project project, @Nullable Channel channel) {
        if (anchor instanceof LocalLinkAnchor && !((LocalLinkAnchor) anchor).isValid()) {
//            System.out.println("ChangeArtifactMetaDataRequestBuilder: Invalid, did not send - "+anchor.getDescription()+" "+anchor.getAnchorId());
            return;
        }
//        System.out.println("ChangeArtifactMetaDataRequestBuilder: Sendcreate: "+anchor.getDescription()+" "+anchor.getAnchorId());
        @Nullable RunCmd cmd = slWebsocketManager.getInstance(project).getCommandsManager().getCommand(CreateArtifactMetaDataCMD.COMMAND);
        if (cmd != null) {
            cmd.sendRequest(parameter, channel, null, false);
        }
    }


    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {
        Collection<String> result = new HashSet<>(1);
        result.add(parameter.getArtifactID());

        return result;
    }

    @Override
    public void send(@NotNull Project project, @Nullable Channel channel) {

        if (isUpdate)
            sendAsUpdate(project, channel);
        else
            sendAsCreate(project, channel);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ChangeArtifactMetaDataRequestBuilder that = (ChangeArtifactMetaDataRequestBuilder) o;

        if (isUpdate != that.isUpdate) return false;
        if (!anchor.equals(that.anchor)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return anchor.hashCode();
    }
}
