/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.google.gson.GsonBuilder;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.websocket.RunCmd;
import plg.websocket.commands.GetContainedAnchorsCMD;
import plg.websocket.commands.parameter.ContainedAnchorsResponseParameter;
import plg.websocket.slWebsocketManager;

import java.util.*;

/**
 * Created by Peter on 05.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetContainedAnchorsResponseBuilder implements slApiResponseBuilder {
    ContainedAnchorsResponseParameter parameter;
    @NotNull
    List<String> links = new ArrayList<>();

    public GetContainedAnchorsResponseBuilder(@NotNull String path) {
        parameter = new ContainedAnchorsResponseParameter();
        parameter.path = path;
    }

    public void add(@NotNull String anchor) {
        links.add(anchor);
    }

    public void addAll(@NotNull Collection<String> anchors) {
        links.addAll(anchors);
    }

    @Override
    public String toString() {
        parameter.containedAnchors = links.toArray(new String[links.size()]);
        return new GsonBuilder().serializeNulls().create().toJson(parameter);
    }

    public void send(@NotNull Project project, @Nullable Channel channel, @NotNull UUID uuid) {
        try {
            @NotNull RunCmd cmd = slWebsocketManager.getInstance(project).getCommandsManager().getCommandOrInstantiate(GetContainedAnchorsCMD.class);
            parameter.containedAnchors = links.toArray(new String[links.size()]);
            cmd.sendResponse(parameter, channel, uuid);
        } catch (@NotNull IllegalAccessException | InstantiationException e) {
            e.printStackTrace();
        }
    }

    public void send(@NotNull GetContainedAnchorsCMD cmd, @NotNull Channel channel, @NotNull UUID uuid) {
        parameter.containedAnchors = links.toArray(new String[links.size()]);
        cmd.sendResponse(parameter, channel, uuid);
    }

    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {
        Collection<String> result = new HashSet<>(links.size());
        result.addAll(links);
        return result;
    }


}
