/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.google.gson.GsonBuilder;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.LinkAnchor;
import plg.slProjectComp;
import plg.websocket.commands.GetLinkedArtifactsCMD;
import plg.websocket.commands.parameter.ArtifactIDParameter;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashSet;

/**
 * Created by Peter on 05.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetLinkedArtifactsRequestBuilder implements slApiRequestBuilder {
    private final Logger LOG = Logger.getInstance(GetLinkedArtifactsRequestBuilder.class);
    private ArtifactIDParameter parameter;

    public GetLinkedArtifactsRequestBuilder(@NotNull LinkAnchor anchor) {
        parameter = new ArtifactIDParameter();
        parameter.setArtifactID(anchor.getAnchorId());
    }

    @Override
    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(parameter);
    }

    public void send(@NotNull Project project, @Nullable Channel channel) {
        if (!slProjectComp.getInstance(project).isConnectedToServer()) {
            return;
        }
        try {
            slWebsocketManager.getInstance(project).getCommandsManager().getCommandOrInstantiate(GetLinkedArtifactsCMD.class).sendRequest(parameter, channel, null, false);
        } catch (@NotNull IllegalAccessException | InstantiationException e) {
            LOG.error(e);
        }
    }

    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {
        Collection<String> result = new HashSet<>(1);
        result.add(parameter.getArtifactID());
        return result;
    }
}
