/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.google.gson.GsonBuilder;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.websocket.RunCmd;
import plg.websocket.commands.GetSketchesCMD;
import plg.websocket.commands.parameter.GetSketchesRequestParameter;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashSet;

/**
 * Created by Peter on 13.10.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class GetSketchesRequestBuilder implements slApiRequestBuilder {
    private GetSketchesRequestParameter parameter;


    public GetSketchesRequestBuilder(boolean showOnlyUnlinked) {
        parameter = new GetSketchesRequestParameter(showOnlyUnlinked);
    }

    public GetSketchesRequestBuilder(@NotNull String filter) {
        parameter = new GetSketchesRequestParameter(filter);
    }


    @Override
    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(parameter);
    }

    public void send(@NotNull Project project, @Nullable Channel channel) {
        @Nullable RunCmd getArtifactMetaData = slWebsocketManager.getInstance(project).getCommandsManager().getCommand(GetSketchesCMD.COMMAND);
        //System.out.println("Trying to request Metadata for " + parameter.getArtifactID() + " " + (getArtifactMetaData != null));
        if (getArtifactMetaData instanceof GetSketchesCMD) {
            //run toString to build data
            getArtifactMetaData.sendRequest(parameter, channel, null, false);
        }

    }

    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {

        return new HashSet<>(1);
    }

}
