/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.slCurrentProjectSettings;
import plg.websocket.CommandTopParameter;
import plg.websocket.Payload;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashSet;

/**
 * Created by Peter on 17.05.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class LoggingRequestBuilder implements slApiRequestBuilder {
    @NonNls
    private static final String COMMAND = "log";
    private final LoggingParam param = new LoggingParam();


    public LoggingRequestBuilder(@NotNull String message) {
        param.message = message;
    }

    public void send(@NotNull Project project) {
        if (!slCurrentProjectSettings.getInstance(project).isServerLoggingEnabled()) {
            return;
        }
        @NotNull CommandTopParameter packet = new CommandTopParameter();
        packet.data = param;
        packet.message = COMMAND;
        packet.type = "request";
        slWebsocketManager.getInstance(project).getChannelManager().broadcastText(packet.toString());
    }

    @Override
    public void send(@NotNull Project project, @Nullable Channel channel) {
        send(project);
    }

    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {
        Collection<String> result = new HashSet<>(1);

        return result;
    }

    private class LoggingParam implements Payload {
        String message;
    }

}
