/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.google.gson.GsonBuilder;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.websocket.RunCmd;
import plg.websocket.commands.RegisterCMD;
import plg.websocket.commands.parameter.RegisterRequestParameter;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class RegisterRequestBuilder implements slApiRequestBuilder {

    @NotNull
    Map<String, Object> buildMap = new HashMap<>(3);
    @NotNull
    private RegisterRequestParameter parameter = new RegisterRequestParameter();

    public RegisterRequestBuilder(@NotNull String preferredName) {
        buildMap.put("preferredDisplayName", preferredName); //NON-NLS
        buildMap.put("protocolVersion", 1.0); //NON-NLS
        buildMap.put("clientType", "ide"); //NON-NLS NON-NLS
        parameter.clientType = "ide";
        parameter.protocolVersion = 1.0f;
        parameter.preferredDisplayName = preferredName;

    }

    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(buildMap);
    }

    public void send(@NotNull Project project, @NotNull Channel channel) {
        @Nullable RunCmd command = slWebsocketManager.getInstance(project).getCommandsManager().getCommand(RegisterCMD.COMMAND);
        if (command != null) {

            command.sendRequest(parameter, channel, null, false);
        }
    }

    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {

        return new HashSet<>(1);
    }
}
