/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.builder;

import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.ui.preview.AvailableViews;
import plg.websocket.RunCmd;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.ShowCMD;
import plg.websocket.commands.parameter.ShowRequestParameter;
import plg.websocket.slWebsocketManager;

import java.util.Collection;
import java.util.HashSet;

/**
 * Created by Peter on 07.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class ShowRequestBuilder implements slApiRequestBuilder {
    private ShowRequestParameter parameter;


    public ShowRequestBuilder(@NotNull String artifactID) {
        parameter = new ShowRequestParameter();
        parameter.setArtifactID(artifactID);
    }

    public void addView(@NotNull AvailableViews view) {
        parameter.clients.add(view.getIdentifier());
    }


    public void send(@NotNull Project project, @Nullable Channel channel) {
        @Nullable RunCmd show = slWebsocketManager.getInstance(project).getCommandsManager().getCommand(ShowCMD.COMMAND);
        if (show instanceof RunCmdStub) {
            show.sendRequest(parameter, channel, null, true);
        }
    }


    @NotNull
    @Override
    public Collection<String> getContainedAnchorIDs() {
        Collection<String> result = new HashSet<>(1);
        result.add(parameter.getArtifactID());
        return result;
    }
}
