/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands;


import com.google.gson.reflect.TypeToken;
import com.intellij.openapi.project.Project;
import io.netty.channel.Channel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.slApplicationState;
import plg.ui.preview.AvailableViewImpl;
import plg.ui.preview.PreviewManager;
import plg.websocket.CommandsManager;
import plg.websocket.PayloadWithData;
import plg.websocket.RunCmdStub;
import plg.websocket.commands.parameter.AvailableViewsResponseParameter;
import plg.websocket.commands.parameter.EmptyPayload;

import java.lang.reflect.Type;
import java.util.UUID;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class getAvailableClientsCMD extends RunCmdStub<EmptyPayload, AvailableViewsResponseParameter> {
    @NonNls
    public static final String COMMAND = "getAvailableClients";

    @Override
    public Type getRequestWrapClass() {
        return new TypeToken<requestWrap>() {
        }.getType();
    }

    @Override
    public Type getResponseWrapClass() {
        return new TypeToken<responseWrap>() {
        }.getType();
    }

    @Override
    public void response(@NotNull AvailableViewsResponseParameter params, @NotNull UUID receivedUUID, @NotNull Channel inboundChannel) {
        //JOptionPane.showMessageDialog(null, "Received clients\n" + params);
        @Nullable Project project = CommandsManager.getProjectForCommand(this);
        if (project == null) {
            return;
        }
        @NotNull PreviewManager previewManager = PreviewManager.getInstance(project);

        previewManager.clearAvailablePreviews();

        for (@NotNull AvailableViewImpl availableView : params.clients) {

            if ("127.0.0.1".equals(availableView.getIpAddress())) {
                String ip = inboundChannel.remoteAddress().toString();

                ip = ip.indexOf(':') >= 0 ? ip.substring(1, ip.indexOf(':')) : ip;
                availableView.setIpAddress(ip);
            } else {
                if (inboundChannel.localAddress().toString().substring(1).startsWith(availableView.getIpAddress())) {
                    availableView.setIpAddress(slApplicationState.RESOURCE_BUNDLE.getString("existing.local.webview"));
                }
            }
            previewManager.addViewer(availableView);
        }
    }

    @NotNull
    @Override
    public String getCommandIdentifier() {
        return COMMAND;
    }

    private static class requestWrap implements PayloadWithData<EmptyPayload> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public EmptyPayload data;

        @Override
        public EmptyPayload getData() {
            return data;
        }
    }

    private static class responseWrap implements PayloadWithData<AvailableViewsResponseParameter> {
        @Nullable
        public UUID id = null;
        @Nullable
        public String message = null;
        @Nullable
        public String type = null;
        public AvailableViewsResponseParameter data;

        public AvailableViewsResponseParameter getData() {
            return this.data;
        }
    }
}
