/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.commands.parameter;

import com.google.gson.GsonBuilder;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import plg.sketchlinks.SketchMetadata;
import plg.sketchlinks.SketchMetadataTypeC;
import plg.sketchlinks.SketchMetadataTypeM;
import plg.sketchlinks.SketchMetadataTypeS;
import plg.websocket.Payload;

/**
 * Created by Peter on 03.04.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class ArtifactMetaDataParameter extends ArtifactIDParameter implements Payload {
    private SketchMetadataWrap metaData;

    public boolean isAvailable() {
        return metaData != null;
    }

    public SketchMetadataWrap getMetaData() {
        return metaData;
    }

    public void setMetaData(SketchMetadata metaData) {
        this.metaData = new SketchMetadataWrap();
        if (metaData instanceof SketchMetadataTypeM) {
            this.metaData.set((SketchMetadataTypeM) metaData);

        } else {
            if (metaData instanceof SketchMetadataTypeS) {
                this.metaData.set((SketchMetadataTypeS) metaData);
            } else {
                if (metaData instanceof SketchMetadataTypeC) {
                    this.metaData.set((SketchMetadataTypeC) metaData);
                }
            }
        }

    }

    public boolean isKnownMetaType() {
        return super.isValid();
    }

    @Nullable
    public SketchMetadata getConvertedMetaData() {
        if (metaData == null) {
            return null;
        }
        if (getArtifactID().startsWith("c")) { //NON-NLS NON-NLS
            return metaData.getAsTypeC();
        } else {
            if (getArtifactID().startsWith("s")) { //NON-NLS
                return metaData.getAsTypeS();
            } else {
                if (getArtifactID().startsWith("m")) { //NON-NLS
                    return metaData.getAsTypeM();
                }
            }
        }
        return null;
//        if (!isKnownMetaType()) {
//            return null;
//        }
//        if (super.getAnchorType() == LinkAnchor.AnchorTypes.SVG_Rectangle_Anchor) {
//            return new GsonBuilder().serializeNulls().create().fromJson(metaData, SketchMetadataTypeM.class);
//        }
//        if (super.getAnchorType() == LinkAnchor.AnchorTypes.SVG_Anchor) {
//            return new GsonBuilder().serializeNulls().create().fromJson(metaData, SketchMetadataTypeS.class);
//        }
//        if (super.getAnchorType() == LinkAnchor.AnchorTypes.Code_Anchor) {
//            return new GsonBuilder().serializeNulls().create().fromJson(metaData, SketchMetadataTypeC.class);
//        }
//        return null;
    }

    @Override
    public String toString() {
        return new GsonBuilder().serializeNulls().create().toJson(this);
    }

    private class SketchMetadataWrap {
        private String timestamp;
        private String authors;
        private String annotation;
        private String title;
        @Nullable
        private String parentID;
        private String path;
        private String targetType;
        @Nullable
        private String targetName;
        private String projectName;

        @NotNull
        SketchMetadataTypeC getAsTypeC() {
            @NotNull SketchMetadataTypeC result = new SketchMetadataTypeC();
            result.setParentID(parentID != null ? parentID : "");
            result.setPath(path != null ? path : "");
            result.setTargetName(targetName != null ? targetName : "");
            result.setTargetType(targetType != null ? targetType : "");
            result.setProjectName(projectName != null ? projectName : "");
            return result;
        }

        @NotNull
        SketchMetadataTypeM getAsTypeM() {
            @NotNull SketchMetadataTypeM result = new SketchMetadataTypeM();
            result.setParentID(parentID != null ? parentID : "");
            result.setAnnotation(annotation != null ? annotation : "");
            return result;
        }

        @NotNull
        SketchMetadataTypeS getAsTypeS() {
            @NotNull SketchMetadataTypeS result = new SketchMetadataTypeS();
            result.setAnnotation(annotation != null ? annotation : "");
            result.setTitle(title != null ? title : "");
            result.setAuthors(authors != null ? authors : "");
            result.setTimestamp(timestamp != null ? timestamp : "");
            return result;
        }

        void set(@NotNull SketchMetadataTypeM data) {
            parentID = data.getParentID();
            annotation = data.getAnnotation();
        }

        void set(@NotNull SketchMetadataTypeC data) {
            parentID = data.getParentID();
            path = data.getPath();
            targetName = data.getTargetName();
            projectName = data.getProjectName();
            targetType = data.getTargetType();
        }

        void set(@NotNull SketchMetadataTypeS data) {
            timestamp = data.getTimestamp();
            authors = data.getAuthors();
            title = data.getTitle();
            annotation = data.getAnnotation();
        }
    }
}
