/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket.extensionPoints;

import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.extensions.AbstractExtensionPointBean;
import com.intellij.openapi.extensions.ExtensionPointName;
import com.intellij.util.xmlb.annotations.Attribute;
import org.jetbrains.annotations.Nullable;
import plg.websocket.RunCmd;

/**
 * Created by Peter on 06.07.2014.
 * Part of Sketchlink Plugin for IntelliJ
 */
public class WebsocketCommandBean extends AbstractExtensionPointBean {
    public static final ExtensionPointName<WebsocketCommandBean> EP_NAME = ExtensionPointName.create("com.sketchlink.idea.websocket.message");

    @Attribute("command")
    public String command;

    @Attribute("implementationClass")
    public String implementationClass;

    private RunCmd myImpl;
    private Class<? extends RunCmd> myClassType;

    public String getCommand() {
        return command;
    }

    public String getImplementationClass() {
        return implementationClass;
    }


    /**
     * Call this method to instantiate the Class
     */
    @Nullable
    public RunCmd getCommandHandler() {
        if (myImpl == null) {
            try {
                myImpl = instantiate(getClassType(), ApplicationManager.getApplication().getPicoContainer());
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        return myImpl;
    }

    @Nullable
    public Class<? extends RunCmd> getClassType() {
        if (myClassType == null) {
            try {
                myClassType = findClass(implementationClass);
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }
        return myClassType;
    }


}
