/*
 * Copyright (c) 2014. by Peter Schmitz (ptsmail01@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package plg.websocket;

import com.intellij.openapi.components.ServiceManager;
import com.intellij.openapi.project.Project;
import org.jetbrains.annotations.NotNull;
import plg.SLConfigForm;
import plg.sketchlinks.SketchlinkManager;
import plg.websocket.commands.builder.LoggingRequestBuilder;

import java.net.URISyntaxException;


/**
 * Created by Peter on 22.02.14.
 */
public abstract class slWebsocketManager {
    @NotNull
    private final Project project;
    @NotNull
    private final SendQueue queue = new SendQueue(this);

    public slWebsocketManager(@NotNull Project project) {
        this.project = project;

    }

    @NotNull
    public static slWebsocketManager getInstance(@NotNull Project project) {
        //return slProjectComp.getInstance(project).getWebsocketManager();
        return ServiceManager.getService(project, slWebsocketManager.class);
    }

    public static void sendBroadcast(@NotNull String textData, @NotNull Project project) {
        slWebsocketManager.getInstance(project).getChannelManager().broadcastText(textData);
    }

    @NotNull
    public SendQueue getSendQueue() {
        return this.queue;
    }


    public boolean isUpdateServerOnCreate() {
        return this.queue.isSendDirect();
    }

    /**
     * If true, some websocket messages are queued and not send directly to the server, i.e. create code anchor
     * If false, all websocket messages are send directly.
     */
    public void setUpdateServerOnCreate(boolean sendUpdates) {
        slWebsocketManager.getInstance(project).getSendQueue().setSendDirect(sendUpdates);
    }

    @NotNull
    public Project getProject() {
        return project;
    }

    /**
     * Checks for active Websockets
     *
     * @return true if at least one Websocket is available, false otherwise
     */
    public abstract boolean isConnected();

    @NotNull
    public abstract ChannelManager getChannelManager();

    @NotNull
    public abstract CommandsManager getCommandsManager();

    public void connect() {
        if (slWebsocketManager.getInstance(project).isConnected())
            return;
        try {
            slWebsocketManager.getInstance(project).getChannelManager().connectToDefault();
        } catch (URISyntaxException e1) {
            SLConfigForm.showInvalidServerMessage(e1.getMessage(), project);
        }
    }

    public void disconnect() {
        if (!slWebsocketManager.getInstance(project).isConnected())
            return;
        new LoggingRequestBuilder("Websocket disconnect;" + project.getName()).send(project);
        SketchlinkManager.getInstance(project).updateServerMetaDataOnLocalData();
        slWebsocketManager.getInstance(project).getChannelManager().closeAll(1000, "User invoked disconnection"); //NON-NLS
    }

    /**
     * Connects or Disconnects the Websocket. On Connect, automatically tries to update all locally found Sketchlink IDs.
     */
    public void toggleWebsocketConnection(boolean connect_not_toggle) {
        if (slWebsocketManager.getInstance(project).isConnected() && !connect_not_toggle) {
            connect();
        } else {
            disconnect();
        }
    }

}
